<?php
	/*
	* TifFP composant passerelel de TourInFrance vers GMapFP pour Joomla! 3.x
	* Version J4.4
	* Creation date: Février 2024
	* Author: Fabrice4821 - www.gmapfp.org
	* Author email: webmaster@gmapfp.org
	* License GNU/GPL
	*/

namespace Joomla\Component\Tiffps\Administrator\Controller;

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Application\CMSApplication;
use Joomla\CMS\Language\Text;
use Joomla\CMS\MVC\Controller\AdminController;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Router\Route;
use Joomla\Input\Input;
use Joomla\Utilities\ArrayHelper;

class TiffpsController extends AdminController
{
	protected $text_prefix = 'COM_TIFFPS';

	public function __construct($config = array(), MVCFactoryInterface $factory = null, $app = null, $input = null)
	{
		parent::__construct($config, $factory, $app, $input);

		$this->registerTask('sticky_unpublish', 'sticky_publish');
	}

	function view()
	{
		$this->input->set( 'view', 'tiffps' );
		$this->input->set( 'layout', 'default'  );
		$this->input->set('hidemainmenu', 0);

		parent::display();
	}

	public function getModel($name = 'Tiffp', $prefix = 'Administrator', $config = array('ignore_request' => true))
	{
		return parent::getModel($name, $prefix, $config);
	}

	public function publish2()
	{
		// Check for request forgeries.
		$this->checkToken();

		// Initialise variables.
		$user	= Factory::getUser();
		$ids	= $this->input->get('cid', array(), 'array');
		$values	= array('sticky_publish' => 1, 'sticky_unpublish' => 0);
		$task	= $this->getTask();
		$value	= ArrayHelper::getValue($values, $task, 0, 'int');

		if (empty($ids)) {
			$this->setMessage(JText::_('COM_TIFFPS_NO_TIFFPS_SELECTED'));
		} else {
			// Get the model.
			$model	= $this->getModel();

			// Change the state of the records.
			if (!$model->stick($ids, $value)) {
				$this->setMessage($model->getError());
			} else {
				if ($value == 1) {
					$ntext = 'COM_TIFFPS_N_TIFFPS_STUCK';
				} else {
					$ntext = 'COM_TIFFPS_N_TIFFPS_UNSTUCK';
				}
				$this->setMessage(JText::plural($ntext, count($ids)));
			}
		}

		$this->setRedirect('index.php?option=com_tiffps&view=tiffps');
	}

	public function extract()
	{
		// Check for request forgeries.
		$this->checkToken();

		// Initialise variables.
		$user	= Factory::getUser();
		$ids	= $this->input->get('cid', array(), 'array');

		if (empty($ids)) {
			$this->setMessage(Text::_('COM_TIFFPS_EXTRACT_XML_NOK'));
		} else {
			// Get the model.
			$model	= $this->getModel('Tiffpbase');

			if (!$model->getTifXML($ids)) {
				$this->setMessage($model->getError());
			} else {
				$ntext = 'COM_TIFFPS_EXTRACT_XML_OK';
				$this->setMessage(Text::plural($ntext, count($ids)));
			}
		}

		$this->setRedirect('index.php?option=com_tiffps&view=tiffps');
	}

	public function extract_file()
	{
		// Check for request forgeries.
		$this->checkToken('get');

		$model	= $this->getModel('Tiffpbase');
		$model->extract_file();
		
		$link = 'index.php?option=com_tiffps&view=tiffps';
		$this->setRedirect($link);
	}
	
	public function extract_image()
	{
		// Check for request forgeries.
		$this->checkToken('get');

		$model	= $this->getModel('Tiffpbase');
		$model->import_images();
		
		$link = 'index.php?option=com_tiffps&view=tiffps';
		$this->setRedirect($link);
	}
	
	public function purge()
	{
		// Check for request forgeries.
		// $this->checkToken();
	
		$model	= $this->getModel('tiffps');
		$model->purge();
		
		$this->setRedirect('index.php?option=com_tiffps');
	}

}
