<?php
	/*
	* TifFP composant passerelel de TourInFrance vers GMapFP pour Joomla! 4.x
	* Version J4.3
	* Creation date: Février 2024
	* Author: Fabrice4821 - www.gmapfp.org
	* Author email: support@gmapfp.org
	* License GNU/GPL
	*/

namespace Joomla\Component\Tiffps\Administrator\Helper;

defined('_JEXEC') or die;

use Joomla\CMS\Helper\ContentHelper;
use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\URI\URI;
use Joomla\CMS\Session\Session;

class TiffpsHelper extends ContentHelper
{
	public static function getGmapfpFields()
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true);
		$query = 'SHOW COLUMNS FROM '.($db->quoteName('#__gmapfp'));
		$db->setQuery($query);
		$fields = $db->loadObjectList('Field');
		
		return $fields;
	}

	public static function getContentFields()
	{
		$db    = Factory::getDbo();
		$query = $db->getQuery(true);
		$query = 'SHOW COLUMNS FROM '.($db->quoteName('#__content'));
		$db->setQuery($query);
		$fields = $db->loadObjectList('Field');
		
		return $fields;
	}

	public static function getGmapfpCustomFields()
	{
		$db    = Factory::getDbo();
		$query = $db
			->getQuery(true)
			->select('f.id, f.title as title, CONCAT("field_", f.id) as name, fg.title as fg_title')
			->from($db->quoteName('#__fields', 'f'))
			->join('LEFT', $db->quoteName('#__fields_groups', 'fg') . ' ON ' . $db->quoteName('f.group_id') . ' = ' . $db->quoteName('fg.id'))
			->where($db->quoteName('f.context') . " = " . $db->quote('com_gmapfp.item'))
			->where($db->quoteName('f.state') . " IN (0,1) ")
			->order('fg.title, f.title');
		$query->select('category_id')
			->join('LEFT', '#__fields_categories AS c ON c.field_id = f.id');
		$db->setQuery($query);
		$fields = $db->loadObjectList('name');
		
		return $fields;
	}
	
	public static function getGmapfpCustomFieldsContent()
	{
		$db    = Factory::getDbo();
		$query = $db
			->getQuery(true)
			->select('f.id, f.title as title, CONCAT("field_", f.id) as name, fg.title as fg_title')
			->from($db->quoteName('#__fields', 'f'))
			->join('LEFT', $db->quoteName('#__fields_groups', 'fg') . ' ON ' . $db->quoteName('f.group_id') . ' = ' . $db->quoteName('fg.id'))
			->where($db->quoteName('f.context') . " = " . $db->quote('com_content.item'))
			->where($db->quoteName('f.state') . " IN (0,1) ")
			->order('fg.title, f.title');
		$query->select('category_id')
			->join('LEFT', '#__fields_categories AS c ON c.field_id = f.id');
		$db->setQuery($query);
		$fields = $db->loadObjectList('name');
		
		return $fields;
	}
	
	public static function getXMLNames ($name, $type)
	{
		$db    = Factory::getDbo();
		$query = $db
			->getQuery(true)
			->select($db->quoteName('xml'))
			->from($db->quoteName('#__tiffp_xml', 'tif_xml'))
			->join('LEFT', '#__tiffp_syndications AS sy ON sy.id = tif_xml.syndication_id')
			->where($db->quoteName('name'). ' = '.$db->quote($name))
			->where($db->quoteName('syndication_format'). ' = '.$db->quote($type));
		$db->setQuery($query);
		$results = $db->loadResult();

		return $results;
	}
	
	public static function getTiffps_syndication_info ($nameXML)
	{
		$db    = Factory::getDbo();
		$query = $db
			->getQuery(true)
			->select($db->quoteName('syndication_type').', '.$db->quoteName('syndication_format').', '.$db->quoteName('tiffps_photos_path').', '.$db->quoteName('tiffps_fields_separator').', '.$db->quoteName('tiffps_values_separator'))
			->from($db->quoteName('#__tiffp_xml', 'a'))
			->join('LEFT', $db->quoteName('#__tiffp_syndications', 's') . ' ON ' . $db->quoteName('s.id') . ' = ' . $db->quoteName('a.syndication_id'))
			->where($db->quoteName('a.name'). ' = '.$db->quote($nameXML));
		$db->setQuery($query);
		$results = $db->loadAssoc();

		return $results;
	}
	
	public static function getTif_id ($idGMapFP = '')
	{
		$db    = Factory::getDbo();
		$query = $db
			->getQuery(true)
			->select(array('gmapfp_id', 'tif_id'))
			->from($db->quoteName('#__tiffp_gmapfp'));
		if ($idGMapFP) $query->where($db->quoteName('gmapfp_id'). ' = '.(int)$idGMapFP);
		$db->setQuery($query);
		if ($idGMapFP) 
			$results = $db->loadResult();
		else
			$results = $db->loadObjectList('gmapfp_id');

		return $results;
	}
	
	public static function getListNamesXML($verifs_codes = '.', $type = '')
	{
		$results = array();

		$db    = Factory::getDbo();
		$query = $db
			->getQuery(true)
			->select($db->quoteName('name'))
			->from($db->quoteName('#__tiffp_xml', 'a'))
			->innerJoin('#__tiffp_syndications AS s on s.id = a.syndication_id');
		
		if ($type) $query->where($db->quoteName('s.syndication_format').' = '.$db->quote($type));
			
		$db->setQuery($query);
		$xmls = $db->loadColumn();

		if ($verifs_codes and $verifs_codes != '.') {
			foreach($xmls as $xml) {
				preg_match('/'.$verifs_codes.'/', $xml, $matches, PREG_OFFSET_CAPTURE);
				if (!empty($matches)) {
					$results[] = $xml;
				}
			}
		} else
			$results = $xmls;
		
		return $results;
	}
		
	public static function getDeleteNamesXML($nameXML)
	{
		$db    = Factory::getDbo();
		$query = $db
			->getQuery(true)
			->delete($db->quoteName('#__tiffp_xml'))
			->where($db->quoteName('name'). ' = '.$db->quote($nameXML));
		$db->setQuery($query);
		$results = $db->execute();

		return $results;
	}
		
	public static function getProcessImage ( $image, $reste_images, $nbre_image) {
		$token = '&'.Session::getFormToken().'=1';
		$link = 'index.php?option=com_tiffps&task=tiffps.extract_image';

		$courent_file = $nbre_image - $reste_images;
		$thumbs_text = Text::_('COM_TIFFPS_EXTRAT_IMAGES');
		echo '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">' . "\n";
		echo '<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en-en" lang="en-en" dir="ltr" >'. "\n";
		echo '<head>'. "\n";
		echo '<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />'. "\n\n";
		echo '<title>'.$thumbs_text.'</title>'. "\n";
		echo '</head>'. "\n";
		echo '<body>'. "\n";
			
		//entête
		echo '<center><div style="width:70%;border:5px solid #0000CC; margin-top:30px;font-family: sans-serif, Arial;font-weight:normal;color:#666;font-size:14px;padding:10px">';
		echo '<span>'.$thumbs_text.'</span><br /><span style="font-size:200%;">'.Text::_( 'COM_TIFFPS_WAIT' ). '</span>';
		
		echo '<p>&nbsp;</p>';
		//extration fichiers
		echo '<p>' .Text::_('COM_TIFFPS_EXTRAT_IMAGE') 
		.' <span style="color:#0066cc;">'. $image . '</span>' 
		.' ... <b style="color:#009900">'.Text::_( 'OK' ).'</b></p>';
	
		// Creation bars de progresion fichiers
		$perfiles = 0; // display percents
		if ($nbre_image > 0) {
			$perfiles = round(($courent_file / $nbre_image)*100, 2);
		}
		$perCSS = ($perfiles * 400/100) - 400;
		$bgCSSfiles = 'background: url(\''.URI::base(true).'/../media/com_tiffps/images/process.jpg\') '.$perCSS.'px 0 repeat-y;';
		
		// BEGIN -----------------------------------------------------------------------
		echo '<p>' . Text::_('COM_TIFFPS_IMPORTATION_IMAGE'). ': <span style="color:#0066cc">'. $courent_file .'</span> '.Text::_('COM_TIFFPS_SUR'). ' <span style="color:#0066cc">'. $nbre_image .'</span> '.Text::_('COM_TIFFPS_FILES').'</p>';
		
		echo '<span style="font-size:10px;font-weight:bold">'.$perfiles.' &#37;</span><div style="width:400px;height:20px;font-size:20px;border-top:2px solid #666;border-left:2px solid #666;border-bottom:2px solid #ccc;border-right:2px solid #ccc;'.$bgCSSfiles.'"></div>';
		// END -------------------------------------------------------------------------

		echo '<p>&nbsp;</p>';
		echo '<p>TifFPs : composant de <a target="_blank" href="https://creation-web.pro/">GMapFP D&eacute;veloppement</a></p>';
		
		echo '<meta http-equiv="refresh" content="0;url='.$link.'&reste_image='.$reste_images.'&nbre_images='.$nbre_image.$token.'" />';
		echo '</div></center></body></html>';
		exit;
	}
	
	public static function getProcessPage ( $filename, $courent_file, $refresh_url, $files, $errorMsg = '', $tiffps_id = 0, $courent_id = 0, $nom_batch ='', $nbre_files='') {
		$token = '&'.Session::getFormToken().'=1';
		
		$thumbs_text = Text::_('COM_TIFFPS_EXTRAT_FILES');
		echo '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">' . "\n";
		echo '<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en-en" lang="en-en" dir="ltr" >'. "\n";
		echo '<head>'. "\n";
		echo '<meta http-equiv="Content-Type" content="text/html; charset=utf-8" />'. "\n\n";
		echo '<title>'.$thumbs_text.'</title>'. "\n";
		echo '</head>'. "\n";
		echo '<body>'. "\n";
			
		//entête
		echo '<center><div style="width:70%;border:5px solid #0000CC; margin-top:30px;font-family: sans-serif, Arial;font-weight:normal;color:#666;font-size:14px;padding:10px">';
		echo '<span>'.$thumbs_text.'</span><br /><span style="font-size:200%;">'.Text::_( 'COM_TIFFPS_WAIT' ). '</span>';
		
		echo '<p>&nbsp;</p>';
		//extraction batch
		echo '<p>' .Text::_('COM_TIFFPS_EXTRAT_BATCH') 
			.($courent_id+1).Text::_('COM_TIFFPS_SUR').count($tiffps_id).'&nbsp;:&nbsp; <span style="color:#0066cc;">'. htmlentities($nom_batch,ENT_NOQUOTES,'UTF-8') . '</span></p>';
		// Creation bars de progresion batch
		$perfiles = 0; // display percents
		if ($files > 0) {
			$perfiles = round((($courent_id+1) / count($tiffps_id))*100, 0);
		}
		$perCSS = ($perfiles * 400/100) - 400;
		$bgCSSfiles = 'background: url(\''.URI::base(true).'/../media/com_tiffps/images/process.jpg\') '.$perCSS.'px 0 repeat-y;';
		
		echo '<span style="font-size:10px;font-weight:bold">'.$perfiles.' &#37;</span><div style="width:400px;height:20px;font-size:20px;border-top:2px solid #666;border-left:2px solid #666;border-bottom:2px solid #ccc;border-right:2px solid #ccc;'.$bgCSSfiles.'"></div>';
		// END -------------------------------------------------------------------------
		echo '<p>&nbsp;</p>';
		//extration fichiers
		if ( $errorMsg == '' ) {
			echo '<p>' .Text::_('COM_TIFFPS_EXTRAT_FILE') 
			.' <span style="color:#0066cc;">'. $filename . '</span>' 
			.' ... <b style="color:#009900">'.Text::_( 'OK' ).'</b></p>';
		} else {
			echo '<p>' .Text::_('COM_TIFFPS_EXTRAT_FILE') 
			.' <span style="color:#0066cc;padding:0;margin:0"> '. htmlentities($filename,ENT_NOQUOTES,'UTF-8') . '</span>' 
			.' ... <b style="color:#fc0000">'.Text::_( 'Error' ).'</b></p>';
			
		}
	
		// Creation bars de progresion fichiers
		$perfiles = 0; // display percents
		if ($files > 0) {
			$perfiles = round(($courent_file / $files)*100, 2);
		}
		$perCSS = ($perfiles * 400/100) - 400;
		$bgCSSfiles = 'background: url(\''.URI::base(true).'/../media/com_tiffps/images/process.jpg\') '.$perCSS.'px 0 repeat-y;';
		
		// BEGIN -----------------------------------------------------------------------
		echo '<p>' . Text::_('COM_TIFFPS_IMPORTATION'). ': <span style="color:#0066cc">'. $courent_file .'</span> '.Text::_('COM_TIFFPS_SUR'). ' <span style="color:#0066cc">'. $files .'</span> '.Text::_('COM_TIFFPS_FILES').'</p>';
		
		echo '<span style="font-size:10px;font-weight:bold">'.$perfiles.' &#37;</span><div style="width:400px;height:20px;font-size:20px;border-top:2px solid #666;border-left:2px solid #666;border-bottom:2px solid #ccc;border-right:2px solid #ccc;'.$bgCSSfiles.'"></div>';
		// END -------------------------------------------------------------------------

		echo '<p>&nbsp;</p>';
		echo '<p>TifFPs : composant de <a target="_blank" href="https://creation-web.pro/">GMapFP D&eacute;veloppement</a></p>';
		
		echo '<meta http-equiv="refresh" content="0;url='.$refresh_url.'&courent_file='.$courent_file.'&tiffps_id[]='.implode('&tiffps_id[]=', $tiffps_id).'&courent_id='.$courent_id.'&nbre_files='.$files.$token.'" />';
		echo '</div></center></body></html>';
		exit;
	}

}
