<?php
	/*
	* TifFP composant passerelel de TourInFrance vers GMapFP pour Joomla! 4.x
	* Version J4.3
	* Creation date: Janvier 2024
	* Author: Fabrice4821 - www.gmapfp.org
	* Author email: webmaster@gmapfp.org
	* License GNU/GPL
	*/

namespace Joomla\Component\Tiffps\Administrator\Model;

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Form\Form;
use Joomla\CMS\Language\LanguageHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\MVC\Model\ListModel;
use Joomla\Database\ParameterType;
use Joomla\Registry\Registry;
use Joomla\Utilities\ArrayHelper;
use Joomla\CMS\Component\ComponentHelper;

class AccueilModel extends ListModel
{
	public $typeAlias = 'com_tiffps.accueil';

	public function __construct($config = array())
	{
		if (empty($config['filter_fields'])) {
			$config['filter_fields'] = array('id','name','syndication_format','language','syndication_type','modified','style_update_date');
		}

		parent::__construct($config);
	}

	public function getForm($data = array(), $loadData = true)
	{
		return false;
	}

	public function getPublishedTabs() {
		return array();
	}
	
	public function getTable($type = 'Tiffp', $prefix = 'Administrator', $config = array())
	{
		return parent::getTable($type, $prefix, $config);
	}


	public function getItems()
	{
		$params 		= ComponentHelper::getParams('com_tiffps');
		$languages 		= LanguageHelper::getLanguages();
		$gmapfp_data 	= self::getdate();
		$where			= $this->_buildContentWhere();
		
        $query = $this->_db->getQuery(true);
        $query->select('a.id, a.name, a.title, syndication_format as format, syndication_type as type, a.modified')
            ->from($this->_db->quoteName('#__tiffp_xml', 'a'))
			->join('LEFT', $this->_db->quoteName('#__tiffp_syndications', 's') . ' ON ' . $this->_db->quoteName('a.syndication_id') . ' = ' . $this->_db->quoteName('s.id'));
			
		if ($where) $query->where($where);
 		$orderCol	= $this->state->get('list.ordering', 'a.nom_fiche');
		$orderDirn	= $this->state->get('list.direction', 'ASC');
		$query->order($this->_db->escape($orderCol.' '.$orderDirn));
		$this->_db->setQuery($query);
        $items = $this->_db->loadObjectList('id');
		$items_filtre = array();

		foreach($items as $key => $item) {
			$item->update_date_gmapfp = '';
			$item->style_update_date = 'table-success';
			if (array_key_exists($item->name, $gmapfp_data)) {
				$item->update_date_gmapfp = $gmapfp_data[$item->name]->modified;
				if ($item->modified > $item->update_date_gmapfp)
					$item->style_update_date = 'table-danger';
			} else $item->style_update_date = 'table-danger';
			// Filter on the update_date.
			if ($style_update_date = $this->getState('filter.style_update_date')) {
				if ($item->style_update_date == $style_update_date) {
					$items_filtre[$key] = $key;
				}
			}
		}
		$items = array_diff_key($items, $items_filtre);

		return $items;
	}

	private function getdate() {
        $query = $this->_db->getQuery(true)
            ->select('`tif_id`, `gmapfp_id`, `modified`')
            ->from('#__tiffp_gmapfp AS a')
			->innerJoin('#__gmapfp AS b on b.id = a.gmapfp_id');
        $this->_db->setQuery($query);


		return $this->_db->loadObjectList('tif_id');
	}
	
	protected function _buildContentWhere()
	{
		$db			= Factory::getDBO();
		
		$where = array();

		// Filter by search in title
		$search = $this->getState('filter.search');
		if (!empty($search)) {
			if (stripos($search, 'id:') === 0) {
				$where[] = 'id = '.(int) substr($search, 3);
			} else {
				$search = $db->Quote('%'.$db->escape($search, true).'%');
				$where[] = '(name LIKE '.$search.' OR CONVERT(a.title USING utf8) LIKE '.$search.' OR s.title LIKE '.$search.')';
			}
		}
		// Filter on the language.
		if ($language = $this->getState('filter.language')) {
			$where[] = 'language = ' . $db->quote($language);
		}
		// Filter on the syndication_format.
		if ($syndication_format = $this->getState('filter.syndication_format')) {
			$where[] = 'syndication_format = ' . $db->quote($syndication_format);
		}
		// Filter on the type.
		if ($syndication_type = $this->getState('filter.syndication_type')) {
			$where[] = 'syndication_type = ' . $db->quote($syndication_type);
		}

		$where 		= ( count( $where ) ? implode( ' AND ', $where ) : '' );

		return $where;
	}

	protected function getStoreId($id = '')
	{
		// Compile the store id.
		$id	.= ':'.$this->getState('filter.search');
		$id	.= ':'.$this->getState('filter.syndication_format');
		$id	.= ':'.$this->getState('filter.language');
		$id	.= ':'.$this->getState('filter.style_update_date');
		$id	.= ':'.$this->getState('filter.syndication_format');

		return parent::getStoreId($id);
	}

	protected function populateState($ordering = null, $direction = null)
	{
		// Initialise variables.
		$app = Factory::getApplication('administrator');

		// Load the filter state.
		$search = $this->getUserStateFromRequest($this->context.'.filter.search', 'filter_search');
		$this->setState('filter.search', $search);

		$language = $this->getUserStateFromRequest($this->context.'.filter.language', 'filter_language', '');
		$this->setState('filter.language', $language);

		$style_update_date = $this->getUserStateFromRequest($this->context.'.filter.style_update_date', 'filter_update_date', '');
		$this->setState('filter.style_update_date', $style_update_date);

		$syndication_format = $this->getUserStateFromRequest($this->context.'.filter.syndication_format', 'filter_type', '');
		$this->setState('filter.syndication_format', $syndication_format);

		// Load the parameters.
		$params = ComponentHelper::getParams('com_tiffps');
		$this->setState('params', $params);

		// List state information.
		parent::populateState('id', 'ASC');
	}
}
