<?php
	/*
	* TifFP composant passerelel de TourInFrance vers GMapFP pour Joomla! 3.x
	* Version J3.3
	* Creation date: Juillet 2016
	* Author: Fabrice4821 - www.gmapfp.org
	* Author email: webmaster@gmapfp.org
	* License GNU/GPL
	*/

namespace Joomla\Component\Tiffps\Administrator\Model;

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Form\Form;
use Joomla\CMS\Language\LanguageHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\MVC\Model\AdminModel;
use Joomla\CMS\Versioning\VersionableModelTrait;
use Joomla\Database\ParameterType;
use Joomla\Registry\Registry;
use Joomla\Utilities\ArrayHelper;
use Joomla\Component\Tiffps\Administrator\Model\SyndicationbaseModel;

class SyndicationModel extends AdminModel
{
	use VersionableModelTrait;

	public $typeAlias = 'com_tiffps.syndication';
    protected $formName = 'syndication';
	
	public function getImport(&$pks)
	{
		$Import = new SyndicationbaseModel;
		$cn = $Import->getImport($pks);
		return $cn;
	}

	public function publish(&$pks, $value = 1)
	{
		$user = Factory::getUser();
		$table = $this->getTable('Syndication', 'TifFPsTable');
		$pks = (array) $pks;

		// Attempt to change the state of the records.
		if (!$table->publish($pks, $value, $user->get('id')))
		{
			$this->setError($table->getError());
			return false;
		}

		// Clear the component's cache
		$this->cleanCache();

		return true;
	}
	
	public function delete(&$pks)
	{
		$pks = (array) $pks;
		$table = $this->getTable('Syndication', 'TifFPsTable');

		foreach ($pks as $i => $pk) {
			if ($table->load($pk)) {
				if ($this->canDelete($table)) {
					if (!$table->delete($pk)) {
						$this->setError($table->getError());
						return false;
					}
				} else {
					unset($pks[$i]);
					$error = $this->getError();
					if ($error) {
						JError::raiseWarning(500, $error);
						return false;
					} else {
						JError::raiseWarning(403, JText::_('JLIB_APPLICATION_ERROR_DELETE_NOT_PERMITTED'));
						return false;
					}
				}
			} else {
				$this->setError($table->getError());
				return false;
			}
		}
		$this->cleanCache();

		return true;
	}

	protected function canDelete($record)
	{
		if (!empty($record->id)) {
			$user = Factory::getUser();

			if (!empty($record->catid)) {
				return $user->authorise('core.delete', 'com_tiffps.category.'.(int) $record->catid);
			} else {
				return $user->authorise('core.delete', $this->option);
			}
		}
	}

	protected function canEditState($record)
	{
		$user = Factory::getUser();

		// Check against the category.
		if (!empty($record->catid)) {
			return $user->authorise('core.edit.state', 'com_tiffps.category.'.(int) $record->catid);
		}
		// Default to component settings if category not known.
		else {
			return parent::canEditState($record);
		}
	}

    protected function loadFormData()
    {
        $app = Factory::getApplication();

        // Check the session for previously entered form data.
        $data = $app->getUserState('com_tiffps.edit.syndication.data', array());

        if (empty($data)) {
            $data = $this->getItem();
        }

        $this->preprocessData('com_tiffps.syndication', $data);

        return $data;
    }

	public function getForm($data = array(), $loadData = true)
	{
		$app  = Factory::getApplication();
		$user = $app->getIdentity();

		// Get the form.
		$form = $this->loadForm('com_tiffps.syndication', 'syndication', array('control' => 'jform', 'load_data' => $loadData));

		if (empty($form))
		{
			return false;
		}

        // Modify the form based on access controls.
        if (!$this->canEditState((object) $data)) {
            // Disable fields for display.
            $form->setFieldAttribute('ordering', 'disabled', 'true');
            $form->setFieldAttribute('published', 'disabled', 'true');

            // Disable fields while saving.
            // The controller has already verified this is a record you can edit.
            $form->setFieldAttribute('ordering', 'filter', 'unset');
            $form->setFieldAttribute('published', 'filter', 'unset');
        }

        // Don't allow to change the created_by user if not allowed to access com_users.
        if (!Factory::getUser()->authorise('core.manage', 'com_users')) {
            $form->setFieldAttribute('created_by', 'filter', 'unset');
        }

        return $form;
	}

}
