<?php
	/*
	* TifFP composant passerelel de TourInFrance vers GMapFP pour Joomla! 4.x
	* Version J4.3
	* Creation date: Novembre 2023
	* Author: Fabrice4821 - www.gmapfp.org
	* Author email: webmaster@gmapfp.org
	* License GNU/GPL
	*/

namespace Joomla\Component\Tiffps\Administrator\Model;

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\MVC\Model\AdminModel;
use Joomla\Filesystem\Folder;
use Joomla\Filesystem\File;
use Joomla\Filesystem\Path;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Log\Log;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\MVC\Factory\FormFactoryInterface;

class SyndicationbaseModel extends AdminModel
{
	protected $path = array();
	protected $log = 0;
	
    public function __construct($config = [], MVCFactoryInterface $factory = null, FormFactoryInterface $formFactory = null)
    {
        parent::__construct($config, $factory, $formFactory);
		
		$params = ComponentHelper::getParams('com_tiffps');
		$this->log = $params->get('log_files', 0);
		
		if($this->log >= 1)
			Log::addLogger(array('text_file' => 'TifFP_import.log.php'), Log::ALL, array('Syndications'));
	}
	
	public function step_cron_import_flux($options=array())
	{		
		$courent_id = 0;
		$ids		= array();

		if (array_key_exists('ids', $options)) $ids = $options['ids']; //ids des fiches  traiter
		if (array_key_exists('courent_id', $options)) $courent_id = $options['courent_id']; //id en cour de traitement
				
		if(empty($ids) and isset($options['Update_firstrun']) and $options['Update_firstrun']) {
			$ids = $this->get_published();
		}
		
		$trace = json_encode($options);
		if($this->log >= 1)
			Log::add(Text::_('COM_TIFFP_DEBUT_IMPORT').' : '.$trace, Log::INFO, 'Syndications');

		if (count($ids) or $courent_id) {
			if ($courent_id == 0) {
				$courent_id = array_shift($ids);
			}
			if($this->log >= 1)
				Log::add(Text::_('COM_TIFFP_DEBUT_TRAITEMENT_IMPORT'), Log::INFO, 'Syndications');
			$id = array($courent_id);
			$reuslt = $this->getImport($id);
			if($this->log >= 1) {
				Log::add(Text::_('COM_TIFFP_FIN_TRAITEMENT_TRAITER').$reuslt[0], Log::INFO, 'Syndications');
				Log::add(Text::_('COM_TIFFP_FIN_TRAITEMENT_IMPORTER').$reuslt[1], Log::INFO, 'Syndications');
			}
			if(isset($ids[0])) {
				$courent_id = array_shift($ids);
				$options['HasRun'] = 0;
			} else {
				$courent_id = 0;
				$options['HasRun'] = 1;
			}
		} else {
			$options['HasRun'] = 1;
		}
		$options['courent_id'] 		= $courent_id;
		$options['Error'] 			= '';
		$options['Update_firstrun'] = 0;
		$options['ids'] 			= $ids;
		
		$trace = json_encode($options);
		if($this->log >= 1)
			Log::add(Text::_('COM_TIFFP_NEXT_IMPORT').' : '.$trace, Log::INFO, 'Syndications');

		return ($options);
	}
	
	public function get_published()
	{
		// Create a new query object.
		$db = $this->getDbo();
		$query = $db->getQuery(true);

		// Select the required fields from the table.
		$query
			->select('id')
			->from('#__tiffp_syndications')
			->where('published = 1');

		$db->setQuery($query);
		$this->_id= $db->loadColumn();
		
		return $this->_id;
	}
	
	public function getImport(&$pks)
	{
		$traite = 0;
		$importe = 0;
		
		foreach ($pks as $id) {
			$result = array('traite' => 0, 'importe' => 0);;
			$table	= $this->getTable('Syndication', 'TifFPsTable');
			$table->load($id);
			$src = $table->syndication;
			if ($table->language == '*') $table->language ='all';
			$this->path['xml'] = JPATH_SITE.'/media/com_tiffps/xml/'.$table->language.'/';
			if (!is_dir(Path::clean($this->path['xml']))) {
				Folder::create($this->path['xml']);
			}
			$this->path['json'] = JPATH_SITE.'/media/com_tiffps/json/'.$table->language.'/';
			if (!is_dir(Path::clean($this->path['json']))) {
				Folder::create($this->path['json']);
			}

			//importation du fichier du flux
			$content = @file_get_contents(trim($src));

			if($content) {
				switch($table->syndication_type){
					case 1: //TIF
						$result = self::extraction_TIF_unitaire($content, $table);
						break;
					case 2: //DT
						$result = self::extraction_DataTourisme_unitaire($content, $table);
						break;
					case 3: //IntraMuros
						$result = self::extraction_Intramuros_unitaire($content, $table);
						break;
					default :
						Factory::getApplication()->enqueueMessage(Text::_('COM_TIFFP_NO_SYNDICATION_TYPE_FIND'), 'error');
				}

				unset($content);
			} else 
				Factory::getApplication()->enqueueMessage(Text::_('COM_TIFFP_NO_FLUX_RECOVER'), 'error');

			$traite = $traite + $result[0];
			$importe = $importe + $result[1];
		}
		return array($traite, $importe);
	}
	
	private function extraction_Intramuros_unitaire($content, $syndication)
	{
		if ($syndication->syndication_format == 'json') {
			$data_file = $content;
		} else {
			$dom = new \DomDocument;
			$dom->loadXML($content);
			$dom->formatOutput = true;
			$data_file = $dom->saveXML();
		}
		
		if($this->log >= 1)
			Log::add(Text::_('COM_TIFFP_EXTRACTION_INTRAMUROS_ID').' : '.$syndication->id, Log::INFO, 'Syndications');

		$src = $this->path[$syndication->syndication_format].'import_id_'.$syndication->id.'.'.$syndication->syndication_format;
		File::write($src, $data_file);

		$fp 	= fopen($src,"r");
		$save 	= false;
		$feed 	= false;
		$name 	= 'error';
		$importe 	= 0;
		$traite 	= 0;
		$entry 		= 0;
		
		if ($syndication->syndication_format == 'xml') {
			$dest_content = '<?xml version="1.0" encoding="utf-8" standalone="yes"?>'."\n";
			$feed_content = '<root>'."\n";
			
			while (!feof($fp)) {
				$line = fgets($fp);
				if (strpos(trim($line), '<root')===0) {
					$feed = true;
					$dest_content .= $feed_content;
				}
				if (strpos($line, '<list-item>')!==false OR strpos($line, '<list-item ')!==false) {
					$entry ++;
					$save = true;
				}
				if ($save) {
					$dest_content .= $line;
				}
				if (strpos($line, '</list-item>')!==false OR strpos($line, '</list-item ')!==false) {
					$entry --;
					if ($entry == 0) {
						if ($feed) $dest_content .='</root>'."\n";
						$update_date = '';
						$title = '';
						$xml_file = new \DOMDocument();
						$xml_file->loadXML($dest_content,LIBXML_NOERROR  );
						$xpathcontext = new \domXPath($xml_file);
						if ($xpathcontext->query("/root/list-item/id")->length) 
							$name = $xpathcontext->query("/root/list-item/id")->item(0)->nodeValue;
						if ($xpathcontext->query("/root/list-item/title")->length) 
							$title = $xpathcontext->query("/root/list-item/title")->item(0)->nodeValue;
						else $title = Text::_('COM_TIFFPS_ERROR_READ_XML');
						if ($xpathcontext->query("/root/list-item/last_modified")->length) 
							$update_date = substr($xpathcontext->query("/root/list-item/last_modified")->item(0)->nodeValue, 0, 10);
						$table	= $this->getTable('Xml', 'TifFPsTable');
						$data = new \stdclass();
						$data->name = $name;
						$data->title = $title;
						$data->syndication_id = $syndication->id;
						if ($update_date) $data->modified = $update_date;
						$data->xml = $dest_content;
						
						//si un enregistrement de 
						$update_xml_date = self::getIdTiffXml(array('name' => 'name', 'value' => $name), $syndication->id);
						if (!empty($update_xml_date)) {
							$data->id = $update_xml_date->id;
						}
						if (empty($update_xml_date) or $update_date > $update_xml_date->modified) {
							$table->bind($data);
							if (!$table->store(true)) {
								Factory::getApplication()->enqueueMessage($table->getError(), 'error');
							} else {
								$importe++;
							}
						}
						if($this->log >= 1)
							Log::add(Text::_('COM_TIFFP_EXTRACTION_FICHE_INTRAMUROS_XML').' : '.$data->name.' '.$data->title, Log::INFO, 'Syndications');
						$traite++;
						$dest_content = '<?xml version="1.0" encoding="utf-8" standalone="yes"?>'."\n";
						if ($feed) $dest_content .= $feed_content;
					}
				}
			}
		}
		
		if ($syndication->syndication_format == 'json') {
			$datas = json_decode($data_file, true);
			foreach ($datas as $fiche) {
				$update_date = '';
				$table	= $this->getTable('Xml', 'TifFPsTable');
				$data = new \stdclass();
				$data->name = $fiche['id'];
				if ($data->name) $name = $data->name;
				$data->title = $fiche['title'];
				if (array_key_exists('last_modified', $fiche)) {
					$data->modified = $fiche['last_modified'];
					$update_date = $fiche['last_modified'];
				}
				$data->xml = json_encode($fiche, true);
				$data->syndication_id = $syndication->id;
				
				//si un enregistrement de 
				$update_xml_date = self::getIdTiffXml(array('name' => 'name', 'value' => $name), $syndication->id);
				if (!empty($update_xml_date)) {
					$data->id = $update_xml_date->id;
				}
				if (empty($update_xml_date) or $update_date > $update_xml_date->modified) {
					$table->bind($data);
					if (!$table->store(true)) {
						Factory::getApplication()->enqueueMessage($table->getError(), 'error');
					} else {
						$importe++;
					}
				}
				if($this->log >= 1)
					Log::add(Text::_('COM_TIFFP_EXTRACTION_FICHE_INTRAMUROS_JSON').' : '.$data->name.' '.$data->title, Log::INFO, 'Syndications');
				$traite++;
			}
		}
		return array($traite, $importe);
	}
	
	private function extraction_DataTourisme_unitaire($content, $syndication)
	{
		$index_content 	= array();
		$importe 		= 0;
		$traite 		= 0;
		
		//le content est un fichiers ZIP
		$zip_file_name = $this->path[$syndication->syndication_format].'datatourisme_uplaoded.zip';
		$zip_files_path = $this->path[$syndication->syndication_format].'unzip/';
		$content = gzdecode($content);
		
		if (is_file($zip_file_name))
			File::delete($zip_file_name);
		if (is_dir($zip_files_path))
			Folder::delete($zip_files_path);
		
		file_put_contents($zip_file_name, $content);

		$zip = new \ZipArchive;
		$res = $zip->open($zip_file_name);
		if ($res === TRUE) {
			$zip->extractTo($zip_files_path);
			$zip->close();
		} else {
			Factory::getApplication()->enqueueMessage(Text::sprintf('COM_TIFFP_ECHEC_OPEN_ZIP_CODE', $res), 'error');
		}
		
		//traite le fichier index
		$filename = $zip_files_path.'index.'.$syndication->syndication_format;
		$handle = fopen($filename, "r");
		$index_file = fread($handle, filesize($filename));
		fclose($handle);
		if ($syndication->syndication_format == 'json') {
			$index_content = json_decode($index_file, true);
		}
		if ($syndication->syndication_format == 'xml') {
			//converstion xml en array
			$xml = simplexml_load_string($index_file, "SimpleXMLElement", LIBXML_NOCDATA);
			$json = json_encode($xml);
			$array = json_decode($json, true);
			if (is_array($array) and array_key_exists('object', $array)) $index_content = $array['object'];
		}
		
		if ($index_content) {
			foreach ($index_content as $content_infos) {
				$traite++;
				$sub_content = '';
				$table	= $this->getTable('Xml', 'TifFPsTable');
				$data = new \stdclass();
				$data->path_file = $content_infos['file'];
				$data->title = $content_infos['label'];
				$data->syndication_id = $syndication->id;
				$data->modified = str_replace(array('T', 'Z'), array(' ', ''), $content_infos['lastUpdateDatatourisme']);
				
				if($this->log >= 1)
					Log::add(Text::_('COM_TIFFP_EXTRACTION_DATATOURISME_ID').' : '.$syndication->id, Log::INFO, 'Syndications');
					
				//si un enregistrement de 
				$update_xml_date = self::getIdTiffXml(array('name' => 'path_file', 'value' => $content_infos['file']), $syndication->id);
				if (!empty($update_xml_date)) {
					$data->id = $update_xml_date->id;
				}
				if (empty($update_xml_date) or $update_date > $update_xml_date->modified) {
					$filename = $zip_files_path.'objects/'.$content_infos['file'];
					$handle = fopen($filename, "r");
					$data->xml = fread($handle, filesize($filename));
					fclose($handle);
					if ($syndication->syndication_format == 'json') {
						$sub_content = json_decode($data->xml, true);
					}
					if ($syndication->syndication_format == 'xml') {
						//converstion xml en array
						$xml = simplexml_load_string($data->xml, "SimpleXMLElement", LIBXML_NOCDATA);
						$json = json_encode($xml);
						$array = json_decode($json, true);
						if (is_array($array) and array_key_exists('object', $array)) $sub_content = $array['object'];
					}
					if ($sub_content and array_key_exists('dc:identifier', $sub_content) ) {
						$data->name = $sub_content['dc:identifier'];
					}
					$table->bind($data);
					if (!$table->store(true)) {
						Factory::getApplication()->enqueueMessage($table->getError(), 'error');
					} else {
						$importe++;
					}
				}
			}
		} else {
			Factory::getApplication()->enqueueMessage(Text::_('COM_TIFFP_NO_DATATOURISME_INDEX_FILE_RECOVER'), 'error');
		}
		
		return array($traite, $importe);
	}
	
	private function extraction_TIF_unitaire($content, $syndication)
	{
		$params = ComponentHelper::getParams('com_tiffps');
		
		//prpare le traitement des fichiers expirs
		$old_fiches = '';
		if ($params->get('tiffps_del_old', 0) and $syndication->tiffps_type_fiche) {
			$db = $this->getDbo();
			$query = $db->getQuery(true);

			$query
				->select($db->quoteName(array('id_table', 'gmapfp_id', 'tif_id')))
				->from($db->quoteName('#__tiffp_gmapfp'))
				->where($db->quoteName('tif_id').' like '.$db->quote($syndication->tiffps_type_fiche.'%'));

			$db->setQuery($query);
			$old_fiches = $db->loadObjectlist('tif_id');
		}

		if ($syndication->syndication_format == 'json') {
			$data_file = $content;
		} else {
			$dom = new \DomDocument;
			$dom->loadXML($content);
			$dom->formatOutput = true;
			$data_file = $dom->saveXML();
		}

		if($this->log >= 1)
			Log::add(Text::_('COM_TIFFP_EXTRACTION_TIF_ID').' : '.$syndication->id, Log::INFO, 'Syndications');

		$src = $this->path[$syndication->syndication_format].'import_id_'.$syndication->id.'.'.$syndication->syndication_format;
		File::write($src, $data_file);

		$fp 	= fopen($src,"r");
		$save 	= false;
		$feed 	= false;
		$name 	= 'error';
		$importe 	= 0;
		$traite 	= 0;
		$entry 		= 0;
		// $downloaded_date = date();
		
		if ($syndication->syndication_format == 'xml') {
			$dest_content = '<?xml version="1.0" encoding="utf-8" standalone="yes"?>'."\n";
			$feed_content = '<feed xmlns:d="http://schemas.microsoft.com/ado/2007/08/dataservices" xmlns:m="http://schemas.microsoft.com/ado/2007/08/dataservices/metadata">'."\n";
			
			while (!feof($fp)) {
				$line = fgets($fp);
				if (strpos(trim($line), '<feed ')===0) {
					$feed = true;
					$dest_content .= $feed_content;
				}
				if (strpos($line, '<entry>')!==false OR strpos($line, '<entry ')!==false) {
					$entry ++;
					$save = true;
				}
				if ($save) {
					$dest_content .= $line;
					if(strpos(trim($line), '<id>')===0) {
						if (strstr($line, "Objects('")) {
							$name = strstr($line, "Objects('");
							$name = strstr($name, "')", true);
							$name = substr($name, 9);
						}
					}
				}
				if (strpos($line, '</entry>')!==false OR strpos($line, '</entry ')!==false) {
					$entry --;
					if ($entry == 0) {
						if ($feed) $dest_content .='</feed>'."\n";
						$update_date = '';
						$title = '';
						$xml_file = new \DOMDocument();
						$xml_file->loadXML($dest_content,LIBXML_NOERROR  );
						$xpathcontext = new \domXPath($xml_file);
						if ($xpathcontext->query("/feed/entry/title")->length) 
							$title = $xpathcontext->query("/feed/entry/title")->item(0)->nodeValue;
						else $title = Text::_('COM_TIFFPS_ERROR_READ_XML');
						if ($xpathcontext->query("/feed/entry/updated")->length) 
							$update_date = substr($xpathcontext->query("/feed/entry/updated")->item(0)->nodeValue, 0, 10);
						$table	= $this->getTable('Xml', 'TifFPsTable');
						$data = new \stdclass();
						$data->name = $name;
						$data->title = $title;
						$data->syndication_id = $syndication->id;
						$data->modified = $update_date;
						$data->xml = $dest_content;
						
						//si un enregistrement de 
						$update_xml_date = self::getIdTiffXml(array('name' => 'name', 'value' => $name), $syndication->id);
						if (!empty($update_xml_date)) {
							$data->id = $update_xml_date->id;
						}
						if (empty($update_xml_date) or $update_date > $update_xml_date->modified) {
							$table->bind($data);
							if (!$table->store(true)) {
								Factory::getApplication()->enqueueMessage($table->getError(), 'error');
							} else {
								$importe++;
							}
						}
						if($this->log >= 1)
							Log::add(Text::_('COM_TIFFP_EXTRACTION_FICHE_TIF_XML').' : '.$data->name.' '.$data->title, Log::INFO, 'Syndications');
						$traite++;
						$dest_content = '<?xml version="1.0" encoding="utf-8" standalone="yes"?>'."\n";
						if ($feed) $dest_content .= $feed_content;
					}
				}
			}
		}
		
		if ($syndication->syndication_format == 'json') {
			$datas = json_decode($data_file, true);
			foreach ($datas['value'] as $fiche) {
				$table	= $this->getTable('Xml', 'TifFPsTable');
				$data = new \stdclass();
				$data->name = $fiche['SyndicObjectID'];
				if ($data->name) $name = $data->name;
				$data->title = $fiche['SyndicObjectName'];
				$data->modified = $fiche['Updated'];
				$update_date = $fiche['Updated'];
				$data->xml = json_encode($fiche, true);
				$data->syndication_id = $syndication->id;
				
				//traite les fiches dsactives
				if ($old_fiches and array_key_exists($data->name, $old_fiches)) {
					unset($old_fiches[$data->name]);
				}
				
				//si un enregistrement de 
				$update_xml_date = self::getIdTiffXml(array('name' => 'name', 'value' => $name), $syndication->id);
				if (!empty($update_xml_date)) {
					$data->id = $update_xml_date->id;
				}
				if (empty($update_xml_date) or $update_date > $update_xml_date->modified) {
					$table->bind($data);
					if (!$table->store(true)) {
						Factory::getApplication()->enqueueMessage($table->getError(), 'error');
					} else {
						$importe++;
					}
				}
				if($this->log >= 1)
					Log::add(Text::_('COM_TIFFP_EXTRACTION_FICHE_TIF_JSON').' : '.$data->name.' '.$data->title, Log::INFO, 'Syndications');
				$traite++;
			}
			if (!empty($old_fiches))
				self::disable_old_fiches($old_fiches);
		}
		
		return array($traite, $importe);
	}

	private function disable_old_fiches($datas)
	{
		$params = ComponentHelper::getParams('com_tiffps');
		$db 	= $this->getDbo();
		
		foreach ($datas as $key => $data) {
			if ($params->get('tiffps_del_old', 0) == 2) { //supprime
				//traite la table gmapfp
				$query = $db->getQuery(true);
				$query
					->delete($db->quoteName('#__gmapfp'))
					->where($db->quoteName('id').' = '.$db->quote($data->gmapfp_id));
				$db->setQuery($query);
				$db->execute();
				//traite la table tiffp_gmapfp
				$query = $db->getQuery(true);
				$query
					->delete($db->quoteName('#__tiffp_gmapfp'))
					->where($db->quoteName('tif_id').' = '.$db->quote($key));
				$db->setQuery($query);
				$db->execute();
			} elseif ($params->get('tiffps_del_old', 0) == 1) { //desactive
				//traite la table gmapfp
				$query = $db->getQuery(true);
				$query
					->update($db->quoteName('#__gmapfp'))
					->set($db->quoteName('state') . ' = 0')
					->where($db->quoteName('id').' = '.$db->quote($data->gmapfp_id));
				$db->setQuery($query);
				$db->execute();
			}
			//traite la table tiffp_xml
			$query = $db->getQuery(true);
			$query
				->delete($db->quoteName('#__tiffp_xml'))
				->where($db->quoteName('name').' = '.$db->quote($key));
			$db->setQuery($query);
			$db->execute();
		}
		return;
	}
	
	private function getIdTiffXml($filter, $syndication_id)
	{
		$db = $this->getDbo();
		$query = $db->getQuery(true);

		$query
			->select($db->quoteName(array('id', 'modified')))
			->from($db->quoteName('#__tiffp_xml'))
			->where($db->quoteName($filter['name']).' = '.$db->quote($filter['value']))
			->where($db->quoteName('syndication_id').' = '.$db->quote($syndication_id));

		$db->setQuery($query);
		return $db->loadObject();
	}

	public function getForm($data = array(), $loadData = true)
	{
	}
}
