<?php
	/*
	* TifFP composant passerelel de TourInFrance vers GMapFP pour Joomla! 4.x
	* Version J4.1
	* Creation date: Août 2023
	* Author: Fabrice4821 - www.gmapfp.org
	* Author email: webmaster@gmapfp.org
	* License GNU/GPL
	*/

namespace Joomla\Component\Tiffps\Administrator\Model;

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Language\Associations;
use Joomla\CMS\MVC\Model\ListModel;
use Joomla\CMS\Table\Table;
use Joomla\Database\ParameterType;
use Joomla\Utilities\ArrayHelper;

class SyndicationsModel extends ListModel
{
	public function __construct($config = array())
	{
		if (empty($config['filter_fields'])) {
			$config['filter_fields'] = array(
				'id', 'a.id',
				'title', 'a.title',
				'syndication_format', 'a.syndication_format',
				'type', 'a.syndication_type',
				'published', 'a.published',
				'language', 'a.language',
				);
		}

		parent::__construct($config);
	}

	protected function getStoreId($id = '')
	{
		// Compile the store id.
		$id	.= ':'.$this->getState('filter.state');

		return parent::getStoreId($id);
	}

	protected function getListQuery()
	{
		// Initialise variables.
		$db		= $this->getDbo();
		$query	= $db->getQuery(true);

		// Select the required fields from the table.
		$query->select(
			$this->getState(
				'list.select',
				'a.id AS id, a.title AS name, '.
				'a.published AS published, '.
				'a.syndication_format AS format, '.
				'a.syndication_type AS type, '.
				'a.language AS language, '.
				'a.checked_out, a.checked_out_time, uc.name AS editor'
			)
		);
		$query->from('`#__tiffp_syndications` AS a')
            ->join('LEFT', $db->quoteName('#__users', 'uc'), $db->quoteName('uc.id') . ' = ' . $db->quoteName('a.checked_out'));

         // Filter by published state
        $published = (string) $this->getState('filter.published');

        if (is_numeric($published)) {
            $query->where($db->quoteName('a.published') . ' = :published');
            $query->bind(':published', $published, ParameterType::INTEGER);
        } elseif ($published === '') {
            $query->where('(' . $db->quoteName('a.published') . ' = 0 OR ' . $db->quoteName('a.published') . ' = 1)');
        }

		// Filter on the language.
        if ($language = $this->getState('filter.language')) {
            $query->where($db->quoteName('a.language') . ' = :language');
            $query->bind(':language', $language);
        }

		// Filter on the syndication_format.
        if ($syndication_format = $this->getState('filter.syndication_format')) {
            $query->where($db->quoteName('a.syndication_format') . ' = :syndication_format');
            $query->bind(':syndication_format', $syndication_format);
        }

		// Add the list ordering clause.
		$orderCol	= $this->state->get('list.ordering', 'a.title');
		$orderDirn	= $this->state->get('list.direction', 'ASC');
		$query->order($db->escape($orderCol.' '.$orderDirn));

		//die(print_r($query));
		return $query;
	}

	public function getTable($type = 'Syndications', $prefix = 'TifFPsTable', $config = array())
	{
		return Table::getInstance($type, $prefix, $config);
	}

	protected function populateState($ordering = null, $direction = null)
	{
		// Initialise variables.
		$app = Factory::getApplication('administrator');

		// Load the filter state.
		$state = $this->getUserStateFromRequest($this->context.'.filter.state', 'filter_state', '', 'string');
		$this->setState('filter.state', $state);

		// Load the parameters.
		$params = ComponentHelper::getParams('com_tiffps');
		$this->setState('params', $params);

		// List state information.
		parent::populateState('a.title', 'ASC');
	}
}