<?php
	/*
	* TifFP composant passerelel de TourInFrance vers GMapFP pour Joomla! 4.x
	* Version J4.0
	* Creation date: Juin 2023
	* Author: Fabrice4821 - www.gmapfp.org
	* Author email: webmaster@gmapfp.org
	* License GNU/GPL
	*/

namespace Joomla\Component\Tiffps\Administrator\Model;

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Form\Form;
use Joomla\CMS\Language\LanguageHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\MVC\Model\AdminModel;
use Joomla\Component\Categories\Administrator\Helper\CategoriesHelper;
use Joomla\Database\ParameterType;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\Registry\Registry;
use Joomla\Utilities\ArrayHelper;
use Joomla\CMS\Table\Category;
use Joomla\CMS\Table\Table;
use Joomla\CMS\Table\TableInterface;
use Joomla\CMS\Versioning\VersionableModelTrait;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Form\FormFactoryInterface;


class TiffpModel extends AdminModel
{
	use VersionableModelTrait;

	protected $text_prefix = 'COM_TIFFPS_TIFFP';
	public $typeAlias = 'com_tiffps.tiffp';

	protected $_data;
	protected $_data_content;
	protected $_fields;
	protected $log;
	protected $logger;
	protected $xpathcontext;
	protected $_id;
	protected $path;
	protected $path_zip;

	public function publish(&$pks, $value = 1)
	{
		$user = Factory::getUser();
		$table = $this->getTable('Tiffp', 'TifFPsTable');
		$pks = (array) $pks;

		// Attempt to change the state of the records.
		if (!$table->publish($pks, $value, $user->get('id')))
		{
			$this->setError($table->getError());
			return false;
		}

		// Clear the component's cache
		$this->cleanCache();

		return true;
	}
	
	public function delete(&$pks)
	{
        $pks   = ArrayHelper::toInteger((array) $pks);
		$table = $this->getTable('TifFP', 'TifFPsTable');

		foreach ($pks as $i => $pk)
		{
			if ($table->load($pk))
			{
				if ($this->canDelete($table))
				{
					if (!$table->delete($pk))
					{
						$this->setError($table->getError());
						return false;
					}
				}
				else
				{
					unset($pks[$i]);
					$error = $this->getError();
					if ($error)
					{
						$this->setError($table->getError());
						return false;
					}
					else
					{
						$this->setError(403, Text::_('JLIB_APPLICATION_ERROR_DELETE_NOT_PERMITTED'));
						return false;
					}
				}
			}
			else
			{
				$this->setError($table->getError());
				return false;
			}
		}
		$this->cleanCache();

		return true;
	}

	protected function canDelete($record)
	{
		if (!empty($record->id)) {
			$user = Factory::getUser();

			if (!empty($record->catid)) {
				return $user->authorise('core.delete', 'com_tiffps.category.'.(int) $record->catid);
			}
			else {
				return $user->authorise('core.delete', $this->option);
			}
		}
	}

	protected function canEditState($record)
	{
		$user = Factory::getUser();

		// Check against the category.
		if (!empty($record->catid)) {
			return $user->authorise('core.edit.state', 'com_tiffps.category.'.(int) $record->catid);
		}
		// Default to component settings if category not known.
		else {
			return parent::canEditState($record);
		}
	}

    public function getItem($pk = null)
	{
        $pk = (!empty($pk)) ? $pk : (int) $this->getState($this->getName() . '.id');
		// Load the data
		if (empty( $this->_data )) {
			$query = ' SELECT c.*, v.name AS last_editor '.
					'  FROM #__tiffp AS c '.
					'  LEFT JOIN #__users AS v ON v.id = c.modified_by' .
					'  WHERE c.id = '.$pk;
			$this->_db->setQuery( $query );
			$this->_data = $this->_db->loadObject();
		}
		if (!$this->_data) {
			$createdate = Factory::getDate();
			$this->_data = new \stdClass();

			$query = 'SHOW COLUMNS FROM `#__tiffp`';
				$this->_db->setQuery( $query );
				$fields = $this->_db->loadObjectList('Field');
				
			foreach ($fields as $field => $data) {
				$this->_data->$field = null;
				$option_field = 'options_'.$field;
				$this->_data->$option_field = null;
			}
		} else {
			if ($this->_data->options) {
				$options = json_decode($this->_data->options);
				foreach ($options as $key => $value) {
					$this->_data->$key = $value;
				}
			}
		}
		return $this->_data;
	}

    public function getItem_Content($pk = null)
	{
        $pk = (!empty($pk)) ? $pk : (int) $this->getState($this->getName() . '.id');
		// Load the data
		if (empty( $this->_data_content )) {
			$query = ' SELECT c.*, v.name AS last_editor '.
					'  FROM #__tiffp_content AS c '.
					'  LEFT JOIN #__users AS v ON v.id = c.modified_by' .
					'  WHERE c.id = '.$pk;
			$this->_db->setQuery( $query );
			$this->_data_content = $this->_db->loadObject();
		}
		if (!$this->_data_content) {
			$createdate = Factory::getDate();
			$this->_data_content = new \stdClass();

			$query = 'SHOW COLUMNS FROM `#__tiffp_content`';
				$this->_db->setQuery( $query );
				$fields = $this->_db->loadObjectList('Field');
				
			foreach ($fields as $field => $data) {
				$this->_data_content->$field = null;
				$option_field = 'options_'.$field;
				$this->_data_content->$option_field = null;
			}
		} else {
			if ($this->_data_content->options) {
				$options = json_decode($this->_data_content->options);
				foreach ($options as $key => $value) {
					$this->_data_content->$key = $value;
				}
			}
		}
		return $this->_data_content;
	}

    public function save($data)
	{
		$post_data = $_POST; //garanti le html

        $table      = $this->getTable();
        $context    = $this->option . '.' . $this->name;
        $app        = Factory::getApplication();

        $key = $table->getKeyName();
        $pk = (isset($data[$key])) ? $data[$key] : (int) $this->getState($this->getName() . '.id');
        $isNew = true;

        // Include the plugins for the save events.
        PluginHelper::importPlugin($this->events_map['save']);

        // Allow an exception to be thrown.
        try {
            // Load the row if saving an existing record.
            if ($pk > 0) {
                $table->load($pk);
                $isNew = false;
            }

            // Bind the data.
            if (!$table->bind($data)) {
                $this->setError($table->getError());
                return false;
            }

			//traite les champs options
			$field_options = array();
			foreach ($post_data['jform'] as $post_key => $tiffp_field) {
				if (substr($post_key, 0, 8) == 'options_') $field_options[$post_key] = $tiffp_field;
			}
			$table->options = json_encode($field_options);
			unset($field_options, $post_key, $tiffp_field);
			
			foreach ($post_data['jform'] as $post_key => $tiffp_field) {
				if (substr($post_key, 0, 2) == 'g_') $table->$post_key = $tiffp_field;
			}

			$field_options = array();
			foreach ($post_data['jform'] as $post_key => $tiffp_field) {
				if (substr($post_key, 0, 6) == 'champ_') $field_options[substr($post_key, 6)] = $tiffp_field;
			}
			$table->params_champs = json_encode($field_options);
			unset($field_options, $post_key, $tiffp_field);

			
            // Prepare the row for saving
            $this->prepareTable($table);

            // Check the data.
            if (!$table->check()) {
                $this->setError($table->getError());

                return false;
            }

            // Trigger the before save event.
            $result = $app->triggerEvent($this->event_before_save, array($context, $table, $isNew, $data));

            if (\in_array(false, $result, true)) {
                $this->setError($table->getError());

                return false;
            }

            // Store the data.
            if (!$table->store()) {
                $this->setError($table->getError());

                return false;
            }

            // Clean the cache.
            $this->cleanCache();

            // Trigger the after save event.
            $app->triggerEvent($this->event_after_save, array($context, $table, $isNew, $data));
        } catch (\Exception $e) {
            $this->setError($e->getMessage());

            return false;
        }

        if (isset($table->$key)) {
            $this->setState($this->getName() . '.id', $table->$key);
        }

        $this->setState($this->getName() . '.new', $isNew);

		return true;
	}
	
	public function getForm($data = array(), $loadData = true)
	{
		$app  = Factory::getApplication();
		$user = $app->getIdentity();

		// Get the form.
		$form = $this->loadForm('com_tiffps.tiffp', 'tiffp', array('control' => 'jform', 'load_data' => $loadData));

		if (empty($form))
		{
			return false;
		}

		$jinput = $app->input;

		/*
		 * The front end calls this model and uses a_id to avoid id clashes so we need to check for that first.
		 * The back end uses id so we use that the rest of the time and set it to 0 by default.
		 */
		$id = $jinput->get('a_id', $jinput->get('id', 0));

		// Determine correct permissions to check.
		if ($this->getState('tiffp.id', $id))
		{
			// Existing record. Can only edit in selected categories.
			$form->setFieldAttribute('catid', 'action', 'core.edit');

			// Existing record. Can only edit own articles in selected categories.
			if ($app->isClient('administrator'))
			{
				$form->setFieldAttribute('catid', 'action', 'core.edit.own');
			}
			else
			// Existing record. We can't edit the category in frontend if not edit.state.
			{
				if ($id != 0 && (!$user->authorise('core.edit.state', 'com_tiffps.tiffp.' . (int) $id))
					|| ($id == 0 && !$user->authorise('core.edit.state', 'com_tiffps')))
				{
					$form->setFieldAttribute('catid', 'readonly', 'true');
					$form->setFieldAttribute('catid', 'filter', 'unset');
				}
			}
		}
		else
		{
			// For new articles we load the potential state + associations
			if ($formField = $form->getField('catid'))
			{
				$assignedCatids = (int) ($data['catid'] ?? $form->getValue('catid'));

				$assignedCatids = is_array($assignedCatids)
					? (int) reset($assignedCatids)
					: (int) $assignedCatids;

				// Try to get the category from the html code of the field
				if (empty($assignedCatids))
				{
					$assignedCatids = $formField->getAttribute('default', null);

					// Choose the first category available
					$xml = new \DOMDocument;
					libxml_use_internal_errors(true);
					$xml->loadHTML($formField->__get('input'));
					libxml_clear_errors();
					libxml_use_internal_errors(false);
					$options = $xml->getElementsByTagName('option');

					if (!$assignedCatids && $firstChoice = $options->item(0))
					{
						$assignedCatids = $firstChoice->getAttribute('value');
					}
				}

				// Activate the reload of the form when category is changed
				$form->setFieldAttribute('catid', 'refresh-enabled', true);
				$form->setFieldAttribute('catid', 'refresh-cat-id', $assignedCatids);
				$form->setFieldAttribute('catid', 'refresh-section', 'item');
			}

			// New record. Can only create in selected categories.
			$form->setFieldAttribute('catid', 'action', 'core.create');
		}

		return $form;
	}

	protected function loadFormData()
	{
		// Check the session for previously entered form data.
		$app  = Factory::getApplication();
		$data = $app->getUserState('com_gmapfp.edit.item.data', array());

		if (empty($data))
		{
			$data = $this->getItem();

			// Prime some default values.
			if ($this->getState('item.id') == 0)
			{
				$filters     = (array) $app->getUserState('com_gmapfp.items.filter');
				$filterCatId = $filters['category_id'] ?? null;

				// $data->set('g_catid', $app->input->getInt('g_catid', $filterCatId));
			}
		}
		
		if (isset($data->attribs))
		{
			$attribs = new Registry($data->attribs);
			$data->cp_position = $attribs->get('cp_position');
		}
		
		$this->preprocessData('com_gmapfp.item', $data);

		return $data;
	}
}
