<?php
	/*
	* TifFP composant passerelel de TourInFrance vers GMapFP pour Joomla! 4.x
	* Version J4.5
	* Creation date: Juin 2024
	* Author: Fabrice4821 - www.gmapfp.org
	* Author email: webmaster@gmapfp.org
	* License GNU/GPL
	*/

namespace Joomla\Component\Tiffps\Administrator\Model;

require_once JPATH_ADMINISTRATOR.'/components/com_tiffps/src/Helper/TiffpsHelper.php';

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\Filesystem\Folder;
use Joomla\Filesystem\File;
use Joomla\Filesystem\Path;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Log\Log;
use Joomla\CMS\Client\ClientHelper;
use Joomla\Component\Tiffps\Administrator\Helper\TiffpsHelper;
use Joomla\CMS\Helper\TagsHelper;
use Joomla\CMS\MVC\Model\ListModel;
use Joomla\CMS\MVC\Factory\MVCFactoryInterface;
use Joomla\CMS\Image\Image;

defined('_JEXEC') or die;

class TiffpbaseModel extends ListModel
{
    use TiffpsXMLTrait;
    use TiffpsJSONTrait;
    use TiffpsUserTrait;

	protected $user_id 			= '';
	protected $path_main_images = 'images/gmapfp/';
	protected $path_xml 		= '/media/com_tiffps/xml/';
	protected $path_zip 		= '';
	protected $path 			= '';
	protected $xpathcontext 	= '';
	protected $json 			= '';
	protected $tiffp_name		= '';
	protected $tiffp_type_syndication 			= '';
	protected $tiffp_type_format 				= '';
	protected $extraction_syndication_type 		= '';
	protected $extraction_syndication_format 	= '';
	protected $tiffps_photos_path 				= '';
	protected $tiffps_fields_separator 			= '';
	protected $tiffps_values_separator 			= '';
	protected $log = 0;

    public function __construct($config = [], MVCFactoryInterface $factory = null)
    {
        parent::__construct($config, $factory);
		
		$params = ComponentHelper::getParams('com_tiffps');
		$this->log = $params->get('log_files', 0);
		
		if($this->log >= 1)
			Log::addLogger(array('text_file' => 'TifFP_extracte.log.php'), Log::ALL, array('Extractions'));
	}

	function getPublishedTIF()
	{
		// Load the data
			$query = 'SELECT id FROM #__tiffp WHERE published = 1';
				$this->_db->setQuery( $query );
				$PublishedTIF = $this->_db->loadColumn();
			
		return $PublishedTIF;
	}

	function import_ftp()
	{
		$app	= Factory::getApplication();
		$Params = ComponentHelper::getParams('com_tiffps');
		$return_val = true;

		$ftp_url = $Params->get('com_tiffp_ftp_url');
		$login = $Params->get('com_tiffp_login');
		$passw = $Params->get('com_tiffp_passw');
		$zip_ftp = $Params->get('com_tiffp_zip_ftp');
// **************** ligne spcifique pour cron ot pm ***************************//
		if (!$this->path_zip) 
			$path_zip = $this->path_xml.$zip_ftp;
		else
			$path_zip = $this->path_xml.$this->path_zip;

		$zip_tmp = JPATH_SITE.$path_zip;

		ini_set( 'default_socket_timeout', 300 );
		try {
			// Ouverture de la connexion au ftp dont l'url est donne dans le fichier de conf
			$idClientFtp = ftp_connect( $ftp_url );

			// Exception lance si connexion refuse
			if ( $idClientFtp == false) {
				throw new appException(appException::FTP_CONNECT_REFUSED);
			}
			
			// Identification
			$idh = ftp_login($idClientFtp, $login, $passw);
			
			// Exception lance si identification refuse
			if ( $idh == false) {
				throw new appException(appException::FTP_ID_REFUSED);
			}
			
		} catch(appException $e) {
			$app->enqueueMessage(Text::_('Probleme survenu lors de la connection au FTP : ' . $e->getMessage()."<br/> Impossible d'acceder ou de s'identifier a " .FTP_URL),'WARNING');
			$return_val = false;
		}
		
		if ( !ftp_get ( $idClientFtp , $zip_tmp , $zip_ftp , FTP_BINARY ) ) {
			$app->enqueueMessage(Text::_("Problme dans la rcupration du zip sur le ftp\n"),'WARNING');
			$return_val = false;
		}

		// Fermeture de la connexion au FTP
		ftp_close($idClientFtp);
		
		return $return_val;
	}
	
	function extract_file($id=array())
	{
		$params = ComponentHelper::getParams('com_tiffps');
		$app	= Factory::getApplication();
		$jinput = $app->input;
		$files = array();

		$courent_file   = $jinput->get('courent_file', -1, 'INT');
		$courent_file++;
		$array_id  		= $jinput->get('tiffps_id', 'not found', 'ARRAY');
		if ($array_id != 'not found') {
			$id = $array_id;
		}

		$courent_id   	= $jinput->get('courent_id', 0, 'INT');
		$link = 'index.php?option=com_tiffps&task=tiffps.extract_file';

		if (count($id))
			while ($courent_id < count($id)) {
				$this->init_global_variable($id[(int)$courent_id]);
				
				$files = TiffpsHelper::getListNamesXML($this->table->verif, $this->tiffp_type_format);

				if ($this->log){
					Log::add(Text::_('COM_TIFFP_X_FILES_TO_IMPORT').count($files), Log::INFO, 'Extractions');
				}

				if (count($files)) {
					$nbre_files = count($files);
					while ($courent_file < $nbre_files) {
						$file = $files[(int)$courent_file];
						$erreur = $this->extraction_data_of_file($files[$courent_file], $id[$courent_id]);
						$erreur = '';
						TiffpsHelper::getProcessPage($files[$courent_file], $courent_file, $link, $nbre_files, $erreur, $id, $courent_id, $this->table->nom_fiche);
						$courent_file ++;
						exit;
					}
				} else
					$app->enqueueMessage(Text::_('COM_TIFFP_NO_FILES_TO_IMPORT'),'WARNING');
				
				$courent_id ++;
				$courent_file = 0;
			}
			
		self::import_images();
		
		return;
	}
	
	public function step_cron_extract_file($options=array())
	{
		if ($this->log)
			Log::addLogger(array('text_file' => 'TifFP.cron.log.php'), Log::ALL, array('msgTiffp'));
		$params = ComponentHelper::getParams('com_tiffps');

		$courent_file   = 0;
		$courent_id   	= 0;
		$nbre_files   	= 0;
		$ids			= array();

		if (array_key_exists('ids', $options)) $ids = $options['ids']; //ids des fiches  traiter
		if (array_key_exists('courent_file', $options)) $courent_file = $options['courent_file']; //prochain fichier  traiter
		if (array_key_exists('courent_id', $options)) $courent_id = $options['courent_id']; //id en cour de traitement
		if (array_key_exists('nbre_files', $options)) $nbre_files = $options['nbre_files']; //nombre de fichier total comprenant la fiche
				
		if(empty($ids) and isset($options['Update_firstrun']) and $options['Update_firstrun']) {
			$ids = $this->getPublishedTIF();
		}
		
		$trace = json_encode($options);
		Log::add(Text::_('COM_TIFFP_DEBUT_EXTRACTION').' : '.$trace, Log::INFO, 'CRON');
		
		if (count($ids) or $courent_id) {
			if ($courent_id == 0) {
				$courent_id = array_shift($ids);
			}
			$this->init_global_variable((int)$courent_id);
			$files = TiffpsHelper::getListNamesXML($this->table->verif, $this->tiffp_type_format);
			if ($courent_file == 0) $nbre_files = count($files);
			if ($nbre_files > 0)
				if ($courent_file < count($files)) {
					$file = $files[(int)$courent_file];
					Log::add(Text::_('COM_TIFFP_DEBUT_TRAITEMENT').' : '.$file, Log::INFO, 'TRAITEMENT');
					$erreur = $this->extraction_data_of_file($file, $courent_id);
					$courent_file ++;
					Log::add(Text::_('COM_TIFFP_COURENT_FILE').' : '.$courent_file, Log::INFO, 'CRON');
					Log::add(Text::_('COM_TIFFP_FIN_TRAITEMENT').' : '.$file, Log::INFO, 'TRAITEMENT');
				}
			if ($courent_file >= $nbre_files) {
				if(isset($ids[0])) {
					$courent_id = array_shift($ids);
					$nbre_files = 0;
					$options['HasRun'] = 0;
				} else {
					$courent_id = 0;
					$options['HasRun'] = 1;
				}
				$courent_file = 0;
			}
		} else {
			$options['HasRun'] = 1;
		}
		$options['courent_file'] 	= $courent_file;
		$options['courent_id'] 		= $courent_id;
		$options['nbre_files'] 		= $nbre_files;
		$options['Error'] 			= '';
		$options['Update_firstrun'] = 0;
		$options['ids'] 			= $ids;
		
		$trace = json_encode($options);
		Log::add(Text::_('COM_TIFFP_NEXT_IMPORT').' : '.$trace, Log::INFO, 'CRON');

		return ($options);
	}
	
	public function step_cron_extract_image($options=array())
	{
		if ($this->log)
			Log::addLogger(array('text_file' => 'TifFP.cron.log.php'), Log::ALL, array('msgTiffp'));

		$options['HasRun'] = !self::import_images(true);
		$options['Error'] = '';
		
		if ($this->log)
			Log::add(Text::_('COM_TIFFP_NEXT_IMPORT_IMAGE').' : '.$trace, Log::INFO, 'CRON');

		return ($options);
	}
	
	function init_global_variable($id)
	{
		$table	= $this->getTable('Tiffp', 'TiffpTable');
		$table->load($id);
		$this->table = $table;

		$this->path = JPATH_SITE.$this->path_xml.$this->table->g_language.'/';

		$this->tiffp_type_syndication = $this->table->type_xml;
		$this->tiffp_type_format = $this->table->type_format;
		$this->user_id = $this->getCurrentUser()->id;
	}
	
	function getTifXML($id='')
	{
		$params = ComponentHelper::getParams('com_tiffps');

		$this->init_global_variable($id[0]);

		/*****************/
		/* Pasing du XML */
		/*****************/
		$verif = $this->table->verif;
		if (!$verif) { 
			$verif = '.';
		}
		$this->table->verif = $verif;
		$this->extract_file($id);
		
		return true;
	}

	function function_exists($function)
	{
		$return = true;
		$maClass = new TiffpbaseModel();
		$pattern = '|this[^>]+>(.*)\(|U';

		preg_match_all($pattern,$function,$matches, PREG_PATTERN_ORDER);
		
		// die(print_r($matches));
		
		if(array_key_exists(1, $matches)) 
			foreach($matches[1] as $ma_function) {
				if(!method_exists($maClass, $ma_function)) {
					$return = false;
					print_r(Text::sprintf('COM_TIFFPS_FUNCTION_NOT_EXISTS', $ma_function).'<br/>');
				}
			}
		return $return;
	}
	
	function extraction_data_of_file($file, $id_tiffp)
	{
		$params = ComponentHelper::getParams('com_tiffps');
		$update_date_tif = "";
		$user_id = $this->getCurrentUser()->id;
		$tif_id = '';
		
		$this->tiffp_name = $file;

		// crer un objet journaliseur
		if ($this->log)
			Log::addLogger(array('text_file' => 'TifFP'.$file.'.log.php'), Log::ALL, array($this->tiffp_name));

		//rcupre le chemin image associ  la syndication
		$syndication_infos = TiffpsHelper::getTiffps_syndication_info($file);
		$this->extraction_syndication_type = $syndication_infos['syndication_type'];
		$this->extraction_syndication_format = $syndication_infos['syndication_format'];
		$this->tiffps_photos_path = $syndication_infos['tiffps_photos_path'];
		$this->tiffps_fields_separator = $syndication_infos['tiffps_fields_separator'];
		$this->tiffps_values_separator = $syndication_infos['tiffps_values_separator'];

		if ($this->tiffp_type_format == 'xml') {
			// rcupre les donnes du fichier XML
			$tree = new \domDocument();
			$tree->loadXML(TiffpsHelper::getXMLNames($file, 'xml'));
					
			//methode XQuery	
			$this->xpathcontext = new \domXPath($tree);
			
			//vrification si fichier tif3 (prsence de l'ID tif) et de la date de mise  jour de la fiche
			if (@$this->xpathcontext->query("/feed/entry/content/m:properties/d:SyndicObjectID")->length) 
				$tif_id = $this->xpathcontext->query("/feed/entry/content/m:properties/d:SyndicObjectID")->item(0)->nodeValue;
			if ($this->xpathcontext->query("/feed/entry/updated")->length) 
				$update_date_tif = substr($this->xpathcontext->query("/feed/entry/updated")->item(0)->nodeValue, 0, 10);
		} 
		elseif ($this->tiffp_type_format == 'json') {
			$this->json = json_decode(TiffpsHelper::getXMLNames($file, 'json'), true);
			if(empty($this->json)){
				if ($this->log){
					Log::add($file.' : '.Text::sprintf('COM_TIFFP_IMPORT_JSON_IMPOSSIBLE', $file), Log::INFO, $this->tiffp_name);
				}
				$this->setError(Text::sprintf('COM_TIFFP_IMPORT_JSON_IMPOSSIBLE', $file));
			} else {
				$tif_id = $this->json['SyndicObjectID'];
				$update_date_tif = $this->json['Updated'];
			}
		}
		else {
			if ($this->log){
				Log::add($file.' : '.Text::_('COM_TIFFP_FORMAT_EXTRATION_ICONNU'), Log::INFO, $this->tiffp_name);
			}
			return false;
		}
		
		if ($tif_id) {
			if ($this->log){
				Log::add(Text::_('COM_TIFFP_DEBUT_IMPORT').' : '.$file, Log::INFO, $this->tiffp_name);
			}
		} else {
			if ($this->log){
				Log::add($file.' : '.Text::_('COM_TIFFP_FICHIER_SANS_ID_TIF'), Log::INFO, $this->tiffp_name);
			}
			return false;
		}
		
		//test si le lieu existe et rcupre ces donnes.
		$query = "select * from #__gmapfp inner join #__tiffp_gmapfp on id = gmapfp_id where tif_id = '".$tif_id."' and language ='".$this->table->g_language."' ";
		$this->_db->setQuery( $query );
		$old_values = $this->_db->loadObject();
		
		if (is_object($old_values) and property_exists($old_values, 'id') and $old_values->id) {
			//vrifie que l'enregistrement de la table GMapFP n'a pas t supprim
			$query = "select id from #__gmapfp where id = " . (int)$old_values->id;
			$this->_db->setQuery( $query );
			$enregistrement_exist = $this->_db->loadResult();
			//si l'enregistrement n'existe pas surprimer celui de la table tiffp_gmapfp
			if (empty($enregistrement_exist)) {
				$query = 'DELETE FROM #__tiffp_gmapfp WHERE gmapfp_id = '.(int)$old_values->id;		
				$this->_db->setQuery( $query );
				$result = $this->_db->execute();
				//supprime les lments de $old_values
				$old_values = '';
			}
		}

		//ne met  jour que les fichiers plus rcent que ceux de la base
		if ($params->get('import_only_latest_files', 0)) {
			if ($this->log){
				Log::add(Text::sprintf('COM_TIFFPS_UPDATES_DATE', substr($old_values->modified, 0, 10), $update_date_tif), Log::INFO, $this->tiffp_name);
			}
			if (!empty($update_date_tif) and !empty($old_values)) {
				$update_date_gmapfp = @substr($old_values->modified, 0, 10);
				if ($update_date_tif <= $update_date_gmapfp) {
					if ($this->log){
						Log::add(Text::_('COM_TIFFP_OLD_FICHIER'), Log::INFO, $this->tiffp_name);
					}
					return false;
				}
			}
		}

		$data = array();
		
		$GmapfpCustomFields = TiffpsHelper::getGmapfpCustomFields();
		$GMapfpFields = $this->table->getFields();
		$FieldsResult = array_merge($GMapfpFields, $GmapfpCustomFields);
		unset($GmapfpCustomFields, $GMapfpFields);
		$Customs_fields_values = json_decode($this->table->params_champs, true);
		foreach ($Customs_fields_values as $key => $value) {
			$tmp = 'field_'.$key;
			$this->table->$tmp = $value;
		}

		//Pour chaque champ, rcupre les donnes du fichier XML => $data[champ]
		foreach ($FieldsResult as $key => $field) {
			if ( substr($key, 0, 2) == 'g_' or substr($key, 0, 6) == 'field_') {
			//traitement de la valeur du champ
				$resultat = '';
				$gmapfp_field = $key;
				if (substr($gmapfp_field, 0, 2) == 'g_') $gmapfp_field = substr($key, 2);
				$this->field = $gmapfp_field;

				if (property_exists($this->table, $key) and $this->table->$key) {
					if ($gmapfp_field != 'access' and $gmapfp_field != 'language' and self::function_exists($this->table->$key)) {
						$parantheses_ouvrantes = substr_count($this->table->$key, '(');
						$parantheses_fermantes = substr_count($this->table->$key, ')');
						if ($parantheses_ouvrantes == $parantheses_fermantes){
							eval('$resultat = '.$this->table->$key.';');
						} else
							echo Text::_('COM_TIFFP_DEFAUT_NOMBRE_PARANTHESES_PARAMETRE') . $this->table->$key;
					} else {
						$resultat = $this->table->$key;
					}
				}
				
				//mise en forme et stockage du rsultat
				if (empty($resultat)) $resultat = '';
				$data[$gmapfp_field] = $resultat;

				if ($this->log){
					if (property_exists($this->table, $key)) 
						Log::add($key.' : '.$this->table->$key.' : '.json_encode($resultat), Log::INFO, $this->tiffp_name);
					else
						Log::add($key.' : Champ non trait', Log::ERROR, $this->tiffp_name);
				}
			} else {
				if ($this->log){
					Log::add($key.' : Champ  ne pas traiter', Log::ERROR, $this->tiffp_name);
				}
			}
		}

		//si pas de catgorie, ne pas l'crire dans la table GMapFP
		if ($data['catid'] == '') {
			if ($this->log){
					Log::add('No cat id', Log::ERROR, $this->tiffp_name);
			}
			return;
		}
		
		//insertion des donnes dans la table gmapfp $data
		//renseigne la date de mise  jour.
		$data['modified'] = $this->XML_DateHeure();
		$catids = explode(',', $data['catid']);
		$data['catid'] = '['.$data['catid'].']';
		
		if (empty($data['alias']) && empty($data['title'])) $data['alias']='fiche sans nom !!! '.date("Y-m-d H:i:s");
		if (empty($data['title'])) $data['title']='fiche sans nom !!! '.date("Y-m-d H:i:s");

		if ($old_values) {
		//mthode update

			//rcupre les options de traitement
			$options = json_decode($this->table->options);
			$set = array();
			$setField = array();
			$setTag = array();
			// die(print_r($data));
			foreach ($data as $key => $value) {
				//traite tous les champs de la table gmapfp sauf catid, access, modified et language
				if ($key != 'tag' and $key != 'catid' and $key != 'access' and $key != 'language' and $key != 'modified' and substr($key, 0 , 6) != 'field_') {
					$option_field = 0;
					$option_name = 'options_g_'.$key;
					if (isset($options->$option_name))
						$option_field = $options->$option_name;
					$set[$key] = self::setFieldValue($option_field, $key, $value, $old_values);
				//traite tous les champs customs
				} else if (substr($key, 0 , 6) == 'field_') {
					$option_field = 0;
					$option_name = 'options_'.$key;
					if (isset($options->$option_name))
						$option_field = $options->$option_name;
					$setField[substr($key, 6)] = self::setFieldValue2($option_field, $key, $value, $old_values);
				//traite tous le champ tag
				} else if ($key == 'tag') {
					$option_field = 0;
					$option_name = 'options_g_'.$key;
					if (isset($options->$option_name))
						$option_field = $options->$option_name;
					$setTag = self::setTagValue($option_field, $key, $value, $old_values);
				//traite les champs catid, access, modified et language
				} else {
					if ($value!==false) 
						$set[$key] = $this->_db->Quotename($key).' = '.$this->_db->Quote($value);
					else
						$set[$key] = $this->_db->Quotename($key).' = ""'; //vide les champ sans rponce.
				}
			}
			if (array_key_exists('created_by', $set) and $set['created_by'] == "`created_by` = ''")
				$set['created_by'] = "`created_by` = ".$this->_db->Quote($user_id);
			if ($set['publish_up']=="`publish_up` = ''") $set['publish_up'] = "`publish_up` = ".$this->_db->Quote(self::XML_DateHeure());
			if ($set['publish_down']=="`publish_down` = ''") $set['publish_down'] = "`publish_down` = NULL";
			
			//mise  jour de la table gmapfp_catid
				//supprime les anciennes donnes
				//l'insertion est faite plus bas pour tout les cas de figure
			$query = $this->_db->getQuery(true);
			$query->delete($this->_db->quoteName('#__gmapfp_catid'))
				->where($this->_db->quoteName('gmapfp_id') . ' = '.$old_values->id);
			$this->_db->setQuery($query);
			$this->_db->execute();
			
			//mise  jour de la table gmapfp
			$query = 'UPDATE `#__gmapfp` '.
				' SET '.implode(', ', $set).
				' WHERE id = '.$old_values->id;
			unset($set);
			
			if ($this->log){
					Log::add('Requte Update : '.$query, Log::INFO, $this->tiffp_name);
			}
			
			try {
				$this->_db->setQuery( $query );
				$this->_db->execute();
				if ($this->log)
					Log::add('Requette : '.$query, Log::INFO, $this->tiffp_name);
			}
			catch (RuntimeException $e) {
				if ($this->log)
					Log::add('Erreur sur mthode update '.$e->getMessage(), Log::ERROR, $this->tiffp_name);
			}
			//mise  jour de la table field
			self::updateFieldsTable($setField, $old_values->id);
			//mise  jour de la table tag
			self::updateTagsTable($setTag, $old_values->id);
			
			$tif_id_exist = $old_values->id;
		} else {
		//mthode insert
			$set = array();
			$setField = array();
			$setTag = array();
			$data['created'] = $this->XML_DateHeure();
			$data['created_by'] = $this->table->modified_by;
			if (empty($data['publish_up'])) $data['publish_up'] = $this->XML_DateHeure();
			if (empty($data['publish_down'])) $data['publish_down'] = 'NULL';
			$data['attribs'] = '{}';
			$data['state'] = (int)$data['state'];
			// die(print_r($data));
			foreach ($data as $key => $value) {
				if (substr($key, 0 , 6) == 'field_') { //traite les champs custom
					$option_field = 0; //methode insert, donc toujours mettre  jour
					$setField[substr($key, 6)] = self::setFieldValue2($option_field, $key, $value);
				} elseif ($key == 'tag') { //traite le champ tag
					$option_field = 0; //methode insert, donc toujours mettre  jour
					if (!empty($value)) {
						if ($value != 'NULL') $set[$key] = $this->_db->Quote($value);
						else $setTag = self::setTagValue($option_field, $key, $value);
					}
				} else  {
					if (!empty($value)) {
						if ($value != 'NULL') $set[$key] = $this->_db->Quote($value);
						else $set[$key] = $value;
					}
				}
			}
			$mes_key = '`'.implode('`, `',array_keys($set)).'`';
			$mes_values = implode(', ', $set);

			// die(print_r($set));
			$query = 'INSERT INTO `#__gmapfp` '.
				'('.$mes_key.')'.
				'VALUES ('.$mes_values.')';
				// die(print_r($query));
				
			try {	
				$this->_db->setQuery( $query );
				$this->_db->execute();
				
				//rcupre l'id pour mettre  jour la galerie
					// $query = "select id from #__gmapfp where created = '".$data['created']."' and title = ".$this->_db->quote($data['title']);
					// $this->_db->setQuery( $query );
					// $query_result = $this->_db->loadRow();
					// $tif_id_exist = $query_result[0];
				$tif_id_exist = $this->_db->insertid();

				//met  jour la table tiffp_gmapfp
				$query = 'INSERT INTO `#__tiffp_gmapfp` '.
					'(tif_id, gmapfp_id)'.
					'VALUES ("'.$tif_id.'", '.$tif_id_exist.')';
				$this->_db->setQuery( $query );
				$this->_db->execute();		

				//met  jour les champs perso
				self::updateFieldsTable($setField, $tif_id_exist);		
				//met  jour le champ tag
				self::updateTagsTable($setTag, $tif_id_exist);		
			}
			catch (RuntimeException $e) {
				if ($this->log)
					Log::add('Erreur sur mthode insert '.$e->getMessage(), Log::ERROR, $this->tiffp_name);
			}
		}

		//insert les nouvelles donnes de la table gmapfp_catid
		foreach ($catids as $cat_id) {
			$query = $this->_db->getQuery(true);
			$columns = array('gmapfp_id', 'cat_id');
			$values = array($tif_id_exist, $cat_id);
			$query
				->insert($this->_db->quoteName('#__gmapfp_catid'))
				->columns($this->_db->quoteName($columns))
				->values(implode(',', $values));
			$this->_db->setQuery($query);
			$this->_db->execute();
		}

		//met  jour la table des images
		$query = 'UPDATE `#__tiffp_images` '.
			' SET gmapfp_id = '.$tif_id_exist.
			' WHERE gmapfp_id = -1';
		$this->_db->setQuery($query);
		$this->_db->execute();
			
		unset($data, $file);
		return;
	}
	
	private function updateFieldsTable($setField, $item_id)
	{
		// die(print_r($setField));
		foreach ($setField as $key => $value){
			$query = 'DELETE FROM `#__fields_values` '.
				' WHERE `field_id` = '.$key.' and `item_id` = '.$item_id;
			try {
				$this->_db->setQuery( $query );
				$this->_db->execute();
				if ($this->log)
					Log::add('Requette : '.$query, Log::INFO, $this->tiffp_name);
			}
			catch (RuntimeException $e) {
				if ($this->log)
					Log::add('Erreur sur mthode update '.$e->getMessage(), Log::ERROR, $this->tiffp_name);
			}
			
			if ($value != "''" and !empty($value)) {
				$values = array();
				if (is_array($value)) 
					$values = $value;
				else
					$values[] = $value;
				foreach($values as $tmp) {
					
					$tmp = str_replace('__rep__temp__gmapfp__', 'GA'.$item_id, $tmp);
					$query = 'INSERT INTO `#__fields_values` '.
						' (`field_id`, `item_id`, `value`) VALUES ('.$key.','.$item_id.','.$tmp.')';
					try {
						$this->_db->setQuery( $query );
						$this->_db->execute();
						if ($this->log)
							Log::add('Requette : '.$query, Log::INFO, $this->tiffp_name);
					}
					catch (RuntimeException $e) {
						if ($this->log)
							Log::add('Erreur sur mthode update '.$e->getMessage(), Log::ERROR, $this->tiffp_name);
					}
				}
			}
		}
	}
	
	private function updateTagsTable($tags, $item_id)
	{
		//$tag doit tre un array de string!!!
		$outputs = array();
		if (is_array($tags)) 
			foreach ($tags as $tag){
				$outputs[] = (string)$tag;
			}
		//rcupre les infos pour complter la table des tag
		$tag_type_id = TagsHelper::getTypes('objectList', 'com_gmapfp.item');
		$gmapfp_component = $this->getInstance('Item', 'Gmapfp');
		$table = $gmapfp_component->getTable('Item', 'Administrator');

		$table->reset();
		$table->load($item_id);

		$setTagsEvent = \Joomla\CMS\Event\AbstractEvent::create(
			'onTableSetNewTags',
			array(
				'subject'     => $table,
				'newTags'     => $outputs,
				'replaceTags' => true,
			)
		);

		try {
			$table->getDispatcher()->dispatch('onTableSetNewTags', $setTagsEvent);
		} catch (\RuntimeException $e) {
			$this->setError($e->getMessage());

			return false;
		}
	}
	
	private function setFieldValue($option_field, $key, $value, $old_values)
	{
		if ($key == 'state') $value = (int)$value;
		switch ($option_field) {
			case 0: //toujours mettre  jour
				if ($value!==false) 
					$result = '`'.$key.'` = '.$this->_db->Quote($value);
				else
					$result = '`'.$key.'` = ""'; //vide les champ sans rponce.
				break;
			case 1: //ne jamais mettre  jour
				$result = '`'.$key.'` = '.$this->_db->Quote($old_values->$key);
				break;
			case 2: //ne mettre  jour que si destination vide
				if (!$old_values->$key) {
					if ($value!==false) 
						$result = '`'.$key.'` = '.$this->_db->Quote($value);
					else
						$result = '`'.$key.'` = ""'; //vide les champ sans rponce.
				} else {
					$result = '`'.$key.'` = '.$this->_db->Quote($old_values->$key);
				}
				break;
			case 3: // ne mettre  jour que si source non vide
				if ($value!==false) 
					$result = '`'.$key.'` = '.$this->_db->Quote($value);
				else
					$result = '`'.$key.'` = '.$this->_db->Quote($old_values->$key); //vide les champ sans rponce.
				//break;
		}
		return $result;
	}
	
	private function setFieldValue2($option_field, $key, $value, $old_values = array())
	{
		$result = '""';
		switch ($option_field) {
			case 0: //toujours mettre  jour
				if ($value!==false) 
					$result = $this->_db->Quote($value);
				else
					$result = '""'; //vide les champ sans rponce.
				break;
			case 1: //ne jamais mettre  jour
				if (!empty($old_values) and property_exists($old_values, $key)) 
					$result = $this->_db->Quote($old_values->$key);
				break;
			case 2: //ne mettre  jour que si destination vide
				if (!empty($old_values) and property_exists($old_values, $key) and !$old_values->$key) {
					if ($value!==false) 
						$result = $this->_db->Quote($value);
				} else {
					$result = $this->_db->Quote($old_values->$key);
				}
				break;
			case 3: // ne mettre  jour que si source non vide
				if ($value!==false) 
					$result = $this->_db->Quote($value);
				elseif (!empty($old_values) and property_exists($old_values, $key) and !$old_values->$key)
					$result = $this->_db->Quote($old_values->$key); //vide les champ sans rponce.
				//break;
		}
		return $result;
	}

	private function setTagValue($option_field, $key, $value, $old_values = array())
	{
		$result = '""';
		switch ($option_field) {
			case 0: //toujours mettre  jour
				if ($value!==false) 
					$result = $value;
				else
					$result = '""'; //vide les champ sans rponce.
				break;
			case 1: //ne jamais mettre  jour
				if (!empty($old_values) and property_exists($old_values, $key)) 
					$result = $old_values->$key;
				break;
			case 2: //ne mettre  jour que si destination vide
				if (!empty($old_values) and property_exists($old_values, $key) and !$old_values->$key) {
					if ($value!==false) 
						$result = $value;
				} else {
					$result = $old_values->$key;
				}
				break;
			case 3: // ne mettre  jour que si source non vide
				if ($value!==false) 
					$result = $value;
				elseif (!empty($old_values) and property_exists($old_values, $key) and !$old_values->$key)
					$result = $old_values->$key; //vide les champ sans rponce.
				//break;
		}
		return $result;
	}

// stocke les infos des images  rcuprer en table.
	function stocke_infos_image($file_name, $user_id, $main_images, $options) {
		$config		= ComponentHelper::getParams('com_tiffps');
        $type_image = array(".gif",".jpg",".jpeg",".png",".bmp",".webp"); 
		if ($main_images) {
			$folder 	= 'images/gmapfp/'.$user_id.'__';
			$gmapfp_id = 0;
		} else {
			$folder 	= 'images/gmapfp/__rep__temp__gmapfp__/';
			$gmapfp_id = -1;
		}
		
		// Make the filename safe
		$file['name']	= strtolower(basename($file_name));
		
        // Test si extension ok
		$ext = strrchr($file_name,'.');
        $ext = strtolower($ext);
        if (!in_array( $ext, $type_image )) 
        {
			if ($this->log){
				Log::add(Text::_('TIFFP_UPLOAD_IMG_BAD_EXT'), Log::WARNING, $this->tiffp_name);
			}
			return;
        }
		
		if (strlen($file_name) > 0){
			$query = $this->_db->getQuery(true);
			$source = $this->_db->quote($file_name);
			$destination = $this->_db->quote(JPATH_ROOT.'/'.$folder.$file['name']);
			$attribs = $this->_db->quote(json_encode($options));
			$columns = array('url_source', 'url_destination', 'gmapfp_id', 'attribs');
			$values = array($source, $destination, $gmapfp_id, $attribs);
			$query
				->insert($this->_db->quoteName('#__tiffp_images'))
				->columns($this->_db->quoteName($columns))
				->values(implode(',', $values));
			$this->_db->setQuery($query);
			$this->_db->execute();
		}
		
		return $folder.$file['name'];
	}

//rcupre une image de sur le serveur distant.
	public function import_images($cron = false) {
		$params		= ComponentHelper::getParams('com_tiffps');
		$folder 	= JPATH_ROOT.'/images/gmapfp/';
		$loaderror 	= false;
		$errorMsg	= '';
		
		if(!$cron) {
			$app	= Factory::getApplication();
			$jinput = $app->input;
			$nbre_images   = $jinput->get('nbre_images', '', 'INT');
			if (!$nbre_images) {
				$query = $this->_db->getQuery(true);
				$query
					->select('COUNT(*)')
					->from($this->_db->quoteName('#__tiffp_images'));
				$this->_db->setQuery($query);
				$nbre_images = $this->_db->loadResult();
			}
			$reste_image   = $jinput->get('reste_image', $nbre_images, 'INT');
		}
		
		$largeur = $params->get('tiffps_file_img_largeur', 800);
		$hauteur = $params->get('tiffps_file_img_hauteur', 600);
		$crop = $params->get('tiffps_file_img_crop', 1);
		$redim = $params->get('tiffpd_file_img_redim', 0);
		
		// Set FTP credentials, if given
		ClientHelper::setCredentialsFromRequest('ftp');

		$query = $this->_db->getQuery(true);
		$query
			->select('*')
			->from($this->_db->quoteName('#__tiffp_images'));
		$this->_db->setQuery($query);
		$image = $this->_db->loadObject();
		
		if ($image) {
			$options = json_decode($image->attribs, true);
			if (is_array($options)){
				if(array_key_exists('largeur', $options)) 		$largeur = $options['largeur'];
				if(array_key_exists('hauteur', $options)) 		$hauteur = $options['hauteur'];
				if(array_key_exists('crop', $options)) 			$crop = $options['crop'];
				if(array_key_exists('redim', $options)) 		$redim = $options['redim'];
			}
			if($image->gmapfp_id) {
				$destination = str_replace('__rep__temp__gmapfp__', 'GA'.$image->gmapfp_id, $image->url_destination);
			} else {
				$destination = $image->url_destination;
			}
			if($params->get('tiffps_remplace_file_img', 0) and is_file($destination)){
				if ($this->log){
					Log::add(Text::_('TIFFP_UPLOAD_IMG_REMPLACE'), 'Erase : '.$destination, $this->tiffp_name);
				}
				File::delete($destination);
			}
			if(!is_file($destination)) {
				//crer le rpertoire s'il n'existe pas
				$repertoire = dirname($destination);
				if (!is_dir($repertoire))
					Folder::create($repertoire);
				if (!$redim) {
					if ($this->log){
						Log::add(Text::_('TIFFP_UPLOAD_IMG_REDIM').' Non', $this->tiffp_name);
					}
					if (!copy($image->url_source, $destination)) {
						// Error in upload
						if ($this->log){
							Log::add(Text::_('TIFFP_UPLOAD_TRANFERT_NOK'), Log::ERROR, $this->tiffp_name);
						}
						return;
					}
				} else {
					if ($this->log){
						Log::add(Text::_('TIFFP_UPLOAD_IMG_REDIM').' Oui : '.$largeur.'_'.$hauteur.'_'.$crop, $this->tiffp_name);
					}
					copy($image->url_source, $destination);
					$mon_image = new Image($destination);
					
					self::fixOrientation($mon_image);
					if ($crop)
						$mon_image->cropResize($largeur, $hauteur, false);
					else
						$mon_image->resize($largeur, $hauteur, false);
					$mon_image->toFile($destination);
					
				}
			} else $loaderror=true;		
			
			if($loaderror) {
				if ($this->log){
					Log::add(Text::_('TIFFP_UPLOAD_IMG_NOK'), Log::ERROR, $this->tiffp_name);
				}
			}

			//supprime l'enregistrement
			$query = $this->_db->getQuery(true);
			$query->delete($this->_db->quoteName('#__tiffp_images'))
				->where($this->_db->quoteName('id_table') . ' = '.$image->id_table);
			$this->_db->setQuery($query);
			$this->_db->execute();
			
			if (!$cron)
				TiffpsHelper::getProcessImage($image->url_source, $reste_image - 1, $nbre_images);
			else
				return true;
		}
		return false;
	}
	
	public static function getOrientation($path)
	{
		if (!$exif = @exif_read_data($path))
		{
			return;
		}

		return intval(@$exif['Orientation']);
	}

	public static function fixOrientation(&$image, $orientation = null)
	{
		$orientation = self::getOrientation($image->getPath());
		
		if(!in_array($orientation, [3, 6, 8]))
		{
			return;
		}

		switch ($orientation)
		{
            case 3:
                $image->rotate(180, -1, false);
                break;

            case 6:
                $image->rotate(270, -1, false);
                break;

            case 8:
                $image->rotate(90, -1, false);
                break;
		}

		return true;
	}

	public static function getImageType($mime)
	{
		switch ($mime)
		{
			case 'image/png':
				return IMAGETYPE_PNG;
				break;
			case 'image/gif':
				return IMAGETYPE_GIF;
				break;
			case 'image/jpeg':
			default:
				return IMAGETYPE_JPEG;
				break;
		}
	}

	function suppr_accents($str, $encoding='utf-8'){
		// transformer les caractres accentus en entits HTML
		$str = htmlentities($str, ENT_NOQUOTES, $encoding);
	 
		// remplacer les entits HTML pour avoir juste le premier caractres non accentus
		// Exemple : "&ecute;" => "e", "&Ecute;" => "E", "" => "a" ...
		$str = preg_replace('#&([A-za-z])(?:acute|grave|cedil|circ|orn|ring|slash|th|tilde|uml);#', '\1', $str);
	 
		// Remplacer les ligatures tel que : ?,  ...
		// Exemple "" => "oe"
		$str = preg_replace('#&([A-za-z]{2})(?:lig);#', '\1', $str);
		// Supprimer tout le reste
		$str = preg_replace('#&[^;]+;#', '', $str);
	 
		return $str;
	}
}
// {"items":[{"caption":"FP","thumbnail":"media\/acfgallery\/tmp\/FP_thumb.png","image":"media\/acfgallery\/tmp\/FP.png"},{"caption":"Hono","thumbnail":"media\/acfgallery\/tmp\/Hono-marcilly-2020-B4_thumb.jpg","image":"media\/acfgallery\/tmp\/Hono-marcilly-2020-B4.jpg"}]}