<?php

	/*
	* TifFP composant passerelel de TourInFrance vers GMapFP pour Joomla! 3.x
	* Version J4.5
	* Creation date: Juin 2024
	* Author: Fabrice4821 - www.gmapfp.org
	* Author email: webmaster@gmapfp.org
	* License GNU/GPL
	*/

namespace Joomla\Component\Tiffps\Administrator\Model;

\defined('JPATH_PLATFORM') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Filter\OutputFilter;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Log\Log;
use Joomla\CMS\Date\Date;

trait TiffpsJSONTrait
{
	function JSON_value($options) {
		$path = $options;
		$qte_max = 0;
		$result = array();
		$separator = '';
		$filter = '';
		$format = '';
		
		if (is_array($options)){
			if(array_key_exists('path', $options)) 		$path = $options['path'];
			if(array_key_exists('format', $options)) 	$format = $options['format'];
			if(array_key_exists('filter', $options)) 	$filter = $options['filter'];
			if(array_key_exists('qte_max', $options)) 	$qte_max = (int)$options['qte_max'];
		}
		if (empty($path)) {
			Log::add(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field, Log::ERROR, 'field');
		} else {
			$way = self::check_json_path($path);
			if ( $way !== false) {
				//lecture et mise au format d'une valeur
				if (empty($format)) $format = '%s';
				$value = $way;
				if ($qte_max and strlen($value) > $qte_max) $value = substr($value, 0, $qte_max).'...';
				if ($value and $filter) {
					if (strpos($value, $filter) === false) $value = null;
				}
				if (is_array($value)) $value = json_encode($value, true);
				if ($value) {
					$result[] = sprintf($format, nl2br((string)$value)); //lecture de toutes les valeurs contenue en dessous
				}
			}
		}
		return implode( $separator, $result);
	}
	
	function JSON_alias_value($options) {
		$result = array();
		$path 	= $options;
		if (is_array($options)){
			if(array_key_exists('path', $options)) 		$path = $options['path'];
		}

		if (empty($path)) {
			if (defined('JDEBUG') && JDEBUG){
				Log::add(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field, Log::ERROR, 'field');
			}
		} else {
			$way = self::check_json_path($path);
			if ( $way !== false) {
				$result = OutputFilter::stringURLSafe($way);
				if(trim(str_replace('-','',$result)) == '') {
					$this->alias = HtmlHelper::date('now', Text::_('DATE_FORMAT_CALENDAR_DATETIME')); //%Y-%m-%d %H:%M:%S
				}
			}
		}
		return $result;
	}

// Récupère la date et heure courante ou met au format sql une date heure
	function JSON_DateHeure($options) {
		$thedate = new Date('now');
		if (!empty($options)){
			$path = $options;
			$format_entree = '';
			$y = array(0,4);
			$m = array(5,2);
			$d = array(8,2);
			$h = array(11,2);
			$i = array(14,2);
			$s = array(17,2);
			if (is_array($options)) {
				if(array_key_exists('path', $options)) $path = $options['path'];
				if(array_key_exists('y', $options)) $y = $options['y'];
				if(array_key_exists('m', $options)) $m = $options['m'];
				if(array_key_exists('d', $options)) $d = $options['d'];
				if(array_key_exists('h', $options)) $h = $options['h'];
				if(array_key_exists('i', $options)) $i = $options['i'];
				if(array_key_exists('s', $options)) $s = $options['s'];
			}
			if (empty($path)) {
				if (defined('JDEBUG') && JDEBUG){
					Log::add(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field, Log::ERROR, 'field');
				}
			} else {
				$way = self::check_json_path($path);
				if (!empty($way)) {
					$annee = substr($way, $y[0], $y[1]);
					$mois = substr($way, $m[0], $m[1]);
					$jour = substr($way, $d[0], $d[1]);
					$heure = substr($way, $h[0], $h[1]);
					$minute = substr($way, $i[0], $i[1]);
					$seconde = substr($way, $s[0], $s[1]);
					$thedate = new Date($annee.$mois.$jour.'T'.$heure.$minute.$seconde.'Z');
				} else return '';
			}
		}
		return $thedate->toSQL();
	}

	/******************************/
	// fonction JSON_Syndication
	// récupère une info de l'utilisateur courant.
	//
	// paramètres : 
	//
	// info : id / name / username / email 
	//
	// exemple de saisie :
	//		$this->JSON_UserID();
	//		$this->JSON_UserID("name");
	//		$this->JSON_UserID("email");
	//
	/******************************/
	function JSON_UserID($info="id") {
		$user = Factory::getUser();

		return $user->$info;
	}

	function JSON_main_img($options)
	{
		$user_id 		= $this->user_id;
		$default_file 	= '';
		$position 		= 0;
		$alt 			= '';
		$caption 		= '';
		$no_image 		= true;
		$tmp 			= '';
		
		$path = $options;
		if (is_array($options)){
			if(array_key_exists('path', $options)) 		$path = $options['path'];
			if(array_key_exists('user_id', $options)) 	$user_id = $options['user_id'];
			if(array_key_exists('default_file', $options)) $default_file = $options['default_file'];
			if(array_key_exists('alt', $options)) 		$alt = $options['alt'];
			if(array_key_exists('caption', $options)) 	$caption = $options['caption'];
		}

		if ($this->tiffps_photos_path == '' and defined('JDEBUG') && JDEBUG) {
			print_r(Text::_('COM_TIFFP_ERREUR_REPERTOIRE_IMG_NON_CONFIG').'<br/>');
		}
		//lecture et mise au format d'une valeur
		if (empty($path)) {
			if (defined('JDEBUG') && JDEBUG){
				Log::add(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field, Log::ERROR, 'field');
			}
		} else  {
			$way = self::check_json_path($path);
			if ($way !== false and is_array($way)) {
				if (array_key_exists('Url', $way)) {
					if(strstr($way['Url'], 'http') === false) 
						$this->stocke_infos_image($this->tiffps_photos_path.$way['Url'], $user_id, true, $options);
					else
						$this->stocke_infos_image($way['Url'], $user_id, true, $options);
					
					$json['image'] = $this->path_main_images.strtolower($user_id.'__'.basename($way['Url']));
					if ($alt) 
						$json['image_alt'] = $alt;
					elseif (array_key_exists('Titre', $way))
						$json['image_alt'] = basename($way['Titre']);
					else
						$json['image_alt'] = basename($way['Url']);
					if ($caption)
						$json['image_caption'] = $caption;
					else
						$json['image_caption'] = '';
					$tmp = json_encode($json);
					$no_image = false;
				}
			}
			if ($no_image) {
				if (defined('JDEBUG') && JDEBUG){
					Log::add(Text::_('TIFFP_LECTURE_FIELD').' : '.$this->field, Log::NOTICE, 'field');
				}
				if ($default_file) $tmp = $default_file;
			}
		}
		return $tmp;
	}
	
	/******************************/
	// fonction JSON_img
	// récupère des images
	// en tand qu'image de galerie
	//
	// paramètres : 
	//	path => chemin JSON
	//  format => format du fichier de recherche : tourinfranceV3 / intramuros / datatourisme
	//	first => traite aussi la première image (qui sert généralement d'image principale)
	//		ex : $this->JSON_img(array('path'=>"/feed/entry/content/m:properties/d:DetailPhotos_DIAPO",'first'=>0));
	//
	/******************************/
	function JSON_img($params) {
		$config	= ComponentHelper::getParams('com_tiffps');
		
		$qte_max 	= 0;
		$cn 		= 0;
		$good_image = 0;
		$first		= true;
		$path 		= '';
		$langue		= 'fr';
		$result		= array();
		
		if (is_array($params)){
			if(array_key_exists('path', $params)) 		$path = $params['path'];
			if(array_key_exists('first', $params)) 		$first = $params['first'];
			if(array_key_exists('qte_max', $params)) 	$qte_max = $params['qte_max'];
		}
		
		if (empty($path)) {
			if (defined('JDEBUG') && JDEBUG){
				Log::add(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field, Log::ERROR, 'field');
			}
		} else  {
			$photos = self::check_json_path($path);
			if ( $photos !== false) {
				foreach ($photos as $photo) {
					$titre = '';
					$credit = '';
					$url = '';
					if ($first or $cn) { // pour ne pas traiter la première image si paramètré
						switch ($this->extraction_syndication_type) {
							case 3 : //intramuros
								if (isset($photo['Url'])) $url = $photo['Url'];
								break;
							case 2 : //datatourisme
								if (isset($photo['ebucore:hasAnnotation'][0]['ebucore:title'][$langue])) $titre = $photo['ebucore:hasAnnotation'][0]['ebucore:title'][$langue];
								if (isset($photo['ebucore:hasAnnotation'][0]['credits'][0])) $credit = $photo['ebucore:hasAnnotation'][0]['credits'][0];
								if (isset($photo['ebucore:hasRelatedResource'][0]['ebucore:locator'][0])) $url = $photo['ebucore:hasRelatedResource'][0]['ebucore:locator'][0];
								break;
							default : //tourinfrance
								if (isset($photo['Photo']['Titre'])) $titre = $photo['Photo']['Titre'];
								if (isset($photo['Photo']['Credit'])) $credit = $photo['Photo']['Credit'];
								if (isset($photo['Photo']['Url'])) $url = $photo['Photo']['Url'];
						}
						if (strpos($url, 'http') !== 0)
							$url = $config->get('tiffps_photos_path').$url;
						if ($return_upload_image = $this->stocke_infos_image($url, '', false, $params)) {
							$good_image ++;
							$data_photo['titre'] = $titre;
							$data_photo['credit'] = $credit;
							$data_photo['url'] = $return_upload_image;
							$result[] = $data_photo;
						}
						if (defined('JDEBUG') && JDEBUG)
							Log::add(Text::_('TIFFP_LECTURE_FIELD').' : '.json_encode($photo), Log::INFO, 'thumbs');
					}
					$cn++;
					if ($qte_max and $qte_max<=$good_image) break;
				}
			}
		}
		return $result;
	}
	
	/******************************/
	// fonction JSON_ACF_img
	// récupère des images
	// et renseigne le champ au format de ACF image :
	// {"items":[{"caption":"FP","thumbnail":"media\/acfgallery\/tmp\/FP_thumb.png","image":"media\/acfgallery\/tmp\/FP.png"},{"caption":"Hono","thumbnail":"media\/acfgallery\/tmp\/Hono-marcilly-2020-B4_thumb.jpg","image":"media\/acfgallery\/tmp\/Hono-marcilly-2020-B4.jpg"}]}
	//
	// paramètres : 
	//	path 	=> chemin json
	//  caption => caption de l'image
	//  alt => alt de l'image
	//  source => source de l'image
	//  tags => tags de l'image	
	//  format 	=> format du fichier de recherche : tourinfranceV3 / intramuros / datatourisme
	//	first 	=> traite aussi la première image (qui sert généralement d'image principale)
	//		ex : $this->JSON_ACF_img(array('path'=>"/feed/entry/content/m:properties/d:LABELS"));
	//
	// Traite les champs de type <d:LABELS>&lt;img alt="Qualité Tourisme" title="Qualité Tourisme" src="ht
	/******************************/
	public function JSON_ACF_img($params)
	{
		$captions 	= array();
		$alt 	= array();
		$source 	= array();
		$tags 	= array();
		$info_img 	= array();
		$return 	= array();
		$qte_max	= 0;
		$first		= true;
		
		if (is_array($params)){
			if(array_key_exists('path', $params)) 		$path = $params['path'];
			if(array_key_exists('caption', $params)) 	$captions = $params['caption'];
			if(array_key_exists('alt', $params)) 		$captions = $params['alt'];
			if(array_key_exists('source', $params)) 	$captions = $params['source'];
			if(array_key_exists('tags', $params)) 		$captions = $params['tags'];
			if(array_key_exists('first', $params)) 		$first = $params['first'];
			if(array_key_exists('qte_max', $params)) 	$qte_max = $params['qte_max'];
		}
		
		$imgs = $this->JSON_img($params);
		$i = 0;
		foreach($imgs as $img) {
			if(is_array($captions) and array_key_exists($i, $captions))
				$caption_text = $captions[$i];
			elseif ($img['titre'])
				$caption_text = $img['titre'];
			else
				$caption_text = '';
			if(is_array($alt) and array_key_exists($i, $alt))
				$alt_text = $alt[$i];
			elseif ($img['titre'])
				$alt_text = $img['titre'];
			else
				$alt_text = '';
			if(is_array($source) and array_key_exists($i, $source))
				$source_text = $source[$i];
			elseif ($img['credit'])
				$source_text = $img['credit'];
			else
				$source_text = '';
				
			$info_img[] = array('alt'=>$alt_text, 'source'=>$source_text, 'tags'=>[], 'caption'=>$caption_text, 'thumbnail'=>$img['url'], 'image'=>$img['url']);
			$i++;
		}
		if (!empty($info_img))
			$return = array('items'=>$info_img);
			
		if (empty($return))
			return '';
		else
			return json_encode($return);
	}
	
	//récupère les valeurs d'un tableau
	// si les paramètres valeurs_de et valeurs_a sont renseignés, fait la conversion des données.
	// si la valeur n'est pas trouvée dans la conversion des données, elle est affichée tel quelle.
	// renvoie un délément dans le field avec un 'implode' des éléments trouvés dans le path
	protected function JSON_array_values ($options)
	{
		$path 			= '';
		$search_value 	= '';
		$path_before 	= '';
		$label_value 	= '';
		$debut			= '';
		$separateur 	= ', ';
		$fin			= '';
		$result 		= array();
		$conteneur 		= '';
		$valeurs_de		= array();
		$valeurs_a		= array();
		
		if (is_array($options)){
			if(array_key_exists('path', $options)) 			$path = $options['path'];
			if(array_key_exists('conteneur', $options)) 	$conteneur = $options['conteneur'];
			if(array_key_exists('debut', $options)) 		$debut = $options['debut'];
			if(array_key_exists('separateur', $options)) 	$separateur = $options['separateur'];
			if(array_key_exists('fin', $options)) 			$fin = $options['fin'];
			if(array_key_exists('valeurs_de', $options)) 	$valeurs_de = $options['valeurs_de'];
			if(array_key_exists('valeurs_a', $options)) 	$valeurs_a = $options['valeurs_a'];
		}
		
		if (count($valeurs_de) != count($valeurs_a)) {
			if (defined('JDEBUG') && JDEBUG){
				Log::add(Text::_('TIFFP_DEFAUT_COHERENCE_VALEUR_DE_A').' : '.$this->field, Log::NOTICE, 'field');
			}
			return;
		}
		
		if (!$conteneur) {
			if (defined('JDEBUG') && JDEBUG){
				Log::add(Text::_('TIFFP_DEFAUT_JSON_ARRAY_VALUES_NO_CONTENEUR').' : '.$this->field, Log::NOTICE, 'field');
			}
			return;
		}

		$way = self::check_json_path($path);
		if ( $way !== false and is_array($way)) {
			$values = array();
			foreach ($way as $key => $datas) {
				if (is_array($datas)) {
					$values = self::get_array_key_value($datas, $conteneur, $values);
				} elseif ($key == $conteneur)
					$values[] = $datas;
			}
			
			if ($values and count($valeurs_de)>=1) {
				foreach ($values as $value) {
					$index = array_search($value, $valeurs_de);
					if ($index !== false) {
						$result[] = $valeurs_a[$index];
					}
				}
			} else
				$result = $values;
		}
		if (!empty($result)) { 
			$result = array_unique($result);
			$result = array_filter($result);
			$result = $debut.implode($separateur, $result).$fin;
			return $result;
		}
		return '';
	}
	
	static protected function get_array_key_value($array, $conteneur, $result)
	{
		if(is_array($array)) {
			foreach ($array as $key => $value) {
				if (is_array($value)) {
					$result = self::get_array_key_value($value, $conteneur, $result);
				} elseif ($key == $conteneur) {
					$result[] = $value;
				}
			}
			return $result;
		}
	}
	
	//récupère la valeur du premier champ lié à une recherche
	protected function JSON_search_value ($options)
	{
		$path 			= '';
		$search_value 	= '';
		$label_value 	= '';
		$search_key 	= '';
		$pop			= 0; //nombre délément à dépiler avant d'avoir le bon niveau pour le label
		$response_number= 0; //renvoi la première valeur trouvée.
		
		if (is_array($options)){
			if(array_key_exists('path', $options)) 			$path = $options['path'];
			if(array_key_exists('search_value', $options)) 	$search_value = $options['search_value'];
			if(array_key_exists('search_key', $options)) 	$search_key = $options['search_key'];
			if(array_key_exists('label_value', $options)) 	$label_value = $options['label_value'];
			if(array_key_exists('pop', $options)) 			$pop = $options['pop'];
			if(array_key_exists('response_number', $options))$response_number = $options['response_number'];
		}
		
		$way = self::check_json_path($path);

		$result = self::find_array_value($way, $search_key, $search_value);

		if ($result and isset($result[$response_number])) {
			$full_path = $result[$response_number];
			for ($i = 0; $i <= $pop; $i++) {
				array_pop($full_path);
			}
			foreach($full_path as $key) {
				$way = $way[$key];
			}
			if (array_key_exists($label_value, $way)) {
				return $way[$label_value];
			} else {
				return;
			}
		}
		return;
	}
	
	/******************************/
	// fonction JSON_array_custom_field()
	// Récupérer les éléments d’une suite de valeur dans un champ de syndication
	// et remplace leur valeur par la valeur correspondante
	//
	// valeurs_de : array de valeurs lues
	// valeurs_a : array de valeurs retranscrites en correspondance des valeurs lues
	//
	// index en cas de reponse multiple: 0 est la première valeur
	//
	// exemple de saisie :
	//		$this->JSON_array_custom_field(array('path'=>'ClASs', 'clef'=>'ThesLibelle', 'lues'=>array("1 étoiles", "2 étoiles"), 'retranscrites'=>array(19, 20)));
	//
	// renvoie autant délément dans le field_value qu'il n'y a d'éléments trouvés dans le path
	/******************************/
	function JSON_array_custom_field($params) {
		$path 		= '';
		$conteneur 	= '';
		$valeurs_de = array();
		$valeurs_a 	= array();
		$result 	= array();

		if (is_array($params)){
			if(array_key_exists('path', $params)) 			$path = $params['path'];
			if(array_key_exists('conteneur', $params)) 		$conteneur = $params['conteneur'];
			if(array_key_exists('valeurs_de', $params)) 	$valeurs_de = $params['valeurs_de'];
			if(array_key_exists('valeurs_a', $params)) 		$valeurs_a = $params['valeurs_a'];
		}

		//lecture et mise au format d'une valeur
		if (is_null($way = self::check_json_path($path))) {
			if (defined('JDEBUG') && JDEBUG){
				Log::add(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field, Log::NOTICE, 'field');
			}
			return;
		}

		if (!$conteneur) {
			if (defined('JDEBUG') && JDEBUG){
				Log::add(Text::_('TIFFP_DEFAUT_JSON_ARRAY_VALUES_NO_CONTENEUR').' : '.$this->field, Log::NOTICE, 'field');
			}
			print_r(Text::_('TIFFP_DEFAUT_JSON_ARRAY_VALUES_NO_CONTENEUR').' : '.$this->field.'<br>');
			return $result;
		}

		if (count($valeurs_de) != count($valeurs_a)) {
			if (defined('JDEBUG') && JDEBUG){
				Log::add(Text::_('TIFFP_DEFAUT_COHERENCE_VALEUR_DE_A').' : '.$this->field, Log::NOTICE, 'field');
			}
			return;
		}

		if ($way) {
			foreach($way as $key => $info){
				$value_lue = $this->get_champ_value($info, $key, $conteneur);
				if($value_lue)
					if (is_array($valeurs_de) and is_array($valeurs_a) and count($valeurs_de) > 0 and count($valeurs_de) == count($valeurs_a)) {
						$index = array_search($value_lue, $valeurs_de);
						if ($index !== false) {
							$result[] = $valeurs_a[$index];
						}
					} else {
						$result[] = $value_lue;
					}
			}
		} elseif (defined('JDEBUG') && JDEBUG){
			Log::add(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field, Log::ERROR, 'field');
		}

		return array_unique($result);
	}

	function JSON_ouvertures_TIF_V3($params = array()) {
		$path 			= 'OUVERTUREs';
		$text_du_au 	= '<div class="label_periode">Période du %s au %s</div>';
		$label_ouverture ='<label class="tiffp_ouverture">Ouverture :</label>';
		$label_fermeture ='<label class="tiffp_fermeture">Fermeture :</label>';
		$debut_fermeture = '<ul class="fermetures"><li>';
		$implode_fermeture = '-';
		$fin_fermeture	= '</il></ul>';
		$format_date 	= 'd-m-Y';
		$text_le 		= 'Le %s';
		$text_de_a		= ' de %s à %s';
		$text_a			= ' à ';
		$text_et_liaison_jours = ' et ';
		$debut_jours_ouverture = '<ul class="jours_ouverutre"><li>';
		$separateur_jours_ouverture = '</li><li>';
		$fin_jours_ouverture = '</li></ul>';
		$text_enveloppe_ouverture = '';
		$text_enveloppe_periode = '<div class="periode">%s</div>';
		$text_enveloppe = '<div class="ouvertures_fermetures">%s</div>';
		$periodes		= '';
		$result 		= '';
		$show_periode = true;
		$show_jour_ouverture = true;
		$show_jour_fermeture = true;
		$cn = 0;

		if (is_array($params)){
			if(array_key_exists('path', $params)) 				$path = $params['path'];
			if(array_key_exists('text_du_au', $params)) 		$text_du_au = $params['text_du_au'];
			if(array_key_exists('label_ouverture', $params)) 	$label_ouverture = $params['label_ouverture'];
			if(array_key_exists('label_fermeture', $params)) 	$label_fermeture = $params['label_fermeture'];
			if(array_key_exists('debut_fermeture', $params)) 	$debut_fermeture = $params['debut_fermeture'];
			if(array_key_exists('implode_fermeture', $params)) 	$implode_fermeture = $params['implode_fermeture'];
			if(array_key_exists('fin_fermeture', $params)) 		$fin_fermeture = $params['fin_fermeture'];
			if(array_key_exists('format_date', $params)) 		$format_date = $params['format_date'];
			if(array_key_exists('text_le', $params)) 			$text_le = $params['text_le'];
			if(array_key_exists('text_de_a', $params)) 			$text_de_a = $params['text_de_a'];
			if(array_key_exists('text_a', $params)) 			$text_a = $params['text_a'];
			if(array_key_exists('text_et_liaison_jours', $params)) 		$text_et_liaison_jours = $params['text_et_liaison_jours'];
			if(array_key_exists('debut_jours_ouverture', $params)) 		$debut_jours_ouverture = $params['debut_jours_ouverture'];
			if(array_key_exists('separateur_jours_ouverture', $params)) $separateur_jours_ouverture = $params['separateur_jours_ouverture'];
			if(array_key_exists('fin_jours_ouverture', $params)) 		$fin_jours_ouverture = $params['fin_jours_ouverture'];
			if(array_key_exists('text_enveloppe_ouverture', $params)) 	$text_enveloppe_ouverture = $params['text_enveloppe_ouverture'];
			if(array_key_exists('text_enveloppe_periode', $params)) 	$text_enveloppe_periode = $params['text_enveloppe_periode'];
			if(array_key_exists('text_enveloppe', $params)) 			$text_enveloppe = $params['text_enveloppe'];
			if(array_key_exists('show_periode', $params)) 				$show_periode = $params['show_periode'];
			if(array_key_exists('show_jour_ouverture', $params)) 		$show_jour_ouverture = $params['show_jour_ouverture'];
			if(array_key_exists('show_jour_fermeture', $params)) 		$show_jour_fermeture = $params['show_jour_fermeture'];
		}
		//lecture et mise au format d'une valeur
		if (is_null($way = self::check_json_path($path)))
			print_r(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field.'<br/>');

		if ($way) {
			$ordre; $Datedebut; $Datefin; $Joursfermetures;
			foreach($way as $detail_periode){
				$cn++;
				$text = '';
				$text_ouverture = '';
				if (array_key_exists('Ordre', $detail_periode)) $ordre = $detail_periode['Ordre'];
				if (array_key_exists('Datedebut', $detail_periode) and !empty($detail_periode['Datedebut'])) $Datedebut = new \DateTime($detail_periode['Datedebut']);
				if (array_key_exists('Datefin', $detail_periode) and !empty($detail_periode['Datefin'])) $Datefin = new \DateTime($detail_periode['Datefin']);
				if ($show_periode and isset($Datedebut) and $Datedebut)
					if ($Datedebut == $Datefin)
						$text .= sprintf($text_le, $Datedebut->format($format_date));
					else
						$text .= sprintf($text_du_au, $Datedebut->format($format_date), $Datefin->format($format_date));
				$jours = array('lundi','mardi','mercredi','jeudi','vendredi','samedi','dimanche');
				$jours_traduction = array('lundi'=>'MONDAY','mardi'=>'TUESDAY','mercredi'=>'WEDNESDAY','jeudi'=>'THURSDAY','vendredi'=>'FRIDAY','samedi'=>'SATURDAY','dimanche'=>'SUNDAY');
				$jours_ouvertures = array();
				foreach ($jours as $jour) {
					$heures_ouvertures = array();
					for ($i = 1; $i<=20; $i++) {
						$HeureDebut = '';
						$HeureFin = '';
						if (array_key_exists($jour.'heuredebut'.$i, $detail_periode) and !empty($detail_periode[$jour.'heuredebut'.$i])) $HeureDebut = substr($detail_periode[$jour.'heuredebut'.$i], 0, 5);
						if (array_key_exists($jour.'heurefin'.$i, $detail_periode) and !empty($detail_periode[$jour.'heurefin'.$i])) $HeureFin = substr($detail_periode[$jour.'heurefin'.$i], 0, 5);
						if ($HeureDebut)
							if (empty($HeureFin) or $HeureDebut == $HeureFin) {
								$heures_ouvertures[] .= $text_a.$HeureDebut;
							} else {
								$heures_ouvertures[] .= sprintf($text_de_a, $HeureDebut, $HeureFin);
							}
					}
					if ($heures_ouvertures)
						$jours_ouvertures[] .= Text::_($jours_traduction[$jour]).implode($text_et_liaison_jours, $heures_ouvertures);
				}
				if ($show_jour_ouverture and $jours_ouvertures){
					$text_ouverture = $label_ouverture."\r\n";
					$text_ouverture .= $debut_jours_ouverture.implode($separateur_jours_ouverture, $jours_ouvertures).$fin_jours_ouverture."\r\n";
				}
				if (array_key_exists('Joursfermeture', $detail_periode)) $Joursfermetures = $detail_periode['Joursfermeture'];
				if ($show_jour_fermeture and !empty($Joursfermetures)) {
					$jour = array();
					foreach ($Joursfermetures as $Joursfermeture) {
						$jour[$Joursfermeture['Ordre']] = $Joursfermeture['ThesLibelle'];
					}
					$text_ouverture .= $label_fermeture."\r\n";
					$text_ouverture .= $debut_fermeture;
					$text_ouverture .= implode($implode_fermeture, $jour);
					$text_ouverture .= $fin_fermeture."\r\n";
				}
				$text .= sprintf($text_enveloppe_ouverture, $text_ouverture)."\r\n";
				$text = str_replace('{cn}', $cn, $text);
				if ($text) $periodes .= sprintf($text_enveloppe_periode, $text)."\r\n";
			}
		} elseif (defined('JDEBUG') && JDEBUG){
			Log::add(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field, Log::ERROR, 'field');
		}
		if ($periodes) $result = sprintf($text_enveloppe, $periodes);
		return $result;
	}

	function JSON_tarifs_TIF_V3($params = array()) {
		$path 			= 'TARIFSs';
		$text_prix		= '%s€';
		$text_de_a		= 'Compris entre %s€ et %s€';
		$text_type_tarif 		= '<span class="type_tarif">%s</span>';
		$text_complement_tarif 	= '<span class="complement_tarif">&nbsp;%s</span>';
		$text_titre_tarif 		= '<div class="titre_tarif">%s</div>';
		$text_enveloppe_prix	= '<ul class="prix"><li>%s</li></ul>';
		$text_enveloppe = '<div class="tarifs">%s</div>';
		$tarifs			= array();
		$result 		= '';
		if (is_array($params)){
			if(array_key_exists('path', $params)) 					$path = $params['path'];
			if(array_key_exists('text_prix', $params)) 				$text_prix = $params['text_prix'];
			if(array_key_exists('text_de_a', $params)) 				$text_de_a = $params['text_de_a'];
			if(array_key_exists('text_type_tarif', $params)) 		$text_type_tarif = $params['text_type_tarif'];
			if(array_key_exists('text_complement_tarif', $params)) 	$text_complement_tarif = $params['text_complement_tarif'];
			if(array_key_exists('text_titre_tarif', $params)) 		$text_titre_tarif = $params['text_titre_tarif'];
			if(array_key_exists('text_enveloppe_prix', $params)) 	$text_enveloppe_prix = $params['text_enveloppe_prix'];
			if(array_key_exists('text_enveloppe', $params)) 		$text_enveloppe = $params['text_enveloppe'];
		}
		//lecture et mise au format d'une valeur
		if (is_null($way = self::check_json_path($path)))
			print_r(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field.'<br/>');

		if ($way) {
			foreach($way as $detail_tarif){
				$text = $prix = $ordre = $intitule = $Complementtarif = $MinimumEuro = $MaximumEuro = $Intituletarifs = '';
				if (array_key_exists('Ordre', $detail_tarif)) $ordre = $detail_tarif['Ordre'];
				if (array_key_exists('Complementtarif', $detail_tarif)) $Complementtarif = $detail_tarif['Complementtarif'];
				if (array_key_exists('MinimumEuro', $detail_tarif)) $MinimumEuro = $detail_tarif['MinimumEuro'];
				if (array_key_exists('MaximumEuro', $detail_tarif)) $MaximumEuro = $detail_tarif['MaximumEuro'];
				if (array_key_exists('Intituletarifs', $detail_tarif) and array_key_exists('ThesLibelle', $detail_tarif['Intituletarifs'])) $Intituletarifs = $detail_tarif['Intituletarifs']['ThesLibelle'];
				if ($Intituletarifs) $intitule = sprintf($text_type_tarif, $Intituletarifs);
				if ($Complementtarif) $intitule .= sprintf($text_complement_tarif, $Complementtarif);
				if ($Intituletarifs or $Complementtarif) $text .= sprintf($text_titre_tarif, $intitule)."\r\n";
				if ($MinimumEuro)
					if ($MaximumEuro and $MinimumEuro != $MaximumEuro)
						$prix .= sprintf($text_de_a, $MinimumEuro, $MaximumEuro);
					else
						$prix .= sprintf($text_prix, $MinimumEuro);
				if ($prix) $text .= sprintf($text_enveloppe_prix, $prix)."\r\n";
				if ($text and $ordre) $tarifs[$ordre] = $text;
			}
		} elseif (defined('JDEBUG') && JDEBUG){
			Log::add(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field, Log::ERROR, 'field');
		}
		if ($tarifs) $result = sprintf($text_enveloppe, implode('', $tarifs));
		return $result;
	}

	/******************************/
	// fonction XML_if_value
	// récupère une valeur d'un champ
	// ou la valeur du type d'un champ
	//
	// path : chemin xml suivant les synthaxes XQuery
	//		ex : "tif:Contacts/*/*/*/*/*/*/tif:DetailMoyenCom"
	//
	// $comp : instruction de comparaison. Ex : ">=10"
	// $true : résultat si vrai
	// $false : résultat si faux
	//
	// index en cas de reponse multiple: 0 est la première valeur
	//
	// separator pour définir la valeur séparant les éléments de réponse multiple
	//
	// exemple de saisie :
	//		$this->XML_value("tif:Contacts/*/*/*/*/*/*/tif:DetailMoyenCom");
	//		$this->XML_value("tif:Contacts/tif:DetailContact[attribute::libelle='Etablissement']/tif:Adresses/tif:DetailAdresse/tif:Adr1");
	//		$this->XML_value("tif:Contacts/tif:DetailContact[@libelle='Etablissement']/tif:Adresses/tif:DetailAdresse/tif:Adr1");
	//
	/******************************/
	function JSON_if_value($params) {
		$path = $params;
		
		if (is_array($params)){
			if(array_key_exists('path', $params)) 		$path = $params['path'];
			if(array_key_exists('comp', $params)) 		$comp = $params['comp'];
			if(array_key_exists('val_true', $params)) 	$true = $params['val_true'];
			if(array_key_exists('val_false', $params)) 	$false = $params['val_false'];
			if(array_key_exists('val_default', $params)) $default = $params['val_default'];
			if(array_key_exists('id', $params)) 		$id = $params['id'];
			if(array_key_exists('separator', $params)) 	$separator = $params['separator'];
			if(array_key_exists('comp_type', $params)) $comp_type = $params['comp_type'];
		}

		//lecture et mise au format d'une valeur
		//lecture et mise au format d'une valeur
		if (is_null($way = self::check_json_path($path)))
			print_r(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field.'<br/>');

		$result = array();
		$temps = $default;
		$trace = 'temps = ??? / comp = '.$comp.' / result(strcasecmp) = default';
		if ($way and $nbr = $liste->length) {
			die(print_r($way));
			$result = array();
			$deb = 0;
			if ($id != '') {
				$nbr = $id + 1; //si param id existe ne fait qu'une fois la boucle.
				$deb = $id;
			}
			for ($id = $deb; $id < $nbr; $id ++) {
				$nod_testing = trim($liste->item((int)$id)->nodeValue); //lecture de toutes les valeurs contenue en dessous
				switch ($comp_type) {
					case 'strcasecmp' :
						if (eval('return strcasecmp($nod_testing, $comp);')) {
							$temps = $false;
							$trace = 'temps = '.$temps.' / node value = '.$nod_testing.' / comp = '.$comp.' / result(strcasecmp) = false';
						} else {
							$temps = $true;
							$trace = 'temps = '.$temps.' / node value = '.$nod_testing.' / comp = '.$comp.' / result(strcasecmp) = true';
						}
						break;
					case 'logic' :
						$nod_testing = (int)$nod_testing;
						$test = eval('return '.$nod_testing.$comp.';');
						if (eval('($nod_testing.$comp);')) {
							$temps = $true;
							$trace = 'temps = '.$temps.' / comp = '.$comp.' / result(logic) = true';
						} else {
							$temps = $false;
							$trace = 'temps = '.$temps.' / comp = '.$comp.' / result(logic) = false';
						}
						break;
					case 'strlen' :
						$test = eval('return '.strlen(strval($nod_testing)).$comp.';');
						if ($test) {
							$temps = $true;
							$trace = 'temps = '.$temps.' / comp = '.$comp.' / result(strlen) = true';
						} else {
							$temps = $false;
							$trace = 'temps = '.$temps.' / comp = '.$comp.' / result(strlen) = false';
						}
						break;
				}
				if (defined('JDEBUG') && JDEBUG){
					Log::add(Text::_('TIFFP_IF_VALUE_RESULT').' : '.$trace, Log::DEBUG, 'field');
				}
			}
		} else {
			if (defined('JDEBUG') && JDEBUG){
				Log::add(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field, Log::ERROR, 'field');
			}
		}
		$result[] = $temps; //lecture de toutes les valeurs contenue en dessous
		
		return implode( $separator, $result);
	}

	protected function get_champ_value($array, $key, $key_value)
	{
		$result = '';
		if (isset($array))
			if(is_array($array) and !empty($array))
				foreach ($array as $key2 => $tmp) {
					if (is_array($tmp)) {
						$result = self::get_champ_value($tmp, $key2, $key_value);
					} else {
						if ($key2 == $key_value) {
							$result = $tmp;
							break;
						}
					}
				}
			elseif ($key == $key_value)
				$result = $array;
		return $result;
	}

	protected function find_array_value($array, $keyToFind, $valueToFind, $currentPath = array())
	{
		$paths = [];

		foreach ($array as $k => $v) {
			// Ajoute l'indice au chemin actuel
			$currentPath[] = $k;

			if (is_array($v)) {
				// Si c'est un tableau, explore récursivement
				$nestedPaths = self::find_array_value($v, $keyToFind, $valueToFind, $currentPath);
				$paths = array_merge($paths, $nestedPaths);
			} elseif ($k === $keyToFind && $v === $valueToFind) {
				// Si la clé et la valeur correspondent, ajoute le chemin trouvé
				$paths[] = $currentPath;
			}

			// Retire l'indice du chemin actuel pour l'itération suivante
			array_pop($currentPath);
		}

		return $paths;
	}

	protected function check_json_path ($path)
	{
		$full_path = $this->json;
		$path_array = explode('/', $path);
		$checking_path = '';

		if(!is_array($full_path)) {
			Log::add(Text::_('TIFFP_ERREUR_LECTURE_FIELD_PULL_PATH_EMPTY').' : '.$this->field.' : '.$path, Log::ERROR, 'field');
			return;
		}

		if(!empty($path))
		foreach ($path_array as $index) {
			$checking_path .= '/'.$index;
			if (strpos($index, '[') === false) {
				if(!is_array($full_path)) {
					break;
				}
				if (!array_key_exists($index, $full_path)) {
					Log::add(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field, Log::ERROR, 'field');
					return false;
				} else {
					$full_path = $full_path[$index];
				}
			} else {
				$string = substr($index, 1, -1);
				if (!array_key_exists($string, $full_path)) {
					Log::add(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field, Log::ERROR, 'field');
					return false;
				} else {
					$full_path = $full_path[$string];
				}
			}
		}
		return $full_path;
	}
}
