<?php
	/*
	* TifFP composant passerelel de TourInFrance vers GMapFP pour Joomla! 3.x
	* Version J3.0
	* Creation date: Novembre 2013
	* Author: Fabrice4821 - www.gmapfp.org
	* Author email: webmaster@gmapfp.org
	* License GNU/GPL
	*/

namespace Joomla\Component\Tiffps\Administrator\Model;

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Language\Associations;
use Joomla\CMS\MVC\Model\ListModel;
use Joomla\CMS\Table\Table;
use Joomla\Utilities\ArrayHelper;

class TiffpsModel extends ListModel
{
	public function __construct($config = array())
	{
		if (empty($config['filter_fields'])) {
			$config['filter_fields'] = array(
				'id', 'a.id',
				'nom_fiche', 'a.nom_fiche',
				'published', 'a.published',
				'language', 'a.g_language', 'type_format', 'type_xml'
				);
		}

		parent::__construct($config);
	}

	public function purge()
	{
		$db			= Factory::getDBO();
		
		$query = $db->getQuery(true);
		$query->delete($db->quoteName('#__tiffp_xml'));
		$db->setQuery($query);
		$result = $db->execute();
	}
	
	protected function _buildContentWhere()
	{
		$db			= Factory::getDBO();
		
		$where = array();

		// Filter by search in title
		$search = $this->getState('filter.search');
		if (!empty($search)) {
			if (stripos($search, 'id:') === 0) {
				$where[] = 'a.id = '.(int) substr($search, 3);
			} else {
				$search = $db->Quote('%'.$db->escape($search, true).'%');
				$where[] = 'a.nom_fiche LIKE '.$search;
			}
		}
		// Filter on the type_xml.
		if ($type_xml = $this->getState('filter.type_xml')) {
			$where[] = 'a.type_xml = ' . $db->quote($type_xml);
		}
		// Filter on the type_format.
		if ($type_format = $this->getState('filter.type_format')) {
			$where[] = 'a.type_format = ' . $db->quote($type_format);
		}
		// Filter on the language.
		if ($language = $this->getState('filter.g_language')) {
			$where[] = 'a.g_language = ' . $db->quote($language);
		}
		// Filter by published state
		$published = $this->getState('filter.published');
		if (is_numeric($published)) {
			$where[] = 'a.published = '.(int) $published;
		} elseif ($published === '') {
			$where[] = '(a.published IN (0, 1))';
		}

		$where 		= ( count( $where ) ? implode( ' AND ', $where ) : '' );
// die(print_r($where));
		return $where;
	}

	protected function getStoreId($id = '')
	{
		// Compile the store id.
		$id	.= ':'.$this->getState('filter.search');
		$id	.= ':'.$this->getState('filter.published');
		$id	.= ':'.$this->getState('filter.category_id');
		$id	.= ':'.$this->getState('filter.g_language');
		$id	.= ':'.$this->getState('filter.type_xml');
		$id	.= ':'.$this->getState('filter.type_format');

		return parent::getStoreId($id);
	}

	protected function getListQuery()
	{
		// Initialise variables.
		$db		= $this->getDbo();
		$query	= $db->getQuery(true);

		// Select the required fields from the table.
		$query->select(
			$this->getState(
				'list.select',
				'a.id AS id, a.nom_fiche AS name, '.
				'a.g_language, a.created_by, a.checked_out_time, a.type_xml, a.type_format, '.
				'a.published AS published, a.checked_out AS checked_out, uc.name AS editor'
			)
		);
		$query->from('`#__tiffp` AS a')
            ->join('LEFT', $db->quoteName('#__users', 'uc'), $db->quoteName('uc.id') . ' = ' . $db->quoteName('a.checked_out'));

		$where	= $this->_buildContentWhere();
		//die(print_r($where));
		if ($where) $query->where($where);
		// Add the list ordering clause.
		$orderCol	= $this->state->get('list.ordering', 'a.nom_fiche');
		$orderDirn	= $this->state->get('list.direction', 'ASC');
		$query->order($db->escape($orderCol.' '.$orderDirn));

		//die(print_r($query));
		return $query;
	}

	public function getTable($type = 'Tiffp', $prefix = 'TiffpsTable', $config = array())
	{
		return Table::getInstance($type, $prefix, $config);
	}

	protected function populateState($ordering = null, $direction = null)
	{
		// Initialise variables.
		$app = Factory::getApplication('administrator');

		// Load the filter state.
		$search = $this->getUserStateFromRequest($this->context.'.filter.search', 'filter_search');
		$this->setState('filter.search', $search);

		$state = $this->getUserStateFromRequest($this->context.'.filter.published', 'filter_state', '', 'string');
		$this->setState('filter.published', $state);

		$g_language = $this->getUserStateFromRequest($this->context.'.filter.g_language', 'filter_g_language', '');
		$this->setState('filter.g_language', $g_language);

		// Load the parameters.
		$params = ComponentHelper::getParams('com_tiffps');
		$this->setState('params', $params);

		// List state information.
		parent::populateState('a.nom_fiche', 'ASC');
	}
}