<?php

	/*
	* TifFP composant passerelel de TourInFrance vers GMapFP pour Joomla! 4.x
	* Version J4.4
	* Creation date: Février 2024
	* Author: Fabrice4821 - www.gmapfp.org
	* Author email: webmaster@gmapfp.org
	* License GNU/GPL
	*/

namespace Joomla\Component\Tiffps\Administrator\Model;

// phpcs:disable PSR1.Files.SideEffects
\defined('JPATH_PLATFORM') or die;
// phpcs:enable PSR1.Files.SideEffects

use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Filter\OutputFilter;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Log\Log;

trait TiffpsXMLTrait
{
	/**************************************/
	/* fonctions de traitement des champs */
	/**************************************/
	
	/******************************/
	// fonction XML_value
	// récupère une valeur d'un champ
	// ou la valeur du type d'un champ
	//
	// paramètres : chemin xml suivant les synthaxes XQuery
	//		ex : "tif:Contacts/*/*/*/*/*/*/tif:DetailMoyenCom"
	//
	// format, dans le cas d'une conversion
	//
	// index en cas de reponse multiple: 0 est la première valeur
	//
	// separator pour définir la valeur séparant les éléments de réponse multiple
	//
	// exemple de saisie :
	//		$this->XML_value("tif:Contacts/*/*/*/*/*/*/tif:DetailMoyenCom");
	//		$this->XML_value("tif:Contacts/tif:DetailContact[attribute::libelle='Etablissement']/tif:Adresses/tif:DetailAdresse/tif:Adr1");
	//		$this->XML_value("tif:Contacts/tif:DetailContact[@libelle='Etablissement']/tif:Adresses/tif:DetailAdresse/tif:Adr1");
	//
	/******************************/
	function XML_value($params, $format = '%s', $id = '', $separator = '<br/>', $filter = '') {
		$path = $params;
		$qte_max = 0;
		
		if (is_array($params)){
			if(array_key_exists('path', $params)) 		$path = $params['path'];
			if(array_key_exists('format', $params)) 	$format = $params['format'];
			if(array_key_exists('id', $params)) 		$id = $params['id'];
			if(array_key_exists('separator', $params)) 	$separator = $params['separator'];
			if(array_key_exists('filter', $params)) 	$filter = $params['filter'];
			if(array_key_exists('qte_max', $params)) 	$qte_max = (int)$params['qte_max'];
		}
		//lecture et mise au format d'une valeur
		if (is_null($liste = $this->xpathcontext->query($path)))
			print_r(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field.'<br/>');
		$result = array();
		if (empty($format)) $format = '%s';
		//if ($id) die(print_r($liste->length));
		if ($liste and $nbr = $liste->length) {
			$deb = 0;
			if ($id != '') {
				$nbr = $id + 1; //si param id existe ne fait qu'une fois la boucle.
				$deb = $id;
			}
			for ($id = $deb; $id < $nbr; $id ++) {
				$value = @trim($liste->item((int)$id)->nodeValue);
				if ($qte_max and strlen($value) > $qte_max) $value = substr($value, 0, $qte_max).'...';
				if ($value and $filter) {
					if (strpos($value, $filter) === false) $value = null;
				}
				if ($value) {
					$result[] = sprintf($format, nl2br($value)); //lecture de toutes les valeurs contenue en dessous
				}
			}
		} else {
			if (defined('JDEBUG') && JDEBUG){
				Log::add(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field, Log::ERROR, 'field');
			}
		}
		return implode( $separator, $result);
	}

	/******************************/
	// fonction XML_if_value
	// récupère une valeur d'un champ
	// ou la valeur du type d'un champ
	//
	// path : chemin xml suivant les synthaxes XQuery
	//		ex : "tif:Contacts/*/*/*/*/*/*/tif:DetailMoyenCom"
	//
	// $comp : instruction de comparaison. Ex : ">=10"
	// $true : résultat si vrai
	// $false : résultat si faux
	//
	// index en cas de reponse multiple: 0 est la première valeur
	//
	// separator pour définir la valeur séparant les éléments de réponse multiple
	//
	// exemple de saisie :
	//		$this->XML_value("tif:Contacts/*/*/*/*/*/*/tif:DetailMoyenCom");
	//		$this->XML_value("tif:Contacts/tif:DetailContact[attribute::libelle='Etablissement']/tif:Adresses/tif:DetailAdresse/tif:Adr1");
	//		$this->XML_value("tif:Contacts/tif:DetailContact[@libelle='Etablissement']/tif:Adresses/tif:DetailAdresse/tif:Adr1");
	//
	/******************************/
	function XML_if_value($params, $comp = '==1', $true = '1', $false = '0', $default='0', $id = '', $separator = '<br/>', $comp_type = 'logic') {
		$path = $params;
		
		if (is_array($params)){
			if(array_key_exists('path', $params)) 		$path = $params['path'];
			if(array_key_exists('comp', $params)) 		$comp = $params['comp'];
			if(array_key_exists('val_true', $params)) 	$true = $params['val_true'];
			if(array_key_exists('val_false', $params)) 	$false = $params['val_false'];
			if(array_key_exists('val_default', $params)) $default = $params['val_default'];
			if(array_key_exists('id', $params)) 		$id = $params['id'];
			if(array_key_exists('separator', $params)) 	$separator = $params['separator'];
			if(array_key_exists('comp_type', $params)) $comp_type = $params['comp_type'];
		}

		//lecture et mise au format d'une valeur
		if (is_null($liste = $this->xpathcontext->query($path)))
			print_r(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field.'<br/>');
		$result = array();
		$temps = $default;
		$trace = 'temps = ??? / comp = '.$comp.' / result(strcasecmp) = default';
		if ($liste and $nbr = $liste->length) {
			$result = array();
			$deb = 0;
			if ($id != '') {
				$nbr = $id + 1; //si param id existe ne fait qu'une fois la boucle.
				$deb = $id;
			}
			for ($id = $deb; $id < $nbr; $id ++) {
				$nod_testing = trim($liste->item((int)$id)->nodeValue); //lecture de toutes les valeurs contenue en dessous
				switch ($comp_type) {
					case 'strcasecmp' :
						if (eval('return strcasecmp($nod_testing, $comp);')) {
							$temps = $false;
							$trace = 'temps = '.$temps.' / node value = '.$nod_testing.' / comp = '.$comp.' / result(strcasecmp) = false';
						} else {
							$temps = $true;
							$trace = 'temps = '.$temps.' / node value = '.$nod_testing.' / comp = '.$comp.' / result(strcasecmp) = true';
						}
						break;
					case 'logic' :
						$nod_testing = (int)$nod_testing;
						$test = eval('return '.$nod_testing.$comp.';');
						if ($test) {
							$temps = $true;
							$trace = 'temps = '.$temps.' / comp = '.$comp.' / result(logic) = true';
						} else {
							$temps = $false;
							$trace = 'temps = '.$temps.' / comp = '.$comp.' / result(logic) = false';
						}
						break;
					case 'strlen' :
						$test = eval('return '.strlen(strval($nod_testing)).$comp.';');
						if ($test) {
							$temps = $true;
							$trace = 'temps = '.$temps.' / comp = '.$comp.' / result(strlen) = true';
						} else {
							$temps = $false;
							$trace = 'temps = '.$temps.' / comp = '.$comp.' / result(strlen) = false';
						}
						break;
				}
				if (defined('JDEBUG') && JDEBUG){
					Log::add(Text::_('TIFFP_IF_VALUE_RESULT').' : '.$trace, Log::DEBUG, 'field');
				}
			}
		} else {
			if (defined('JDEBUG') && JDEBUG){
				Log::add(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field, Log::ERROR, 'field');
			}
		}
		$result[] = $temps; //lecture de toutes les valeurs contenue en dessous
		
		return implode( $separator, $result);
	}
	
	/**************************************/
	/* fonctions de traitement des champs */
	/**************************************/
	
	/******************************/
	// fonction XML_alias_value
	// récupère une valeur d'un champ
	// ou la valeur du type d'un champ
	// et transforme ma en alias
	//
	// paramètres : chemin xml suivant les synthaxes XQuery
	//		ex : "tif:Contacts/*/*/*/*/*/*/tif:DetailMoyenCom"
	//
	// index en cas de reponse multiple: 0 est la première valeur
	//
	// exemple de saisie :
	//		$this->XML_value("tif:Contacts/*/*/*/*/*/*/tif:DetailMoyenCom");
	//		$this->XML_value("tif:Contacts/tif:DetailContact[attribute::libelle='Etablissement']/tif:Adresses/tif:DetailAdresse/tif:Adr1");
	//		$this->XML_value("tif:Contacts/tif:DetailContact[@libelle='Etablissement']/tif:Adresses/tif:DetailAdresse/tif:Adr1");
	//
	/******************************/
	function XML_alias_value($path, $id = '') {
		//lecture et mise au format d'une valeur
		if (is_null($liste = $this->xpathcontext->query($path)))
			print_r(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field.'<br/>');
		$result = array();
		//if ($id) die(print_r($liste->length));
		if ($liste and $nbr = $liste->length) {
			$deb = 0;
			if ($id != '') {
				$nbr = $id + 1; //si param id existe ne fait qu'une fois la boucle.
				$deb = $id;
			}
			for ($id = $deb; $id < $nbr; $id ++) {
				$result = OutputFilter::stringURLSafe(trim($liste->item((int)$id)->nodeValue)); //lecture de toutes les valeurs contenue en dessous
				if(trim(str_replace('-','',$result)) == '') {
					$this->alias = HtmlHelper::date('now', Text::_('DATE_FORMAT_CALENDAR_DATETIME')); //%Y-%m-%d %H:%M:%S
				}
			}
		} else {
			if (defined('JDEBUG') && JDEBUG){
				Log::add(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field, Log::ERROR, 'field');
			}
		}
		return $result;
	}

	/******************************/
	// fonction XML_main_img
	// récupère une image
	// et traite la en tand qu'image principale
	//
	// paramètres : 
	//	chemin xml suivant les synthaxes XQuery
	// $user_id, dans le cas de l'affection de l'image à un autre utilisateur que l'administrateur
	//
	//		ex : $this->XML_main_img(array('path'=>"/feed/entry/content/m:properties/d:DetailPhotos_DIAPO",'user_id'=>$this->XML_UserID());
	//
	/******************************/
	function XML_main_img($options)
	{
		$user_id 		= $this->user_id;
		$default_file 	= '';
		$position 		= 0;
		$alt 			= '';
		$caption 		= '';
		
		$path = $options;
		if (is_array($options)){
			if(array_key_exists('path', $options)) 		$path = $options['path'];
			if(array_key_exists('user_id', $options)) 	$user_id = $options['user_id'];
			if(array_key_exists('default_file', $options)) $default_file = $options['default_file'];
			if(array_key_exists('position', $options)) 	$position = $options['position'];
			if(array_key_exists('alt', $options)) 		$alt = $options['alt'];
			if(array_key_exists('caption', $options)) 	$caption = $options['caption'];
		}

		if ($this->tiffps_photos_path == '')
			print_r(Text::_('COM_TIFFP_ERREUR_REPERTOIRE_IMG_NON_CONFIG').'<br/>');
		//lecture et mise au format d'une valeur
		if (is_null($liste = $this->xpathcontext->query($path)))
			print_r(Text::_('COM_TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field.'<br/>');
		$no_image = true;
		if ($liste and $liste->item(0)) {
			$value = explode($this->tiffps_values_separator, trim($liste->item(0)->nodeValue));
			if (array_key_exists($position, $value) and (trim($value[$position])!='')){
				if(strstr($value[$position], 'http') === false) 
					$this->stocke_infos_image($this->tiffps_photos_path.$value[$position], $user_id, true, $options);
				else
					$this->stocke_infos_image($value[$position], $user_id, true, $options);
				$json['image'] = $this->path_main_images.strtolower($user_id.'__'.basename($value[$position]));
				if ($alt) 
					$json['image_alt'] = $alt;
				else
					$json['image_alt'] = basename($value[$position]);
				if ($caption)
					$json['image_caption'] = $caption;
				else
					$json['image_caption'] = '';
				$tmp = json_encode($json);
				$no_image = false;
			}
		}
		if ($no_image) {
			if (defined('JDEBUG') && JDEBUG){
				Log::add(Text::_('TIFFP_LECTURE_FIELD').' : '.$this->field, Log::NOTICE, 'field');
			}
			$tmp = '';
			if ($default_file) $tmp = $default_file;
		}
		return $tmp;
	}
	
	/******************************/
	// fonction XML_img
	// récupère des images
	// en tand qu'image de galerie
	//
	// paramètres : 
	//	chemin xml suivant les synthaxes XQuery
	//	$first => traite aussi la première image (qui sert généralement d'image principale)
	//		ex : $this->XML_img(array('path'=>"/feed/entry/content/m:properties/d:DetailPhotos_DIAPO",'first'=>0));
	//
	/******************************/
	function XML_img($params) {
		$config	= ComponentHelper::getParams('com_tiffps');
		
		$qte_max 	= 0;
		$good_image = 0;
		$first		= true;
		$position 	= 0;
		$tif_version = 'V3';
		
		$path = $params;
		if (is_array($params)){
			if(array_key_exists('path', $params)) 		$path = $params['path'];
			if(array_key_exists('first', $params)) 		$first = $params['first'];
			if(array_key_exists('position', $params)) 	$position = $params['position'];
			if(array_key_exists('qte_max', $params)) 	$qte_max = $params['qte_max'];
			if(array_key_exists('tif_version', $params)) 	$tif_version = $params['tif_version'];
		}
		
		if (is_null($liste = $this->xpathcontext->query($path)))
			print_r(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field);
		$result = array();
		if ($liste->length > 0) {
			if ($tif_version == 'V3') {
				$cn = 0;
				$good_image = 0;
				$images_datas = array();
				for ($item_id = 0; $item_id < $liste->length; $item_id ++) {//boucle sur chaque ensemble de prix
					if ($first or $cn) {
						$Titre_photo = '';
						$Credit_photo = '';
						$Url_photo = '';
						$elements_photos = $liste->item($item_id);
						for ($node_id = 0; $node_id < $elements_photos->childNodes->length; $node_id ++) {//boucle sur chaque element de photo
							if ($elements_photos->childNodes[$node_id]->nodeName == 'content') {
								$elements_photo = $elements_photos->childNodes[$node_id];
								break;
							}
						}
						if ($elements_photo) {
							for ($node_id = 0; $node_id < $elements_photo->childNodes->length; $node_id ++) {//boucle sur chaque element de photo
								$continuer = false;
								if ($elements_photo->childNodes[$node_id]->nodeName == 'm:properties') {
									$elements_photo = $elements_photo->childNodes[$node_id];
									$continuer = true;
									break;
								}
							}
						}
						if ($continuer) {
							for ($node_id = 0; $node_id < $elements_photo->childNodes->length; $node_id ++) {//boucle sur chaque element de photo
								$continuer = false;
								if ($elements_photo->childNodes[$node_id]->nodeName == 'd:Photo') {
									$elements_photo = $elements_photo->childNodes[$node_id];
									$continuer = true;
									break;
								}
							}
						}
						if ($continuer) {
							for ($node_id = 0; $node_id < $elements_photo->childNodes->length; $node_id ++) {//boucle sur chaque element de photo
								if ($elements_photo->childNodes[$node_id]->nodeName == 'd:Titre') {
									$Titre_photo = $elements_photo->childNodes[$node_id]->nodeValue;
								}
								if ($elements_photo->childNodes[$node_id]->nodeName == 'd:Credit') {
									$Credit_photo = $elements_photo->childNodes[$node_id]->nodeValue;
								}
								if ($elements_photo->childNodes[$node_id]->nodeName == 'd:Url') {
									$Url_photo = $elements_photo->childNodes[$node_id]->nodeValue;
								}
							}
						}
					}
					if ($Url_photo) {
						$images_datas[] = array('titre' => $Titre_photo, 'credit' => $Credit_photo, 'url' => $Url_photo);
						$good_image++;
					}
					$cn++;
					if ($qte_max and $qte_max<=$good_image) break;
				}
				foreach ($images_datas as $image_datas) {
					if (strpos($image_datas['url'], 'http') !== 0)
						$image_name = $config->get('tiffps_photos_path').$image_datas['url'];
					else
						$image_name = $image_datas['url'];
					if ($return_upload_image = $this->stocke_infos_image($image_name, '', false, $params)) {
						$good_image ++;
						$result[] = array('url' => $return_upload_image, 'credit' => $image_datas['credit'], 'titre' => $image_datas['titre']);
					}
					if (defined('JDEBUG') && JDEBUG)
						Log::add(Text::_('TIFFP_LECTURE_FIELD').' : '.json_encode($image_datas), Log::INFO, 'thumbs');
				}
			} else {
				$fields = explode($this->tiffps_fields_separator, trim($liste->item(0)->nodeValue));
				$cn = 0;
				$good_image = 0;
				foreach ($fields as $field) {
					if ($first or $cn) {
						$image_datas = explode($this->tiffps_values_separator, $field);
						if (array_key_exists($position, $image_datas)) {
							$image_name = $image_datas[$position];						
							if (strpos($image_name, 'http') !== 0)
								$image_name = $config->get('tiffps_photos_path').$image_datas[$position];
							if ($return_upload_image = $this->stocke_infos_image($image_name, '', false, $params)) {
								$good_image ++;
								$result[] = $return_upload_image;
							}
							if (defined('JDEBUG') && JDEBUG)
								Log::add(Text::_('TIFFP_LECTURE_FIELD').' : '.json_encode($image_datas).'_/_position  : '.$position, Log::INFO, 'thumbs');
						} else 
							if (defined('JDEBUG') && JDEBUG)
								Log::add(Text::_('TIFFP_LECTURE_FIELD_KEY_DOESNT_EXISTS').' : '.$position.' dans '.$field, Log::INFO, 'thumbs');
					}
					$cn++;
					if ($qte_max and $qte_max<=$good_image) break;
				}
			}
		} else {
			if (defined('JDEBUG') && JDEBUG){
				Log::add(Text::_('TIFFP_LECTURE_FIELD').' : '.$this->field, Log::NOTICE, 'thumbs');
			}
		}
	return $result;
	}
	
	/******************************/
	// fonction XML_ACF_img
	// récupère des images
	// et renseigne le champ au format de ACF image :
	// {"items":[{"caption":"FP","thumbnail":"media\/acfgallery\/tmp\/FP_thumb.png","image":"media\/acfgallery\/tmp\/FP.png"},{"caption":"Hono","thumbnail":"media\/acfgallery\/tmp\/Hono-marcilly-2020-B4_thumb.jpg","image":"media\/acfgallery\/tmp\/Hono-marcilly-2020-B4.jpg"}]}
	//
	// paramètres : 
	//	chemin xml suivant les synthaxes XQuery
	//		ex : $this->XML_ACF_img(array('path'=>"/feed/entry/content/m:properties/d:LABELS"));
	//
	// Traite les champs de type <d:LABELS>&lt;img alt="Qualité Tourisme" title="Qualité Tourisme" src="ht
	/******************************/
	public function XML_ACF_img($params)
	{
		$path = $params;
		$captions = array();
		$alt = array();
		$source = array();
		$info_img = array();
		$return = array();
		if (is_array($params)){
			if(array_key_exists('path', $params)) 		$path = $params['path'];
			if(array_key_exists('caption', $params)) 	$captions = $params['caption'];
			if(array_key_exists('alt', $params)) 		$captions = $params['alt'];
			if(array_key_exists('source', $params)) 	$captions = $params['source'];
			if(array_key_exists('tags', $params)) 		$captions = $params['tags'];
			if(array_key_exists('first', $params)) 		$first = $params['first'];
			if(array_key_exists('qte_max', $params)) 	$qte_max = $params['qte_max'];
		}
		
		$params['first'] = true;
		$imgs = $this->XML_img($params);
		$i = 0;
		foreach($imgs as $img) {
			if (is_array($img)) {
				$url_img = $img['url'];
				$caption_text = $img['titre'];
				$alt_text = $img['titre'];
				$source_text = $img['credit'];
			} else {
				$url_img = $img;
				$caption_text = '';
				$alt_text = '';
				$source_text = '';
			}
			if(is_array($captions) and array_key_exists($i, $captions))
				$caption_text = $captions[$i];
			if(is_array($alt) and array_key_exists($i, $alt))
				$alt_text = $alt[$i];
			if(is_array($source) and array_key_exists($i, $source))
				$source_text = $source[$i];
			$info_img[] = array('alt'=>$alt_text, 'source'=>$source_text, 'tags'=>[], 'caption'=>$caption_text, 'thumbnail'=>$url_img, 'image'=>$url_img);
			$i++;
		}
		if (!empty($info_img))
			$return = array('items'=>$info_img);
			
		if (empty($return))
			return '';
		else
			return json_encode($return);
	}
	
	/******************************/
	// fonction XML_array_champ_title
	// récupère les valeurs d'une "balise"
	// et convertie les valeurs en fonction des table de correspondance
	//
	// paramètres : 
	//	chemin xml suivant les synthaxes XQuery
	//		ex : $this->XML_array_champ_title("/feed/entry/content/m:properties/d:LABELS", array("Accueil Paysan"), array("1", "2"), "title",  "\r\n");
	//
	// Traite les champs de type <d:LABELS>&lt;img alt="Qualité Tourisme" title="Qualité Tourisme" src="ht
	/******************************/
	function XML_array_champ_title($path, $array_source = array(), $array_dest = array(), $name='title', $separator=',') {
		//lecture et mise au format d'une valeur
		if (is_null($liste = $this->xpathcontext->query($path)))
			print_r(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field.'<br/>');
		if (defined('JDEBUG') && JDEBUG)
			Log::add(Text::_('TIFFP_LIST_LENGTH').' : '.$liste->length, Log::DEBUG, 'longueur value');
		$result = array();
		if ($liste->length) {
			if (defined('JDEBUG') && JDEBUG)
				Log::add(Text::_('TIFFP_ICON_TYPE_REF').' : '.$liste->item(0)->nodeValue, Log::DEBUG, 'node value');
			preg_match_all ("/".$name."=\"([^\")]+)\"/i", $liste->item(0)->nodeValue, $fields);
			if (defined('JDEBUG') && JDEBUG)
				Log::add(Text::_('TIFFP_ICON_TYPE_REF').' : '.implode(',', $fields[1]), Log::DEBUG, 'array');

			foreach ($fields[1] as $field) {
				if (defined('JDEBUG') && JDEBUG)
					Log::add(Text::_('TIFFP_ICON_TYPE_REF').' : '.$field, Log::DEBUG, 'field');
				$temps = array_search($field, $array_source); //lecture de toutes les valeurs contenue en dessous
				if ($temps !== false) 
					$result[] = $array_dest[$temps];
			}

		} else {
			if (defined('JDEBUG') && JDEBUG){
				Log::add(Text::_('TIFFP_LECTURE_FIELD'), Log::NOTICE, 'thumbs');
			}
		}
		return implode($separator, $result);
	}

	/******************************/
	// fonction XML_List_Array
	// récupère les valeurs d'un groupe
	// et convertie les valeurs en fonction des table de correspondance
	//
	// paramètres : 
	//	chemin xml suivant les synthaxes XQuery
	//		ex : $this->XML_List_Array("tif:Langues/tif:Usage[@type='11.01.01']/tif:Langue", array("fr", "en", "es"), array("1", "2", "3"), 'lang',  "\r\n");
	//
	// Traite les champs de type <d:LABELS>&lt;img alt="Qualité Tourisme" title="Qualité Tourisme" src="ht
	/******************************/
	function XML_List_Array($path, $array_source = array(), $array_dest = array(), $name='title', $separator=',') {
		//lecture et mise au format d'une valeur
		if (is_null($liste = $this->xpathcontext->query($path)))
			print_r(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field.'<br/>');
		if (defined('JDEBUG') && JDEBUG)
			Log::add(Text::_('TIFFP_LIST_LENGTH').' : '.$liste->length, Log::DEBUG, 'longueur value');
		$result = array();
		if ($liste and $nbr = $liste->length) {
			for ($id = 0; $id < $nbr; $id ++) {
				$value_source = trim($liste->item((int)$id)->nodeValue); //lecture de toutes les valeurs contenue en dessous
				if (defined('JDEBUG') && JDEBUG)
					Log::add(Text::_('TIFFP_VALEUR_SOURCE').' : '.$value_source, Log::DEBUG, 'field');
				if (in_array($value_source, $array_source)) {
					$index = array_search($value_source, $array_source);
					$result[] = $array_dest[$index];
				}
			}
		} else {
			if (defined('JDEBUG') && JDEBUG){
				Log::add(Text::_('TIFFP_LECTURE_FIELD'), Log::NOTICE, 'List_array');
			}
		}
		return implode($separator, $result);
	}

	/******************************/
	// fonction XML_periode
	// récupère la valeur des periodes
	// d'ouverture et de fermeture
	//
	// paramètres : chemin xml suivant les synthaxes XQuery
	//		ex : "tif:Contacts/*/*/*/*/*/*/tif:DetailMoyenCom"
	//
	// format, dans le cas d'une conversion
	//
	// index en cas de reponse multiple: 0 est la première valeur
	//
	// separator pour définir la valeur séparant les éléments de réponse multiple
	//
	// exemple de saisie :
	//		$this->XML_value("tif:Contacts/*/*/*/*/*/*/tif:DetailMoyenCom");
	//		$this->XML_value("tif:Contacts/tif:DetailContact[attribute::libelle='Etablissement']/tif:Adresses/tif:DetailAdresse/tif:Adr1");
	//		$this->XML_value("tif:Contacts/tif:DetailContact[@libelle='Etablissement']/tif:Adresses/tif:DetailAdresse/tif:Adr1");
	//
	/******************************/


	
	//'tif:Periodes/tif:DetailPeriode/tif:Dates/tif:DetailDates/tif:DateDebut'
	function XML_periodes($path = 'tif:Periodes/tif:DetailPeriode/tif:Dates', $text_du_au = 'Du %s au %s', $text_ouvert = '<div style="margin-left: 30px;">%s :</div>', $text_devant_jour = '- ', $sep_jour = '<br/>', $text_jour = '<div style="margin-left: 60px;">%s</div>', $text_enveloppe_periode = '<p>%s</p>', $date_f = 'd-m-Y', $text_le = 'Le %s') {
		//lecture et mise au format d'une valeur
		if (is_null($liste = $this->xpathcontext->query($path)))
			print_r(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field.'<br/>');
		$result = '';

		if (!is_null($liste)) {
			$id = 0;
			for ($item_id = 0; $item_id < $liste->length; $item_id ++) {
				$text = '';
				if ($liste->item($item_id)->getElementsByTagName("DateDebut")->item(0)->nodeValue or $liste->item($item_id)->getElementsByTagName("DateFin")->item(0)->nodeValue) {
					$debut = new DateTime($liste->item($item_id)->getElementsByTagName("DateDebut")->item(0)->nodeValue);
					$fin   = new DateTime($liste->item($item_id)->getElementsByTagName("DateFin")->item(0)->nodeValue);
					if ($debut == $fin)
						$text .= sprintf($text_le, $debut->format($date_f));
					else
						$text .= sprintf($text_du_au, $debut->format($date_f), $fin->format($date_f));
				}
				
				$nodes = $liste->item($item_id)->getElementsByTagName("DetailJours");
				$id_node = 0;
				foreach($nodes as $node) {
					$type = $node->getAttribute('libelle');
					$text .= sprintf($text_ouvert, $type);
		
					$jours = $liste->item($item_id)->getElementsByTagName("DetailJours")->item($id_node)->getElementsByTagName("Jour");
					$data_jour = array();
					foreach ($jours as $jour) {
						$horaires = $jour->getElementsByTagName("Horaires");
						$heures = '';
						if ($horaires)
						foreach ($horaires as $horaire) {
							$heures .= $horaire->nodeValue.' ';
						}
						$data_jour[$jour->getAttribute('lib_jour')] = $text_devant_jour.$jour->getAttribute('libelle').' : '.$heures;
					}
					ksort($data_jour);
					$text .= sprintf($text_jour, implode($sep_jour, $data_jour));
					$id_node++;
				}
				$result .= sprintf($text_enveloppe_periode, $text);
			}
		
		 }
		return $result;
	}

	function XML_tarifs($path = 'tif:Tarifs/tif:DetailTarifs/tif:DetailTarif', $options = array()) 
	{
		$params = array();
		$params['text_enveloppe_tarif'] = '<div class="tarif">%s</div>'."\n\r";
		$params['text_du_au'] = '<span class="date">Du %s au %s</span>'."\n\r";
		$params['text_le'] = '<li class="date">Le %s</li>'."\n\r";
		$params['text_libelle'] = '<label class="detail_tarif">%s</label>'."\n\r";
		$params['text_enveloppe_detail_tarifs'] = '<ul class="detail_traifs">%s</ul>'."\n\r";
		$params['text_tarif_standard'] = '<li><label>Tarif standard : </label><span> %s &euro;</span></li>'."\n\r";
		$params['text_tarif_min'] = '<li><label>Tarif minimum : </label><span> %s &euro;</span></li>'."\n\r";
		$params['text_tarif_max'] = '<li><label>Tarif maximum : </label><span> %s &euro;</span></li>'."\n\r";
		$params['text_tarif_desc'] = '<li><label>Remarque : </label><span> %s </span></li>'."\n\r";
		$params['date_f'] = 'd-m-Y';
		
		if (!empty($options))
			foreach ($options as $key => $option) {
				$params[$key] = $option;
			}
		
		//lecture et mise au format d'une valeur
		if (is_null($liste = $this->xpathcontext->query($path)))
			print_r(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field.'<br/>');
		$text = '';

		if (!is_null($liste)) {
			$datas = array();
			$id = 0;
			for ($item_id = 0; $item_id < $liste->length; $item_id ++) {
				$tmp = '';
				if (@$liste->item($item_id)->getElementsByTagName("DateDebut")->item(0)->nodeValue or @$liste->item($item_id)->getElementsByTagName("DateFin")->item(0)->nodeValue) {
					$debut = new DateTime($liste->item($item_id)->getElementsByTagName("DateDebut")->item(0)->nodeValue);
					$fin   = new DateTime($liste->item($item_id)->getElementsByTagName("DateFin")->item(0)->nodeValue);
					if ($debut == $fin)
						$tmp .= sprintf($params['text_le'], $debut->format($params['date_f']));
					else
						$tmp .= sprintf($params['text_du_au'], $debut->format($params['date_f']), $fin->format($params['date_f']));
				}

				$tmp .= sprintf($params['text_libelle'], $liste->item($item_id)->getAttribute("libelle"));

				$tmp1 = '';
				$tarif_standard = @$liste->item($item_id)->getElementsByTagName("TarifStandard")->item(0)->nodeValue;
				$tarif_min = @$liste->item($item_id)->getElementsByTagName("TarifMin")->item(0)->nodeValue;
				$tarif_max = @$liste->item($item_id)->getElementsByTagName("TarifMax")->item(0)->nodeValue;
				$tarif_desc = @$liste->item($item_id)->getElementsByTagName("DescriptionTarif")->item(0)->nodeValue;
				if ($tarif_standard) $tmp1 .= sprintf($params['text_tarif_standard'], $tarif_standard);
				if ($tarif_min) $tmp1 .= sprintf($params['text_tarif_min'], $tarif_min);
				if ($tarif_max) $tmp1 .= sprintf($params['text_tarif_max'], $tarif_max);
				if ($tarif_desc) $tmp1 .= sprintf($params['text_tarif_desc'], $tarif_desc);
				$tmp .= sprintf($params['text_enveloppe_detail_tarifs'], $tmp1);
				
				$text .= sprintf($params['text_enveloppe_tarif'], $tmp);
			}
		}
		return $text;
	}
		
	function XM_tarifs_TIF_V3($params = array()) 
	{
		$path = "//feed/entry/link[@title='tarifss']/m:inline/feed/entry/content/m:properties";
		$text_prix		= '%s€';
		$text_de_a		= 'Compris entre %s€ et %s€';
		$text_type_tarif 		= '<span class="type_tarif">%s</span>';
		$text_complement_tarif 	= '<span class="complement_tarif">&nbsp;%s</span>';
		$text_titre_tarif 		= '<div class="titre_tarif">%s</div>';
		$text_enveloppe_prix	= '<ul class="prix"><li>%s</li></ul>';
		$text_enveloppe = '<div class="tarifs">%s</div>';
		$tarifs			= array();
		$result 		= '';
		if (is_array($params)){
			if(array_key_exists('path', $params)) 					$path = $params['path'];
			if(array_key_exists('text_prix', $params)) 				$text_prix = $params['text_prix'];
			if(array_key_exists('text_de_a', $params)) 				$text_de_a = $params['text_de_a'];
			if(array_key_exists('text_type_tarif', $params)) 		$text_type_tarif = $params['text_type_tarif'];
			if(array_key_exists('text_complement_tarif', $params)) 	$text_complement_tarif = $params['text_complement_tarif'];
			if(array_key_exists('text_titre_tarif', $params)) 		$text_titre_tarif = $params['text_titre_tarif'];
			if(array_key_exists('text_enveloppe_prix', $params)) 	$text_enveloppe_prix = $params['text_enveloppe_prix'];
			if(array_key_exists('text_enveloppe', $params)) 		$text_enveloppe = $params['text_enveloppe'];
		}
		
		//lecture et mise au format d'une valeur
		if (is_null($liste = $this->xpathcontext->query($path)))
			print_r(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field.'<br/>');

		if (!is_null($liste)) {
			$datas = array();
			$id = 0;
			
			for ($item_id = 0; $item_id < $liste->length; $item_id ++) {//boucle sur chaque ensemble de prix
				$detail_tarif = array();
				$elements_prix = $liste->item($item_id);
				for ($node_id = 0; $node_id < $elements_prix->childNodes->length; $node_id ++) {//boucle sur chaque element de prix
					$detail_tarif[$elements_prix->childNodes[$node_id]->nodeName] = $elements_prix->childNodes[$node_id]->nodeValue;
					if ($elements_prix->childNodes[$node_id]->nodeName == 'd:Intituletarifs') {
						$elements_complement_prix = $elements_prix->childNodes[$node_id];
					}

				}
				for ($node_id = 0; $node_id < $elements_complement_prix->childNodes->length; $node_id ++) {//boucle sur chaque element de Intituletarifs
					$detail_tarif['Intituletarifs'][$elements_complement_prix->childNodes[$node_id]->nodeName] = $elements_complement_prix->childNodes[$node_id]->nodeValue;

				}
				
				$text = $prix = $ordre = $intitule = $Complementtarif = $MinimumEuro = $MaximumEuro = $Intituletarifs = '';
				if (array_key_exists('d:Ordre', $detail_tarif)) $ordre = $detail_tarif['d:Ordre'];
				if (array_key_exists('d:Complementtarif', $detail_tarif)) $Complementtarif = $detail_tarif['d:Complementtarif'];
				if (array_key_exists('d:MinimumEuro', $detail_tarif)) $MinimumEuro = $detail_tarif['d:MinimumEuro'];
				if (array_key_exists('d:MaximumEuro', $detail_tarif)) $MaximumEuro = $detail_tarif['d:MaximumEuro'];
				if (array_key_exists('Intituletarifs', $detail_tarif) and array_key_exists('d:ThesLibelle', $detail_tarif['Intituletarifs'])) $Intituletarifs = $detail_tarif['Intituletarifs']['d:ThesLibelle'];
				if ($Intituletarifs) $intitule = sprintf($text_type_tarif, $Intituletarifs);
				if ($Complementtarif) $intitule .= sprintf($text_complement_tarif, $Complementtarif);
				if ($Intituletarifs or $Complementtarif) $text .= sprintf($text_titre_tarif, $intitule)."\r\n";
				if ($MinimumEuro)
					if ($MaximumEuro and $MinimumEuro != $MaximumEuro)
						$prix .= sprintf($text_de_a, number_format((float)$MinimumEuro, 2, ',', ' '), number_format((float)$MaximumEuro, 2, ',', ' '));
					else
						$prix .= sprintf($text_prix, number_format((float)$MinimumEuro, 2, ',', ' '));
				if ($prix) $text .= sprintf($text_enveloppe_prix, $prix)."\r\n";
				if ($text and $ordre) $tarifs[$ordre] = $text;
			}
		}
		if ($tarifs) $result = sprintf($text_enveloppe, implode('', $tarifs));
		return $result;
	}
		
//récupère un picto de sur un serveur distant, 
//adapte ces dimensions (suivant config) 
//et la stocker dans un repertoire temporaire
//en attendant l'Id du lieu (création des miniatures)
	/*function TIF _picto_img() {
		//hh
	}*/

// Récupère la date et heure courante.
	function XML_DateHeure() {
		$config = Factory::getConfig();

		$datenow = Factory::getDate('now');
		return $datenow->toSQL();
	}

	/******************************/
	// fonction XML_Syndication
	// récupère une info de l'utilisateur courant.
	//
	// paramètres : 
	//
	// info : id / name / username / email 
	//
	// exemple de saisie :
	//		$this->XML_UserID();
	//		$this->XML_UserID("name");
	//		$this->XML_UserID("email");
	//
	/******************************/
	function XML_UserID($info="id") {
		$user = Factory::getUser();

		return $user->$info;
	}

	/******************************/
	// fonction XML_Syndication_all_value()
	// récupère les informations d'une syndication
	// ou toutes les valeurs
	//
	// paramètres : chemin xml suivant les synthaxes XQuery
	//		ex : "/feed/entry/content/m:properties/comfax"
	//
	// field : n° du champ dans lequel récupérer la valeur
	//         mettre "" pour tous les champs
	//         Il est possible de récupérer de multiple champ en séparant leur valeur d'une virgule
	//
	// fields_separator : séparateur des champs
	//
	// values_separator : séparateur des valeurs
	//
	// string_assemble : Chaine permettant l'assemblage des champs récupérés
	//
	// exemple de saisie :
	//		$this->XML_1_value("/feed/entry/content/m:properties/d:MODPAIEMENT");
	//      $this->XML_all_value("/feed/entry/content/m:properties/d:TYPEACTIVSPORTPRESTA", "0,1", ", ", ", ", "%0% (%1%)");
	//
	/******************************/
	function XML_all_value($params) {
		$config	= ComponentHelper::getParams('com_tiffps');
		
		$field = '';
		$fields_separator = '';
		$values_separator = '';
		$string_assemble = '';
		$ul_li_class = '';
		$ul_li_add_class = '';
		$entete = '';
		$corp = '';
		$fin = '';
		$comportement = 'separateurs';
		
		if (is_array($params)){
			if(array_key_exists('path', $params)) 				$path = $params['path'];
			if(array_key_exists('field', $params)) 				$field = $params['field'];
			
			if(array_key_exists('fields_separator', $params)) 	$fields_separator = $params['fields_separator'];
			if(array_key_exists('values_separator', $params)) 	$values_separator = $params['values_separator'];
			if(array_key_exists('string_assemble', $params)) 	$string_assemble = $params['string_assemble'];
			
			if(array_key_exists('ul_li_class', $params)) 		$ul_li_class = $params['ul_li_class'];
			if(array_key_exists('ul_li_add_class', $params)) 	$ul_li_add_class = $params['ul_li_add_class'];
			
			if(array_key_exists('entete', $params)) 			$entete = $params['entete'];
			if(array_key_exists('corp', $params)) 				$corp = $params['corp'];
			if(array_key_exists('fin', $params)) 				$fin = $params['fin'];
			
			if(array_key_exists('comportement', $params)) 		$comportement = $params['comportement'];
			
		}

		//lecture et mise au format d'une valeur
		$liste = '';
		if (is_null($liste = $this->xpathcontext->query($path)))
			print_r(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field.'<br/>');
		$result = array();
		$multi_field = explode(',', $field);
		if ($liste) {
			$results = @trim($liste->item((int)$id)->nodeValue);
			if (defined('JDEBUG') && JDEBUG){
				Log::add(Text::_('XML_Syndication_all_value_v2').' : '.$results, Log::INFO, 'results');
			}
			$niveau1s = explode($this->tiffps_fields_separator, $results);
			if (!empty($results))
			foreach ($niveau1s as $niveau1) {
				$niveau2s = explode($this->tiffps_values_separator, $niveau1);
				if ($field==='') {
					switch($comportement) {
						case 'ul_li' :
							if (defined('JDEBUG') && JDEBUG){
								Log::add(Text::_('XML_Syndication_all_value_v2').' : '.$ul_li_add_class, Log::INFO, 'ul_li_add_class');
							}
							$tmp = '';
							$class = '';
							if(!empty($niveau2s))
							foreach($niveau2s as $niveau2){
								if($ul_li_add_class){
									$class = 'class="'.str_replace(' ', '_', substr($this->suppr_accents($niveau2), 0, 6)).'"';
								}
								$tmp .= '<span '.$class.'>'.$niveau2.'</span>';							
							}
							$result[] = $tmp;
							break;
						case 'entete_corp_fin' :
							if (defined('JDEBUG') && JDEBUG){
								Log::add(Text::_('XML_all_value').' : '.$corp, Log::INFO, 'corp');
								Log::add(Text::_('XML_all_value').' : '.implode('|', $niveau2s), Log::INFO, 'corp_value');
								Log::add(Text::_('XML_all_value').' : '.vsprintf($corp, $niveau2s), Log::INFO, 'return_value');
							}
							$result[] = vsprintf($corp, $niveau2s);
							break;
						case 'separateurs' :
						default:
							if (defined('JDEBUG') && JDEBUG){
								Log::add(Text::_('XML_all_value').' : '.$values_separator, Log::INFO, 'values_separator');
							}
							$result[] = implode($values_separator, $niveau2s);
					}
				} else {
					if (array_key_exists('1', $multi_field)) {
						if (array_key_exists('0', $niveau2s) and $niveau2s[0]=="") 
							$ma_chaine_tmp = '';
						else 
							$ma_chaine_tmp = $string_assemble;
						foreach ($multi_field as $tmp) {
							if (count($niveau2s)>$tmp)
								$ma_chaine_tmp = str_replace('%'.$tmp.'%', $niveau2s[$tmp], $ma_chaine_tmp);
						}
						$result[] = $ma_chaine_tmp;
					} else {
						if (count($niveau2s)>$field and array_key_exists($field, $niveau2s))
							$result[] = $niveau2s[$field];
					}
				}
			}
		} else {
			if (defined('JDEBUG') && JDEBUG){
				Log::add(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field, Log::ERROR, 'field');
			}
		}
		if (!empty($result))
			switch($comportement) {
				case 'ul_li' :
					if (defined('JDEBUG') && JDEBUG){
						Log::add(Text::_('XML_all_value').' : '.$ul_li_class, Log::INFO, 'ul_li_class');
					}
					return '<ul class="'.$ul_li_class.'"><li>'.implode('</li><li>', $result).'</li></ul>';
				case 'entete_corp_fin':
					if (defined('JDEBUG') && JDEBUG){
						Log::add(Text::_('XML_all_value').' : '.$entete, Log::INFO, 'entete');
						Log::add(Text::_('XML_all_value').' : '.$fin, Log::INFO, 'fin');
					}
					return $entete.implode('', $result).$fin;				
				case 'separateurs' :
				default:
					if (defined('JDEBUG') && JDEBUG){
						Log::add(Text::_('XML_all_value').' : '.$fields_separator, Log::INFO, 'fields_separator');
						Log::add(Text::_('XML_all_value').' : '.$string_assemble, Log::INFO, 'string_assemble');
					}
					return implode($fields_separator, $result);
			}
		else
			return "";
	}
	
	/******************************/
	// fonction XML_Syndication_all_value()
	// récupère les informations d'une syndication
	// ou toutes les valeurs
	//
	// paramètres : chemin xml suivant les synthaxes XQuery
	//		ex : "/feed/entry/content/m:properties/comfax"
	//
	// field : n° du champ dans lequel récupérer la valeur
	//         mettre "" pour tous les champs
	//         Il est possible de récupérer de multiple champ en séparant leur valeur d'une virgule
	//
	// fields_separator : séparateur des champs
	//
	// values_separator : séparateur des valeurs
	//
	// string_assemble : Chaine permettant l'assemblage des champs récupérés
	//
	// exemple de saisie :
	//		$this->XML_1_value("/feed/entry/content/m:properties/d:MODPAIEMENT");
	//      $this->XML_all_value("/feed/entry/content/m:properties/d:TYPEACTIVSPORTPRESTA", "0,1", ", ", ", ", "%0% (%1%)");
	//
	/******************************/
	function XML_Syndication_OuvertureCompletAvecJours($params) {
		$config	= ComponentHelper::getParams('com_tiffps');
		
		if(array_key_exists('path', $params)) 				$path = $params['path'];
		if(array_key_exists('periode_du', $params)) 		$periode_du = $params['periode_du']; else $periode_du = 'P&eacute;riode de ';
		if(array_key_exists('periode_au', $params)) 		$periode_au = $params['periode_au']; else $periode_au = ' au ';
		if(array_key_exists('balise_periode', $params)) 	$balise_periode = $params['balise_periode']; else $balise_periode = '<h3 class="tiffp_periode">%s<h3>';
		if(array_key_exists('ouverture', $params)) 			$ouverture = $params['ouverture']; else $ouverture = '<h4 class="tiffp_ouverture">Ouverture :</h4>';
		if(array_key_exists('heure_de', $params)) 			$heure_de = $params['heure_de']; else $heure_de = ' de ';
		if(array_key_exists('heure_a', $params)) 			$heure_a = $params['heure_a']; else $heure_a = ' &agrave; ';
		if(array_key_exists('heure_et_de', $params)) 		$heure_et_de = $params['heure_et_de']; else $heure_et_de = ' et de ';
		if(array_key_exists('tous_les_jours', $params)) 	$tous_les_jours = $params['tous_les_jours']; else $tous_les_jours = 'Tous les jours';
		if(array_key_exists('fermeture_les', $params)) 		$fermeture_les = $params['fermeture_les']; else $fermeture_les = '<h4 class="tiffp_fermeture">Fermeture :</h4>';
		if(array_key_exists('show_tous_les_jours', $params)) $show_tous_les_jours = $params['show_tous_les_jours']; else $show_tous_les_jours = true;
		
		//lecture et mise au format d'une valeur
		$liste = '';
		if (is_null($liste = $this->xpathcontext->query($path)))
			print_r(Text::_('TIFFP_ERREUR_LECTURE_FIELD').'<br/>');
		$result = '';
		if ($liste) {
			$results = @trim($liste->item((int)$id)->nodeValue);
			$niveau1s = explode($this->tiffps_fields_separator, $results);
			foreach ($niveau1s as $niveau1) {
				$periode = '';
				$heures = '';
				$fermeture = '';
				if (defined('JDEBUG') && JDEBUG){
					Log::add(Text::_('TIFFP_TRACE_NIVEAU2').' : '.$niveau1, Log::INFO, 'field');
				}
				if(!empty($niveau1)){
					$niveau2s = explode($this->tiffps_values_separator, $niveau1);
					if (count($niveau2s)<38) {
						if (defined('JDEBUG') && JDEBUG){
							Log::add(Text::_('TIFFP_TRACE_NIVEAU2_DEFAUT_TAILLE_CHAMP'), Log::INFO, 'field');
						}					
					} else {
						$tmp = '';
						$class = '';
						$heure = '';
						$jours = array();
						$heures = array();
						for($i = 0; $i < count($niveau2s); $i++){
							if ($i == 0 and !empty($niveau2s[$i])) $periode .= $periode_du.$niveau2s[$i];
							if ($i == 1 and !empty($niveau2s[$i])) $periode .= $periode_au.$niveau2s[$i];
							if (in_array($i, array(2,7,12,17,22,27,32)) and !empty($niveau2s[$i])) $jours[] = ucfirst($niveau2s[$i]);
							if (in_array($i, array(3,8,13,18,23,28,33)) and !empty($niveau2s[$i])) $heure .= $heure_de.$niveau2s[$i];
							if (in_array($i, array(4,9,14,19,24,29,34)) and !empty($niveau2s[$i])) $heure .= $heure_a.$niveau2s[$i];
							if (in_array($i, array(5,10,15,20,25,30,35)) and !empty($niveau2s[$i])) {
								if ($heure)
									$heure .= $heure_et_de.$niveau2s[$i];
								else
									$heure .= $heure_de.$niveau2s[$i];
							}
							if (in_array($i, array(6,11,16,21,26,31,36)) and !empty($niveau2s[$i])) $heure .= $heure_a.$niveau2s[$i];
							if (in_array($i, array(7,12,17,22,27,32,37))) {
								$heures[] = $heure;
								$heure = '';
							}
							if ($i == 38 and !empty($niveau2s[$i])) $fermeture .= $fermeture_les.'<ul class="tiffp_jours"><li>'.$niveau2s[$i].'</li></ul>';
						}
						$result .= sprintf($balise_periode, $periode);
						$result .= $ouverture;
						//teste si tous les jour les mêmes horaires
						if($show_tous_les_jours and count($heures) == 7 and count(array_unique($heures)) == 1){
							$result .= '<ul class="tiffp_jours"><li>'.$tous_les_jours.$heures[0].'</li></ul>';
						} else {
							$result .= '<ul class="tiffp_jours">';
							for($j = 0; $j < count($jours); $j++){
								if ($heures[$j]) $result .= '<li>'.$jours[$j].$heures[$j].'</li>';
							}
							$result .= '</ul>';
						}
						$result .= $fermeture;
					}
				}
			}
		} else {
			if (defined('JDEBUG') && JDEBUG){
				Log::add(Text::_('TIFFP_ERREUR_LECTURE_FIELD'), Log::ERROR, 'field');
			}
		}
		if (!empty($result))
			return $result;
		else
			return "";
	}

	/******************************/
	// fonction XML_1_value()
	// récupère une information d'une syndication
	//
	// paramètres : chemin xml suivant les synthaxes XQuery
	//		ex : "/feed/entry/content/m:properties/comfax"
	//
	// field : n° du champ dans lequel récupérer la valeur
	// value : n° de l'élément contenant la valeur
	//
	// exemple de saisie :
	//		$this->XML_1_value("/feed/entry/content/m:properties/d:comtel", 0, 1);
	//		$this->XML_1_value("/feed/entry/content/m:properties/d:comfax", 0, 1);
	//		$this->XML_1_value("/feed/entry/content/m:properties/d:commail", 0, 1);
	//
	/******************************/
	function XML_1_value($path, $field=0, $value=0) {
		$config	= ComponentHelper::getParams('com_tiffps');
		//lecture et mise au format d'une valeur
		if (is_null($liste = $this->xpathcontext->query($path)))
			print_r(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field.'<br/>');
		$result = array();
		//if ($id) die(print_r($liste->length));
		$cn1 = 0;
		if ($liste) {
			$results = @trim($liste->item((int)$id)->nodeValue);
			$niveau1s = explode($this->tiffps_fields_separator, $results);
			foreach ($niveau1s as $niveau1) {
				$niveau2s = explode($this->tiffps_values_separator, $niveau1);
				$cn2 = 0;
				foreach ($niveau2s as $niveau2) {
					$result[$cn1][$cn2] = $niveau2;
					$cn2++;
				}
				$cn1++;
			}
		} else {
			if (defined('JDEBUG') && JDEBUG){
				Log::add(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field, Log::ERROR, 'field');
			}
		}
		if (isset($result[$field][$value]))
			return $result[$field][$value];
		else
			return "";
	}
	
	/******************************/
	// fonction XML_1_value_fc_texte()
	// récupère une information d'une syndication en focntion de la valeur du texte recherché
	//
	// paramètres : chemin xml suivant les synthaxes XQuery
	//		ex : "/feed/entry/content/m:properties/comfax"
	//
	// field : n° du champ dans lequel récupérer la valeur
	// value : n° de l'élément contenant la valeur
	//
	// exemple de saisie :
	//		$this->XML_1_value_fc_texte("/feed/entry/content/m:properties/d:MOYENSCOMMUNICATION", "FAX", 0);
	//		$this->XML_1_value_fc_texte("/feed/entry/content/m:properties/d:MOYENSCOMMUNICATION", "Site web", 1);
	//
	/******************************/
	function XML_1_value_fc_texte($path, $field="", $value=0) {
		$config	= ComponentHelper::getParams('com_tiffps');
		//lecture et mise au format d'une valeur
		if (is_null($liste = $this->xpathcontext->query($path)))
			print_r(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field.'<br/>');
		$result = array();
		//if ($id) die(print_r($liste->length));
		$cn1 = 0;
		$id = 0;
		if ($liste and $liste->item((int)$id)) {
			$results = @trim($liste->item((int)$id)->nodeValue);
			$niveau1s = explode($this->tiffps_fields_separator, $results);
			foreach ($niveau1s as $niveau1) {
				$niveau2s = explode($this->tiffps_values_separator, $niveau1);
				$cn2 = 0;
				foreach ($niveau2s as $niveau2) {
					if (array_key_exists($niveau2s[0], $result)) {
						if ($niveau2 != $niveau2s[0]) $result[$niveau2s[0]][] = $niveau2;
					} elseif ($niveau2 != $niveau2s[0]) {
						$result[$niveau2s[0]][$cn2] = $niveau2;
						$cn2++;
					}
				}
				$cn1++;
			}
		} else {
			if (defined('JDEBUG') && JDEBUG){
				Log::add(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field, Log::ERROR, 'field');
			}
		}
		if (isset($result[$field][$value]))
			return $result[$field][$value];
		else
			return "";
	}
	
	/******************************/
	// fonction XML_array()
	// Récupérer les éléments d’une suite de valeur dans un champ de syndication
	// et remplace leur valeur par la valeur correspondante
	//
	// paramètres : chemin xml suivant les synthaxes XQuery
	//		ex : "/feed/entry/content/m:properties/d:LANGPARLE"
	//
	// valeurs_lues : array de valeurs lues
	// valeurs_retranscrites : array de valeurs retranscrites en correspondance des valeurs lues
	//
	// index en cas de reponse multiple: 0 est la première valeur
	//
	// separator pour définir la valeur séparant les éléments de réponse multiple
	//
	// exemple de saisie :
	//		$this->XML_array(array('path'=>"/feed/entry/content/m:properties/d:LANGPARLE", 'lue'=>array("Français", "Anglais"), 'retranscrites'=>array(19, 20)));
	//
	/******************************/
	function XML_array($params) {
		$path = $params;
		$lues = array();
		$retranscrites = array();
		$niveau=0;
		$separateur = ',';
		$value_vide = '';
		$dedoublonne = false;

		if (is_array($params)){
			if(array_key_exists('path', $params)) 			$path = $params['path'];
			if(array_key_exists('lues', $params)) 			$lues = $params['lues'];
			if(array_key_exists('retranscrites', $params)) 	$retranscrites = $params['retranscrites'];
			if(array_key_exists('niveau', $params)) 		$niveau = $params['niveau'];
			if(array_key_exists('separateur', $params)) 	$separateur = $params['separateur'];
			if(array_key_exists('value_vide', $params)) 	$value_vide = $params['value_vide'];
			if(array_key_exists('dedoublonne', $params)) 	$dedoublonne = $params['dedoublonne'];
		}
	
		$config	= ComponentHelper::getParams('com_tiffps');
		//lecture et mise au format d'une valeur
		if (is_null($liste = $this->xpathcontext->query($path)))
			print_r(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field.'<br/>');
		$result = array();
		//if ($id) die(print_r($liste->length));
		$cn1 = 0;
		if ($liste) {
			$results = @trim($liste->item((int)$id)->nodeValue);
			if (!empty($results)) {
				$niveau1s = explode($this->tiffps_fields_separator, $results);
				foreach ($niveau1s as $niveau1) {
					if (defined('JDEBUG') && JDEBUG){
						Log::add('XML_array : *'.$niveau1.'*', Log::INFO, Text::sprintf('COM_TIFFPS_VAL_NIVEAU', $niveau));
					};
					$niveau2s = explode($this->tiffps_values_separator, $niveau1);
					if (count($niveau2s) > $niveau) {
						$search_value = $niveau2s[$niveau];
						$key = array_search($search_value, $lues);
						if ($key!==false) $result[] = $retranscrites[$key];
					} else
						if (defined('JDEBUG') && JDEBUG){
							Log::add(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field, Log::ERROR, Text::sprintf('COM_TIFFPS_BAD_NIVEAU', $niveau));
						};
				}
			}
		} else {
			if (defined('JDEBUG') && JDEBUG){
				Log::add(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field, Log::ERROR, 'field');
			}
			return $value_vide;
		}
		if (!empty($result)){
			if ($dedoublonne)
				$result = array_unique($result);
			return implode($separateur, $result);
		} else
			return $value_vide;
	}
	
	/******************************/
	// fonction XML_array_custom_field()
	// Récupérer les éléments d’une suite de valeur dans un champ de syndication
	// et remplace leur valeur par la valeur correspondante
	//
	// paramètres : chemin xml suivant les synthaxes XQuery
	//		ex : "/feed/entry/content/m:properties/d:LANGPARLE"
	//
	// valeurs_lues : array de valeurs lues
	// valeurs_retranscrites : array de valeurs retranscrites en correspondance des valeurs lues
	//
	// index en cas de reponse multiple: 0 est la première valeur
	//
	// exemple de saisie :
	//		$this->XML_array_custom_field(array('path'=>"/feed/entry/content/m:properties/d:LANGPARLE", 'lues'=>array("Français", "Anglais"), 'retranscrites'=>array(19, 20)));
	//
	/******************************/
	function XML_array_custom_field($params) {
		$path = $params;
		$lues = array();
		$retranscrites = array();
		$niveau=0;

		if (is_array($params)){
			if(array_key_exists('path', $params)) 			$path = $params['path'];
			if(array_key_exists('lues', $params)) 			$lues = $params['lues'];
			if(array_key_exists('retranscrites', $params)) 	$retranscrites = $params['retranscrites'];
			if(array_key_exists('niveau', $params)) 		$niveau = $params['niveau'];
		}

		$config	= ComponentHelper::getParams('com_tiffps');
		//lecture et mise au format d'une valeur
		if (is_null($liste = $this->xpathcontext->query($path)))
			print_r(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field.'<br/>');
		$result = array();
		//if ($id) die(print_r($liste->length));
		$cn1 = 0;
		if ($liste) {
			$results = @trim($liste->item((int)$id)->nodeValue);
			if (!empty($results)) {
				$niveau1s = explode($this->tiffps_fields_separator, $results);
				foreach ($niveau1s as $niveau1) {
					if (defined('JDEBUG') && JDEBUG){
						Log::add('XML_array : *'.$niveau1.'*', Log::INFO, Text::sprintf('COM_TIFFPS_VAL_NIVEAU', $niveau));
					};
					$niveau2s = explode($this->tiffps_values_separator, $niveau1);
					if (count($niveau2s) > $niveau) {
						$search_value = $niveau2s[$niveau];
						$key = array_search($search_value, $lues);
						if ($key!==false) $result[] = $retranscrites[$key];
					} else
						if (defined('JDEBUG') && JDEBUG){
							Log::add(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field, Log::ERROR, Text::sprintf('COM_TIFFPS_BAD_NIVEAU', $niveau));
						};
				}
			}
		} elseif (defined('JDEBUG') && JDEBUG){
			Log::add(Text::_('TIFFP_ERREUR_LECTURE_FIELD').' : '.$this->field, Log::ERROR, 'field');
		}
		return array_unique($result);
	}

}
