<?php
	/*
	* TifFP composant passerelel de TourInFrance vers GMapFP pour Joomla! 4.x
	* Version J4.0
	* Creation date: Juin 2023
	* Author: Fabrice4821 - www.gmapfp.org
	* Author email: webmaster@gmapfp.org
	* License GNU/GPL
	*/

namespace Joomla\Component\Tiffps\Administrator\View\Tiffps;

defined('_JEXEC') or die;

use Exception;
use Joomla\CMS\Factory;
use Joomla\CMS\Form\Form;
use Joomla\CMS\Helper\ContentHelper;
use Joomla\CMS\Language\Multilanguage;
use Joomla\CMS\Language\Text;
use Joomla\CMS\MVC\View\GenericDataException;
use Joomla\CMS\MVC\View\HtmlView as BaseHtmlView;
use Joomla\CMS\Object\CMSObject;
use Joomla\CMS\Pagination\Pagination;
use Joomla\CMS\Toolbar\Toolbar;
use Joomla\CMS\Toolbar\ToolbarHelper;
use Joomla\CMS\Uri\Uri;

class HtmlView extends BaseHtmlView
{
	protected $items;
	protected $pagination;
	protected $state;
    public $filterForm;
    public $activeFilters;
    private $isEmptyState = false;

	public function display($tpl = null)
	{
		// Initialise variables.
		$this->items		= $this->get('Items');
		$this->pagination	= $this->get('Pagination');
		$this->state		= $this->get('State');
        $this->filterForm    = $this->get('FilterForm');
        $this->activeFilters = $this->get('ActiveFilters');

        if (!\count($this->items) && $this->isEmptyState = $this->get('IsEmptyState')) {
            $this->setLayout('emptystate');
        }

		// Check for errors.
		if (count($errors = $this->get('Errors')))
		{
			throw new Exception(implode("\n", $errors), 500);
		}

		$this->addToolbar();
		parent::display($tpl);
	}

    protected function addToolbar()
    {
        $canDo = ContentHelper::getActions('com_tiffps', 'category', $this->state->get('filter.category_id'));
        $user  = Factory::getApplication()->getIdentity();

        // Get the toolbar object instance
        $toolbar = Toolbar::getInstance('toolbar');

        ToolbarHelper::title(Text::_('COM_TIFFPS_MANAGER_TIFFPS'), ' fa-object-group tiffp');

        ToolBarHelper::link(URI::base().'index.php?option=com_tiffps', Text::_('COM_TIFFPS_ACCUEIL_TIFFPS'), 'home');
		
		if ($canDo->get('core.create') || \count($user->getAuthorisedCategories('com_tiffps', 'core.create')) > 0) {
            $toolbar->addNew('tiffp.add', 'COM_TIFFPS_NEW_GMAPFP', 'disable');
        }

		if ($canDo->get('core.create') || \count($user->getAuthorisedCategories('com_tiffps', 'core.create')) > 0) {
            // $toolbar->addNew('tiffp.add_content', 'COM_TIFFPS_NEW_CONTENT', true);
        }

        if (!$this->isEmptyState && $canDo->get('core.edit.state')) {
            $dropdown = $toolbar->dropdownButton('status-group')
                ->text('JTOOLBAR_CHANGE_STATUS')
                ->toggleSplit(false)
                ->icon('icon-ellipsis-h')
                ->buttonClass('btn btn-action')
                ->listCheck(true);

            $childBar = $dropdown->getChildToolbar();

            $childBar->publish('tiffps.publish')->listCheck(true);

            $childBar->unpublish('tiffps.unpublish')->listCheck(true);

            $childBar->archive('tiffps.archive')->listCheck(true);

            if ($user->authorise('core.admin')) {
                $childBar->checkin('tiffps.checkin')->listCheck(true);
            }

            $childBar->standardButton('copy')
                ->text('JTOOLBAR_SAVE_AS_COPY')
                ->task('tiffp.copy')
                ->listCheck(true);

			if ($this->state->get('filter.published') != -2) {
                $childBar->trash('tiffps.trash')->listCheck(true);
            }
        }

        if (!$this->isEmptyState && $this->state->get('filter.published') == -2 && $canDo->get('core.delete')) {
            $toolbar->delete('tiffps.delete')
                ->text('JTOOLBAR_EMPTY_TRASH')
                ->message('JGLOBAL_CONFIRM_DELETE')
                ->listCheck(true);
        }

		if ($user->authorise('core.admin', 'com_tiffps') || $user->authorise('core.options', 'com_tiffps')) {
			ToolbarHelper::custom('tiffps.extract', 'unarchive', 'unarchive', 'COM_TIFFPS_IMPORT');
        }
		
		if ($user->authorise('core.admin', 'com_tiffps') || $user->authorise('core.options', 'com_tiffps')) {
            $toolbar->preferences('com_tiffps');
        }
		
		$toolbar->help('', false, 'http://www.creation-web.pro/documentation/TifFP_4.pdf');
    }
}
