<?php
	/*
	* Task_TifFPs Plugin pour composant Tiffps pour Joomla! 4.x
	* Version J4_1
	* Creation date: Février 2024
	* Author: Fabrice4821 - www.gmapfp.org
	* Author email: support@gmapfp.org
	* License GNU/GPL
	*/

namespace Joomla\Plugin\Task\TifFp\Extension;

defined('_JEXEC') or die;

use Joomla\Component\Tiffps\Administrator\Model\Syndications;
use Exception;
use InvalidArgumentException;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Factory;
use Joomla\CMS\Filesystem\Path;
use Joomla\CMS\Form\Form;
use Joomla\CMS\MVC\Factory\MVCFactoryServiceInterface;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\Component\Scheduler\Administrator\Event\ExecuteTaskEvent;
use Joomla\Component\Scheduler\Administrator\Task\Status;
use Joomla\Component\Scheduler\Administrator\Task\Task;
use Joomla\Component\Scheduler\Administrator\Traits\TaskPluginTrait;
use Joomla\Database\DatabaseAwareInterface;
use Joomla\Database\DatabaseAwareTrait;
use Joomla\Database\ParameterType;
use Joomla\Event\EventInterface;
use Joomla\Event\SubscriberInterface;
use Joomla\Registry\Registry;
use RuntimeException;
use function array_key_exists;
use function strlen;

class TifFp extends CMSPlugin implements SubscriberInterface, DatabaseAwareInterface
{
	use DatabaseAwareTrait;
	use TaskPluginTrait;

	private const TASKS_MAP = [
		'tiffp.download'    => [
			'langConstPrefix' => 'PLG_TASK_TIFFP_TASK_DOWNLOAD',
			'method'          => 'takeDownload',
		],
		'tiffp.update' => [
			'langConstPrefix' => 'PLG_TASK_TIFFP_TASK_UPDATE',
			'method'          => 'takeUpdate',
		],
	];

	protected $autoloadLanguage = true;

	public static function getSubscribedEvents(): array
	{
		// This task is disabled if the Akeeba Backup component is not installed or has been unpublished
		if (!ComponentHelper::isEnabled('com_tiffps') or !ComponentHelper::isEnabled('com_gmapfp'))
		{
			return [];
		}

		return [
			'onTaskOptionsList'    => 'advertiseRoutines',
			'onExecuteTask'        => 'standardRoutineHandler',
		];
	}

	//importation des données du flux
	private function takeDownload(ExecuteTaskEvent $event): int
	{
        // Get some basic information about the task at hand.
		/** @var Task $task */
		$task    = $event->getArgument('subject');

		$component = ComponentHelper::isEnabled('com_tiffps')
			? $this->getApplication()->bootComponent('com_tiffps')
			: null;

		if (!($component instanceof MVCFactoryServiceInterface))
		{
			throw new RuntimeException('The TifFPs component is not installed or has been disabled.');
		}

		$factory = $component->getMVCFactory();
		$model = $factory->createModel('Syndicationbase', 'Administrator', ['ignore_request' => true]);

		$this->logTask('Starting a new import.');
		
		// Initialisation de la boucle update
		$updateResult = [
			'HasRun'       		=> 0,
			'Error'        		=> '',
			'Update_firstrun' 	=> '1'
		];

		while (($updateResult['HasRun'] != 1) && (empty($updateResult['Error'])))
		{
			$this->logTask('Starting a new import');
			$updateResult = $model->step_cron_import_flux($updateResult);

			// Print the new progress bar and info
			$this->logTask('ids: ' . json_encode($updateResult['ids']), 'debug');
			$this->logTask('courent_id: ' . $updateResult['courent_id'], 'debug');
			$this->logTask('HasRun: ' . $updateResult['HasRun'], 'debug');
			$this->logTask('Error: ' . $updateResult['Error'], 'debug');
			
		}

		// Did the backup end in an error?
		if (isset($updateResult['Error']) && !empty($updateResult['Error']))
		{
			throw new RuntimeException($updateResult['Error']);
		}

		$this->logTask('Backup finished successfully');

		return Status::OK;
	}

	//extraction des données vers GMapFP
	private function takeUpdate(ExecuteTaskEvent $event): int
	{
        // Get some basic information about the task at hand.
		/** @var Task $task */
		$task    = $event->getArgument('subject');

		$component = ComponentHelper::isEnabled('com_tiffps')
			? $this->getApplication()->bootComponent('com_tiffps')
			: null;

		if (!($component instanceof MVCFactoryServiceInterface))
		{
			throw new RuntimeException('The TifFPs component is not installed or has been disabled.');
		}

		$factory = $component->getMVCFactory();
		$model = $factory->createModel('Tiffpbase', 'Administrator', ['ignore_request' => true]);

		$this->logTask('Starting a new file extract.');
		
		// Initialisation de la boucle update
		$updateResult = [
			'HasRun'       		=> 0,
			'Error'        		=> '',
			'Update_firstrun' 	=> '1'
		];

		while (($updateResult['HasRun'] != 1) && (empty($updateResult['Error'])))
		{
			$this->logTask('Starting a new extract');
			$updateResult = $model->step_cron_extract_file($updateResult);

			// Print the new progress bar and info
			$this->logTask('ids: ' . json_encode($updateResult['ids']), 'debug');
			$this->logTask('courent_file: ' . $updateResult['courent_file'], 'debug');
			$this->logTask('courent_id: ' . $updateResult['courent_id'], 'debug');
			$this->logTask('nbre_files: ' . $updateResult['nbre_files'], 'debug');
			$this->logTask('HasRun: ' . $updateResult['HasRun'], 'debug');
			$this->logTask('Error: ' . $updateResult['Error'], 'debug');
			
		}

		// Did the backup end in an error?
		if (isset($updateResult['Error']) && !empty($updateResult['Error']))
		{
			throw new RuntimeException($updateResult['Error']);
		}

		$this->logTask('Finish file extraction successfully');

		$this->logTask('Starting a new image rapatriement.');
		
		// Initialisation de la boucle update
		$updateResult = [
			'HasRun'       		=> 0
		];

		while ($updateResult['HasRun'] != 1)
		{
			$this->logTask('Starting a new extract');
			$updateResult = $model->step_cron_extract_image($updateResult);

			// Print the new progress bar and info
			$this->logTask('HasRun: ' . $updateResult['HasRun'], 'debug');
			$this->logTask('Error: ' . $updateResult['Error'], 'debug');
			
		}

		// Did the backup end in an error?
		if (isset($updateResult['Error']) && !empty($updateResult['Error']))
		{
			throw new RuntimeException($updateResult['Error']);
		}

		$this->logTask('Finish images extraction successfully');

		return Status::OK;
	}

}