<?php
	/*
	* TifFPsGMapFPInfo Plugin pour composant Tiffps et GMapfp pour Joomla! 4.x
	* Version J4_1
	* Creation date: Mars 2024
	* Author: Fabrice4821 - www.gmapfp.org
	* Author email: support@gmapfp.org
	* License GNU/GPL
	*/

use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Factory;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\Component\Tiffps\Administrator\Helper\TiffpsHelper;
use Joomla\Database\ParameterType;
use Joomla\CMS\Uri\Uri;

\defined('_JEXEC') or die;

class plgSystemTiffpsgmapfpinfo extends CMSPlugin
{
    protected $app;
    protected $name;
    protected $db;

    public function onAfterRoute()
    {
        if ($this->app->isClient('administrator')) {

			$input = $this->app->input;
            $option = $input->get('option', null, 'cmd');
            $view = $input->get('view', 'default', 'cmd');

            $params = ComponentHelper::getParams('com_tiffps');


            if (isset($view) and $option=='com_gmapfp' and $view=='items' and $params->get('show_list', true)) {
                // Intercept the grid.id HTML Field to insert translation status
                $this->name = TiffpsHelper::getTif_id();
				HTMLHelper::register('Grid.id', array($this, 'gridIdHook'));
            }
        }
    }

    public function onBeforeRender()
	{

		if ($this->app->isClient('administrator')) {
			$input = $this->app->input;
			$option = $input->get('option', null, 'cmd');
			$view = $input->get('view', 'default', 'cmd');

			$params = ComponentHelper::getParams('com_tiffps');

			if (isset($view) and $option=='com_gmapfp' and $view=='items' and $params->get('show_list', true))
			{
				HTMLHelper::_('jquery.framework');
				Factory::getDocument()->addScript(URI::root().'plugins/system/tiffpsgmapfpinfo/assets/tiffpsgmapfpinfo.js', array('version' => 'auto', 'relative' => false));
				$this->loadLanguage();
				Text::script('PLG_SYSTEM_TIFFPSGMAPFPINFO_INFO');
				Text::script('JSTATUS');
				Text::script('COM_GMAPFP_NOM');
			}
		}
    }

    public function onContentBeforeDelete($context, $data)
    {
		if ($context == 'com_gmapfp.item') {
			$db     = $this->db;
			$gmapfpId = (int) $data->id;

			$query = $db->getQuery(true)
				->delete($db->quoteName('#__tiffp_gmapfp'))
				->where($db->quoteName('gmapfp_id') . ' = :gmapfpId')
				->bind(':gmapfpId', $gmapfpId, ParameterType::INTEGER);

			try
			{
				$db->setQuery($query)->execute();
			}
			catch (ExecutionFailureException $e)
			{
				// Do nothing.
			}
		}
	}

    public function gridIdHook() 
	{
        $row = func_get_arg(0);
        $id = func_get_arg(1);
        $vars = func_get_args();
        $res = call_user_func_array('Joomla\CMS\HTML\Helpers\Grid::id', $vars);
        //get table by component
        $result = array();

		if (array_key_exists($id, $this->name)) {
			$result['status'] = '1';
			$result['link'] = $this->name[$id]->tif_id;
		} else {
			$result['status'] = '0';
			$result['link'] = 0;
		}
            
        // create array
        $res .= '<script>tiffps['.$row.']='.json_encode($result).';</script>';
        return $res;

    }

}
